@@ -18,7 +18,9 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import unicode_literals
 
+import argparse
 import sys
+import textwrap
 
 from googlecloudsdk.api_lib.util import apis
 
@@ -30,11 +32,12 @@ from googlecloudsdk.calliope.concepts import deps
 from googlecloudsdk.command_lib.ai import constants
 from googlecloudsdk.command_lib.ai import errors
 from googlecloudsdk.command_lib.ai import region_util
+from googlecloudsdk.command_lib.iam import iam_util as core_iam_util
+from googlecloudsdk.command_lib.kms import resource_args as kms_resource_args
 from googlecloudsdk.command_lib.util.apis import arg_utils
 from googlecloudsdk.command_lib.util.concepts import concept_parsers
 from googlecloudsdk.core import properties
 
-
 CUSTOM_JOB_NAME = base.Argument('name', help=('Custom job\'s name to query.'))
 CUSTOM_JOB_DISPLAY_NAME = base.Argument(
     '--display-name',
@@ -71,7 +74,11 @@ Example(YAML):
       machineType: n1-highmem-2
     replicaCount: 1
     containerSpec:
-      imageUri: gcr.io/ucaip-test/ucaip-training-test""")
+      imageUri: gcr.io/ucaip-test/ucaip-training-test
+      args:
+      - port=8500
+      command:
+      - start""")
 
 WORKER_POOL_SPEC = base.Argument(
     '--worker-pool-spec',
@@ -172,12 +179,53 @@ _TASK_NAME = base.Argument(
     default=None,
     help='If set, display only the logs for this particular task.')
 
+_CUSTOM_JOB_COMMAND = base.Argument(
+    '--command',
+    type=arg_parsers.ArgList(),
+    metavar='COMMAND',
+    action=arg_parsers.UpdateAction,
+    help="""\
+Command to be invoked when containers are started.
+It overrides the entrypoint instruction in Dockerfile when provided.
+""")
+_CUSTOM_JOB_ARGS = base.Argument(
+    '--args',
+    metavar='ARG',
+    type=arg_parsers.ArgList(),
+    action=arg_parsers.UpdateAction,
+    help="""\
+Comma-separated arguments passed to containers or python tasks.
+""")
+
+_NETWORK = base.Argument(
+    '--network',
+    help=textwrap.dedent("""\
+      Full name of the Google Compute Engine network to which the Job
+      is peered with. Private services access must already have been configured.
+      If unspecified, the Job is not peered with any network.
+      """))
+
+_TRAINING_SERVICE_ACCOUNT = base.Argument(
+    '--service-account',
+    type=core_iam_util.GetIamAccountFormatValidator(),
+    required=False,
+    help=textwrap.dedent("""\
+      The email address of a service account to use when running the
+      training appplication. You must have the `iam.serviceAccounts.actAs`
+      permission for the specified service account.
+      """))
+
 
 def AddCreateCustomJobFlags(parser):
   """Adds flags related to create a custom job."""
   AddRegionResourceArg(parser, 'to create a custom job')
   CUSTOM_JOB_DISPLAY_NAME.AddToParser(parser)
   PYTHON_PACKGE_URIS.AddToParser(parser)
+  _CUSTOM_JOB_ARGS.AddToParser(parser)
+  _CUSTOM_JOB_COMMAND.AddToParser(parser)
+  _TRAINING_SERVICE_ACCOUNT.AddToParser(parser)
+  _NETWORK.AddToParser(parser)
+  AddKmsKeyResourceArg(parser, 'custom job')
   worker_pool_spec_group = base.ArgumentGroup(
       help='Worker pool specification.', required=True)
   worker_pool_spec_group.AddArgument(CUSTOM_JOB_CONFIG)
@@ -193,9 +241,7 @@ def AddStreamLogsFlags(parser):
 
 def GetModelIdArg(required=True):
   return base.Argument(
-      '--model',
-      help='Id of the uploaded model.',
-      required=required)
+      '--model', help='Id of the uploaded model.', required=required)
 
 
 def GetDeployedModelId(required=True):
@@ -205,6 +251,17 @@ def GetDeployedModelId(required=True):
       required=required)
 
 
+def GetIndexIdArg(required=True, helper_text='ID of the index.'):
+  return base.Argument('--index', help=helper_text, required=required)
+
+
+def GetDeployedIndexId(required=True):
+  return base.Argument(
+      '--deployed-index-id',
+      help='Id of the deployed index.',
+      required=required)
+
+
 def GetDisplayNameArg(noun, required=True):
   return base.Argument(
       '--display-name',
@@ -274,26 +331,48 @@ def AddPredictionResourcesArgs(parser, version):
   """Add arguments for prediction resources."""
   base.Argument(
       '--min-replica-count',
-      required=True,
       type=arg_parsers.BoundedInt(1, sys.maxsize, unlimited=True),
-      help=('Minimum number of machine replicas the deployed model will be '
-            'always deployed on.')
-  ).AddToParser(parser)
+      help=("""\
+Minimum number of machine replicas the deployed model will be always deployed
+on. If specified, the value must be equal to or larger than 1.
+
+If not specified and the uploaded models use dedicated resources, the default
+value is 1.
+""")).AddToParser(parser)
 
   base.Argument(
       '--max-replica-count',
       type=int,
       help=('Maximum number of machine replicas the deployed model will be '
-            'always deployed on.')
-  ).AddToParser(parser)
+            'always deployed on.')).AddToParser(parser)
+
+  base.Argument(
+      '--autoscaling-metric-specs',
+      metavar='METRIC-NAME=TARGET',
+      type=arg_parsers.ArgDict(key_type=str, value_type=int),
+      action=arg_parsers.UpdateAction,
+      help="""\
+Metric specifications that overrides a resource utilization metric's target
+value. At most one entry is allowed per metric.
+
+*METRIC-NAME*::: Resource metric name. Choices are {}.
+
+*TARGET*::: Target resource utilization in percentage (1% - 100%) for the
+given metric. If the value is set to 60, the target resource utilization is 60%.
+
+For example:
+`--autoscaling-metric-specs=cpu-usage=70`
+""".format(', '.join([
+    "'{}'".format(c)
+    for c in sorted(constants.OP_AUTOSCALING_METRIC_NAME_MAPPER.keys())]
+                     ))).AddToParser(parser)
 
-  # TODO(b/168930155): add the link for available machine types after the docs
-  #   are out.
   base.Argument(
       '--machine-type',
       help="""\
-Type of machine on which to serve the model. Currently only applies to online prediction.
-If the uploaded models use dedicated resources, the machine type is a required field for deployment.
+The machine resources to be used for each node of this deployment.
+For available machine types, see
+https://cloud.google.com/ai-platform-unified/docs/predictions/machine-types.
 """).AddToParser(parser)
 
   base.Argument(
@@ -319,6 +398,63 @@ For example:
   ]))).AddToParser(parser)
 
 
+def AddAutomaticResourcesArgs(parser, resource_type):
+  """Add arguments for automatic deployment resources."""
+  base.Argument(
+      '--min-replica-count',
+      type=arg_parsers.BoundedInt(1, sys.maxsize, unlimited=True),
+      help=("""\
+Minimum number of machine replicas the {} will be always deployed
+on. If specified, the value must be equal to or larger than 1.
+""".format(resource_type))).AddToParser(parser)
+
+  base.Argument(
+      '--max-replica-count',
+      type=int,
+      help=('Maximum number of machine replicas the {} will be '
+            'always deployed on.'.format(resource_type))).AddToParser(parser)
+
+
+def GetEnableAccessLoggingArg():
+  return base.Argument(
+      '--enable-access-logging',
+      action='store_true',
+      default=False,
+      required=False,
+      help="""\
+If true, online prediction access logs are sent to Cloud Logging.
+
+These logs are standard server access logs, containing information like
+timestamp and latency for each prediction request.
+""")
+
+
+def GetEnableContainerLoggingArg():
+  return base.Argument(
+      '--enable-container-logging',
+      action='store_true',
+      default=False,
+      required=False,
+      help="""\
+If true, the container of the deployed model instances will send `stderr` and
+`stdout` streams to Cloud Logging.
+
+Currently, only supported for custom-trained Models and AutoML Tables Models.
+""")
+
+
+def GetServiceAccountArg():
+  return base.Argument(
+      '--service-account',
+      required=False,
+      help="""\
+Service account that the deployed model's container runs as. Specify the
+email address of the service account. If this service account is not
+specified, the container runs as a service account that doesn't have access
+to the resource project.
+""")
+
+
 def RegionAttributeConfig():
   return concepts.ResourceParameterAttributeConfig(
       name='region',
@@ -362,6 +498,25 @@ def AddRegionResourceArg(parser, verb):
       required=True).AddToParser(parser)
 
 
+def GetDefaultOperationResourceSpec():
+  return concepts.ResourceSpec(
+      constants.DEFAULT_OPERATION_COLLECTION,
+      resource_name='operation',
+      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
+      locationsId=RegionAttributeConfig(),
+      disable_auto_completers=False)
+
+
+def AddOperationResourceArg(parser):
+  """Add a resource argument for a cloud AI Platform operation."""
+  resource_name = 'operation'
+  concept_parsers.ConceptParser.ForResource(
+      resource_name,
+      GetDefaultOperationResourceSpec(),
+      'The ID of the operation.',
+      required=True).AddToParser(parser)
+
+
 def AddModelResourceArg(parser, verb):
   """Add a resource argument for a cloud AI Platform model.
 
@@ -476,6 +631,45 @@ inclusive.
   )
 
 
+def GetMetadataFilePathArg(noun, required=False):
+  return base.Argument(
+      '--metadata-file',
+      required=required,
+      help='Path to a local JSON file that contains the additional metadata information about the {noun}.'
+      .format(noun=noun))
+
+
+def GetMetadataSchemaUriArg(noun):
+  return base.Argument(
+      '--metadata-schema-uri',
+      required=False,
+      help='Points to a YAML file stored on Google Cloud Storage describing additional information about {noun}.'
+      .format(noun=noun))
+
+
+def AddIndexResourceArg(parser, verb):
+  """Add a resource argument for a cloud AI Platform index.
+
+  NOTE: Must be used only if it's the only resource arg in the command.
+
+  Args:
+    parser: the parser for the command.
+    verb: str, the verb to describe the resource, such as 'to update'.
+  """
+  concept_parsers.ConceptParser.ForResource(
+      'index', GetIndexResourceSpec(), 'Index {}.'.format(verb),
+      required=True).AddToParser(parser)
+
+
+def GetIndexResourceSpec(resource_name='index'):
+  return concepts.ResourceSpec(
+      constants.INDEXES_COLLECTION,
+      resource_name=resource_name,
+      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
+      locationsId=RegionAttributeConfig(),
+      disable_auto_completers=False)
+
+
 def GetEndpointId():
   return base.Argument('name', help='The endpoint\'s id.')
 
@@ -505,6 +699,67 @@ def AddEndpointResourceArg(parser, verb):
       required=True).AddToParser(parser)
 
 
+def AddIndexEndpointResourceArg(parser, verb):
+  """Add a resource argument for a cloud AI Platform index endpoint.
+
+  NOTE: Must be used only if it's the only resource arg in the command.
+
+  Args:
+    parser: the parser for the command.
+    verb: str, the verb to describe the resource, such as 'to update'.
+  """
+  concept_parsers.ConceptParser.ForResource(
+      'index_endpoint',
+      GetIndexEndpointResourceSpec(),
+      'The index endpoint {}.'.format(verb),
+      required=True).AddToParser(parser)
+
+
+def GetIndexEndpointResourceSpec(resource_name='index_endpoint'):
+  return concepts.ResourceSpec(
+      constants.INDEX_ENDPOINTS_COLLECTION,
+      resource_name=resource_name,
+      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
+      locationsId=RegionAttributeConfig(),
+      disable_auto_completers=False)
+
+
+# TODO(b/357812579): Consider switch to use resource arg.
+def GetNetworkArg(required=True):
+  """Add arguments for VPC network."""
+  return base.Argument(
+      '--network',
+      required=required,
+      help="""
+      The Google Compute Engine network name to which the IndexEndpoint should be peered.
+      """)
+
+
+def GetTensorboardResourceSpec(resource_name='tensorboard'):
+  return concepts.ResourceSpec(
+      constants.TENSORBOARDS_COLLECTION,
+      resource_name=resource_name,
+      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
+      locationsId=RegionAttributeConfig(),
+      disable_auto_completers=False)
+
+
+def AddTensorboardResourceArg(parser, verb):
+  """Add a resource argument for a Cloud AI Platform Tensorboard.
+
+  NOTE: Must be used only if it's the only resource arg in the command.
+
+  Args:
+    parser: the parser for the command.
+    verb: str, the verb to describe the resource, such as 'to update'.
+  """
+  concept_parsers.ConceptParser.ForResource(
+      'tensorboard',
+      GetTensorboardResourceSpec(),
+      'The tensorboard {}.'.format(verb),
+      required=True).AddToParser(parser)
+
+
 def ParseAcceleratorFlag(accelerator, version):
   """Validates and returns a accelerator config message object."""
   if accelerator is None:
@@ -522,21 +777,18 @@ The count of the accelerator must be greater than 0.
 """)
   if version == constants.ALPHA_VERSION:
     accelerator_msg = (
-        apis.GetMessagesModule(
-            constants.AI_PLATFORM_API_NAME,
-            constants.AI_PLATFORM_API_VERSION[version])
+        apis.GetMessagesModule(constants.AI_PLATFORM_API_NAME,
+                               constants.AI_PLATFORM_API_VERSION[version])
         .GoogleCloudAiplatformV1alpha1MachineSpec)
   else:
     accelerator_msg = (
-        apis.GetMessagesModule(
-            constants.AI_PLATFORM_API_NAME,
-            constants.AI_PLATFORM_API_VERSION[version])
+        apis.GetMessagesModule(constants.AI_PLATFORM_API_NAME,
+                               constants.AI_PLATFORM_API_VERSION[version])
         .GoogleCloudAiplatformV1beta1MachineSpec)
   accelerator_type = arg_utils.ChoiceToEnum(
       raw_type, accelerator_msg.AcceleratorTypeValueValuesEnum)
   return accelerator_msg(
-      acceleratorCount=accelerator_count,
-      acceleratorType=accelerator_type)
+      acceleratorCount=accelerator_count, acceleratorType=accelerator_type)
 
 
 def GetAcceleratorTypeMapper(version):
@@ -561,11 +813,15 @@ def GetAcceleratorTypeMapper(version):
 
 
 def AddCreateHpTuningJobFlags(parser, algorithm_enum):
+  """Add arguments for creating hp tuning job."""
   AddRegionResourceArg(parser, 'to upload model')
   HPTUNING_JOB_DISPLAY_NAME.AddToParser(parser)
   HPTUNING_JOB_CONFIG.AddToParser(parser)
   HPTUNING_MAX_TRIAL_COUNT.AddToParser(parser)
   HPTUNING_PARALLEL_TRIAL_COUNT.AddToParser(parser)
+  _TRAINING_SERVICE_ACCOUNT.AddToParser(parser)
+  _NETWORK.AddToParser(parser)
+  AddKmsKeyResourceArg(parser, 'hyperparameter tuning job')
 
   arg_utils.ChoiceEnumMapper(
       '--algorithm',
@@ -622,3 +878,142 @@ def AddHptuningJobResourceArg(parser, verb):
       GetHptuningJobResourceSpec(),
       'The hyperparameter tuning job {}.'.format(verb),
       required=True).AddToParser(parser)
+
+
+def AddKmsKeyResourceArg(parser, resource):
+  """Add the --kms-key resource arg to the given parser."""
+  permission_info = ("The 'AI Platform Service Agent' service account must hold"
+                     " permission 'Cloud KMS CryptoKey Encrypter/Decrypter'")
+  kms_resource_args.AddKmsKeyResourceArg(
+      parser, resource, permission_info=permission_info)
+
+
+def AddLocalRunCustomJobFlags(parser):
+  """Add local-run related flags to the parser."""
+
+  # Flags for entry point of the training application
+  application_group = parser.add_mutually_exclusive_group(required=True)
+  application_group.add_argument(
+      '--python-module',
+      metavar='PYTHON_MODULE',
+      help=textwrap.dedent("""
+      Name of the python module to execute, in 'trainer.train' or 'train'
+      format. Its path should be relative to the `work_dir`.
+      """))
+  application_group.add_argument(
+      '--script',
+      metavar='SCRIPT',
+      help=textwrap.dedent("""
+      The relative path of the file to execute. Accepets a Python file,
+      IPYNB file, or arbitrary bash script. This path should be relative to the
+      `work_dir`.
+      """))
+
+  # Flags for working directory.
+  parser.add_argument(
+      '--work-dir',
+      metavar='WORK_DIR',
+      help=textwrap.dedent("""
+      Path of the working directory where the python-module or script exists.
+      If not specified, it use the directory where you run the this command.
+
+      Only the contents of this directory will be accessible to the built
+      container image.
+      """))
+
+  # Flags for extra directory
+  parser.add_argument(
+      '--extra-dirs',
+      metavar='EXTRA_DIR',
+      type=arg_parsers.ArgList(),
+      help=textwrap.dedent("""
+      Extra directories under the working directory to include, besides the one
+      that contains the main executable.
+
+      By default, only the parent directory of the main script or python module
+      is copied to the container.
+      For example, if the module is "training.task" or the script is
+      "training/task.py", the whole "training" directory, including its
+      sub-directories, will always be copied to the container. You may specify
+      this flag to also copy other directories if necessary.
+
+      Note: if no parent is specified in 'python_module' or 'scirpt', the whole
+      working directory is copied, then you don't need to specify this flag.
+      """))
+
+  # Flags for base container image
+  parser.add_argument(
+      '--base-image',
+      metavar='BASE_IMAGE',
+      required=True,
+      help=textwrap.dedent("""
+      URI or ID of the container image in either the Container Registry or local
+      that will run the application.
+      See https://cloud.google.com/ai-platform-unified/docs/training/pre-built-containers
+      for available pre-built container images provided by AI Platform for training.
+      """))
+
+  # Flags for extra requirements.
+  parser.add_argument(
+      '--requirements',
+      metavar='REQUIREMENTS',
+      type=arg_parsers.ArgList(),
+      help=textwrap.dedent("""
+      Python dependencies from PyPI to be used when running the application.
+      If this is not specified, and there is no "setup.py" or "requirements.txt"
+      in the working directory, your application will only have access to what
+      exists in the base image with on other dependencies.
+
+      Example:
+      'tensorflow-cpu, pandas==1.2.0, matplotlib>=3.0.2'
+      """))
+
+  # Flags for extra dependency .
+  parser.add_argument(
+      '--extra-packages',
+      metavar='PACKAGE',
+      type=arg_parsers.ArgList(),
+      help=textwrap.dedent("""
+      Local paths to Python archives used as training dependencies in the image
+      container.
+      These can be absolute or relative paths. However, they have to be under
+      the work_dir; Otherwise, this tool will not be able to acces it.
+
+      Example:
+      'dep1.tar.gz, ./downloads/dep2.whl'
+      """))
+
+  # Flags for the output image
+  parser.add_argument(
+      '--output-image-uri',
+      metavar='OUTPUT_IMAGE',
+      help=textwrap.dedent("""
+      Uri of the custom container image to be built with the your application
+      packed in.
+      """))
+
+  # Flaga for GPU support
+  parser.add_argument(
+      '--gpu', action='store_true', default=False, help='Enable to use GPU.')
+
+  # Flags for docker run
+  parser.add_argument(
+      '--docker-run-options',
+      metavar='DOCKER_RUN_OPTIONS',
+      type=arg_parsers.ArgList(),
+      help=textwrap.dedent("""
+      Custom Docker run options to pass to image during execution.
+      For example, '--no-healthcheck, -a stdin'.
+
+      See https://docs.docker.com/engine/reference/commandline/run/#options for
+      more details.
+      """))
+
+  # User custom flags.
+  parser.add_argument(
+      'args',
+      nargs=argparse.REMAINDER,
+      default=[],
+      help="""Additional user arguments to be forwarded to your application.""",
+      example=('$ {command} --script=my_run.sh --base-image=gcr.io/my/image '
+               '-- --my-arg bar --enable_foo'))
