@@ -36,6 +36,7 @@ from googlecloudsdk.core import exceptions
 from googlecloudsdk.core import execution_utils
 from googlecloudsdk.core import http_proxy
 from googlecloudsdk.core import log
+from googlecloudsdk.core.credentials import creds
 from googlecloudsdk.core.credentials import store
 from googlecloudsdk.core.util import files
 from googlecloudsdk.core.util import platforms
@@ -75,8 +76,7 @@ def AddSshTunnelArgs(parser, tunnel_through_iap_scope):
       forwarding.
 
       To learn more, see the
-      [IAP for TCP forwarding documentation](
-      https://cloud.google.com/iap/docs/tcp-forwarding-overview).
+      [IAP for TCP forwarding documentation](https://cloud.google.com/iap/docs/tcp-forwarding-overview).
       """)
 
 
@@ -199,11 +199,18 @@ def _CloseLocalConnectionCallback(local_conn):
       pass
 
 
-def _GetAccessTokenCallback(creds):
-  if not creds:
+def _GetAccessTokenCallback(credentials):
+  """Callback function to refresh credentials and return access token."""
+  if not credentials:
     return None
-  store.Refresh(creds)
-  return creds.access_token
+  log.debug('credentials type for _GetAccessTokenCallback is [%s].',
+            six.text_type(credentials))
+  store.Refresh(credentials)
+
+  if creds.IsGoogleAuthCredentials(credentials):
+    return credentials.token
+  else:
+    return credentials.access_token
 
 
 def _SendLocalDataCallback(local_conn, data):
@@ -449,7 +456,8 @@ class _BaseIapTunnelHelper(object):
     try:
       websocket_conn = self._InitiateWebSocketConnection(
           conn,
-          functools.partial(_GetAccessTokenCallback, store.LoadIfEnabled()))
+          functools.partial(_GetAccessTokenCallback,
+                            store.LoadIfEnabled(use_google_auth=True)))
       while not self._shutdown:
         data = conn.recv(utils.SUBPROTOCOL_MAX_DATA_FRAME_SIZE)
         if not data:
@@ -514,7 +522,9 @@ class IapTunnelProxyServerHelper(_BaseIapTunnelHelper):
   def _TestConnection(self):
     log.status.Print('Testing if tunnel connection works.')
     websocket_conn = self._InitiateWebSocketConnection(
-        None, functools.partial(_GetAccessTokenCallback, store.LoadIfEnabled()))
+        None,
+        functools.partial(_GetAccessTokenCallback,
+                          store.LoadIfEnabled(use_google_auth=True)))
     websocket_conn.Close()
 
   def _AcceptNewConnection(self):
@@ -549,7 +559,7 @@ class IapTunnelProxyServerHelper(_BaseIapTunnelHelper):
     if self._connections:
       close_count = 0
       for client_thread, conn in self._connections:
-        if client_thread.isAlive():
+        if client_thread.is_alive():
           close_count += 1
           try:
             conn.close()
