@@ -18,11 +18,10 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import unicode_literals
 
-import textwrap
 import enum
+import textwrap
 from googlecloudsdk.api_lib.compute import managed_instance_groups_utils
 from googlecloudsdk.api_lib.compute import utils
-from googlecloudsdk.calliope import actions
 from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import exceptions
@@ -490,7 +489,6 @@ def AddMigStatefulFlagsForUpdateInstanceConfigs(parser):
 
   # Add stateful disk update args
   stateful_disk_argument_name = '--stateful-disk'
-  stateful_disk_argument_name_old = '--update-stateful-disk'
   disk_help_text = textwrap.dedent(
       (STATEFUL_DISKS_HELP_INSTANCE_CONFIGS_UPDATE +
        STATEFUL_DISK_DEVICE_NAME_ARG_HELP + STATEFUL_DISK_SOURCE_ARG_HELP +
@@ -512,29 +510,6 @@ def AddMigStatefulFlagsForUpdateInstanceConfigs(parser):
       action='append',
       help=disk_help_text,
   )
-  # DEPRECATED: --update-stateful-disk
-  parser.add_argument(
-      stateful_disk_argument_name_old,
-      type=arg_parsers.ArgDict(
-          spec={
-              'device-name':
-                  str,
-              'source':
-                  str,
-              'mode':
-                  str,
-              'auto-delete':
-                  AutoDeleteFlag.ValidatorWithFlagName(
-                      stateful_disk_argument_name_old)
-          }),
-      action=actions.DeprecationAction(
-          stateful_disk_argument_name_old,
-          warn='The {flag_name} option is deprecated; '
-          'use --stateful-disk instead.',
-          removed=False,
-          action='append'),
-      dest='stateful_disk',
-      help=disk_help_text)
   # Add remove disk args
   parser.add_argument(
       '--remove-stateful-disks',
@@ -545,7 +520,6 @@ def AddMigStatefulFlagsForUpdateInstanceConfigs(parser):
 
   # Add stateful metadata args
   stateful_metadata_argument_name = '--stateful-metadata'
-  stateful_metadata_argument_name_old = '--update-stateful-metadata'
   metadata_help_text = textwrap.dedent(
       (STATEFUL_METADATA_HELP + STATEFUL_METADATA_HELP_UPDATE).format(
           argument_name=stateful_metadata_argument_name))
@@ -556,20 +530,6 @@ def AddMigStatefulFlagsForUpdateInstanceConfigs(parser):
       action=arg_parsers.StoreOnceAction,
       metavar='KEY=VALUE',
       help=textwrap.dedent(metadata_help_text))
-  # DEPRECATED: --update-stateful-metadata
-  parser.add_argument(
-      stateful_metadata_argument_name_old,
-      type=arg_parsers.ArgDict(min_length=1),
-      default={},
-      action=actions.DeprecationAction(
-          stateful_metadata_argument_name_old,
-          warn='The {flag_name} option is deprecated; '
-          'use --stateful-metadata instead.',
-          removed=False,
-          action=arg_parsers.StoreOnceAction),
-      metavar='KEY=VALUE',
-      dest='stateful_metadata',
-      help=textwrap.dedent(metadata_help_text))
   parser.add_argument(
       '--remove-stateful-metadata',
       metavar='KEY',
@@ -622,39 +582,38 @@ def AddMigStatefulFlagsForInstanceConfigs(parser):
       help=metadata_help_text)
 
 
-def AddCreateInstancesFlags(parser, add_stateful_args=True):
+def AddCreateInstancesFlags(parser):
   """Adding stateful flags for creating and updating instance configs."""
   parser.add_argument(
       '--instance',
       required=True,
       help="""Name of the new instance to create.""")
-  if add_stateful_args:
-    parser.add_argument(
-        '--stateful-disk',
-        type=arg_parsers.ArgDict(
-            spec={
-                'device-name':
-                    str,
-                'source':
-                    str,
-                'mode':
-                    str,
-                'auto-delete':
-                    AutoDeleteFlag.ValidatorWithFlagName('--stateful-disk'),
-            }),
-        action='append',
-        help=textwrap.dedent(STATEFUL_DISKS_HELP_INSTANCE_CONFIGS),
-    )
-    stateful_metadata_argument_name = '--stateful-metadata'
-    parser.add_argument(
-        stateful_metadata_argument_name,
-        type=arg_parsers.ArgDict(min_length=1),
-        default={},
-        action=arg_parsers.StoreOnceAction,
-        metavar='KEY=VALUE',
-        help=textwrap.dedent(
-            STATEFUL_METADATA_HELP.format(
-                argument_name=stateful_metadata_argument_name)))
+  parser.add_argument(
+      '--stateful-disk',
+      type=arg_parsers.ArgDict(
+          spec={
+              'device-name':
+                  str,
+              'source':
+                  str,
+              'mode':
+                  str,
+              'auto-delete':
+                  AutoDeleteFlag.ValidatorWithFlagName('--stateful-disk'),
+          }),
+      action='append',
+      help=textwrap.dedent(STATEFUL_DISKS_HELP_INSTANCE_CONFIGS),
+  )
+  stateful_metadata_argument_name = '--stateful-metadata'
+  parser.add_argument(
+      stateful_metadata_argument_name,
+      type=arg_parsers.ArgDict(min_length=1),
+      default={},
+      action=arg_parsers.StoreOnceAction,
+      metavar='KEY=VALUE',
+      help=textwrap.dedent(
+          STATEFUL_METADATA_HELP.format(
+              argument_name=stateful_metadata_argument_name)))
 
 
 def AddMigStatefulUpdateInstanceFlag(parser):
@@ -688,12 +647,8 @@ def ValidateMigStatefulFlagsForInstanceConfigs(args,
                                                for_update=False,
                                                need_disk_source=False):
   """Validates the values of stateful flags for instance configs."""
-  if for_update:
-    stateful_disks = args.stateful_disk
-    flag_name = '--stateful-disk'
-  else:
-    stateful_disks = args.stateful_disk
-    flag_name = '--stateful-disk'
+  stateful_disks = args.stateful_disk
+  flag_name = '--stateful-disk'
   device_names = set()
   for stateful_disk in stateful_disks or []:
     if not stateful_disk.get('device-name'):
@@ -719,7 +674,7 @@ def ValidateMigStatefulFlagsForInstanceConfigs(args,
           parameter_name=flag_name,
           message='[source] is required for all stateful disks')
 
-    if mode_value and not stateful_disk.get('source'):
+    if not for_update and mode_value and not stateful_disk.get('source'):
       raise exceptions.InvalidArgumentException(
           parameter_name=flag_name,
           message='[mode] can be set then and only then when [source] is given')
@@ -730,7 +685,7 @@ def ValidateMigStatefulFlagsForInstanceConfigs(args,
       if stateful_disk_to_update.get(
           'device-name') in remove_stateful_disks_set:
         raise exceptions.InvalidArgumentException(
-            parameter_name=flag_name,
+            parameter_name='--remove-stateful-disks',
             message=('the same [device-name] `{0}` cannot be updated and'
                      ' removed in one command call'.format(
                          stateful_disk_to_update.get('device-name'))))
@@ -741,7 +696,7 @@ def ValidateMigStatefulFlagsForInstanceConfigs(args,
         update_stateful_metadata_set)
     if keys_intersection:
       raise exceptions.InvalidArgumentException(
-          parameter_name=flag_name,
+          parameter_name='--remove-stateful-metadata',
           message=('the same metadata key(s) `{0}` cannot be updated and'
                    ' removed in one command call'.format(
                        ', '.join(keys_intersection))))
@@ -752,15 +707,14 @@ def AddMigUpdateStatefulFlags(parser):
   stateful_disks_help = textwrap.dedent(STATEFUL_DISKS_HELP_BASE + """
       Use this argument multiple times to update more disks.
 
-      If stateful disk with given `device-name` exists in current instance
-      config, its properties will be replaced by the newly provided ones. In
-      other case new stateful disk definition will be added to the instance
-      config.
+      If a stateful disk with the given device name already exists in the
+      current instance config, its properties will be replaced by the newly
+      provided ones. Otherwise, a new stateful disk definition will be added to
+      the instance config.
 
       *device-name*::: (Requied) Device name of the disk to mark stateful.
       """ + STATEFUL_DISK_AUTO_DELETE_ARG_HELP)
   stateful_disk_flag_name = '--stateful-disk'
-  stateful_disk_flag_name_old = '--update-stateful-disk'
   parser.add_argument(
       stateful_disk_flag_name,
       type=arg_parsers.ArgDict(
@@ -774,26 +728,6 @@ def AddMigUpdateStatefulFlags(parser):
       help=stateful_disks_help,
   )
 
-  # DEPRECATED: --update-stateful-disk
-  parser.add_argument(
-      stateful_disk_flag_name_old,
-      type=arg_parsers.ArgDict(
-          spec={
-              'device-name':
-                  str,
-              'auto-delete':
-                  AutoDeleteFlag.ValidatorWithFlagName(
-                      stateful_disk_flag_name_old)
-          }),
-      action=actions.DeprecationAction(
-          stateful_disk_flag_name_old,
-          warn='The {flag_name} option is deprecated; '
-          'use --stateful-disk instead.',
-          removed=False,
-          action='append'),
-      dest='stateful_disk',
-      help=stateful_disks_help,
-  )
   parser.add_argument(
       '--remove-stateful-disks',
       metavar='DEVICE_NAME',
@@ -884,31 +818,39 @@ def ValidateMigInstanceRedistributionTypeFlag(instance_redistribution_type,
             'managed instance groups only.'))
 
 
-DISTRIBUTION_POLICY_TARGET_SHAPES = ['EVEN', 'ANY']
+DISTRIBUTION_POLICY_TARGET_SHAPE_CHOICES = {
+    'EVEN':
+        'The group schedules VM instance creation and deletion to achieve and '
+        'maintain an even number of managed instances across the selected '
+        'zones. The distribution is even when the number of managed instances '
+        'does not differ by more than 1 between any two zones. Recommended for'
+        ' highly available serving workloads.',
+    'BALANCED':
+        'The group prioritizes acquisition of resources, scheduling VMs in '
+        'zones where resources are available while distributing VMs as evenly '
+        'as possible across selected zones to minimize the impact of zonal '
+        'failure. Recommended for highly available serving or batch workloads '
+        'that do not require autoscaling.',
+    'ANY': 'The group picks zones for creating VM instances to fulfill the '
+           'requested number of VMs within present resource constraints and to '
+           'maximize utilization of unused zonal reservations. Recommended for '
+           'batch workloads that do not require high availability.'
+}
 
 
 def AddMigDistributionPolicyTargetShapeFlag(parser):
   """Add --target-distribution-shape flag to the parser."""
+  help_text = """\
+      Specifies how a regional managed instance group distributes its instances
+      across zones within the region. The default shape is ``EVEN''.
+    """
+
   parser.add_argument(
       '--target-distribution-shape',
       metavar='SHAPE',
       type=lambda x: x.upper(),
-      choices=DISTRIBUTION_POLICY_TARGET_SHAPES,
-      help="""\
-      Specify distribution policy target shape.
-
-      Target shape may be specified for regional managed instance group only.
-      By default it is set to EVEN.
-
-      The following target shapes are available:
-
-       * EVEN - managed instance group will create and delete instances
-         in a manner preserving or converging to even distribution.
-
-       * ANY - managed instance group will create instances based on present
-         capacity constraints and will not attempt to converge to even
-         distribution.
-      """)
+      choices=DISTRIBUTION_POLICY_TARGET_SHAPE_CHOICES,
+      help=help_text)
 
 
 def ValidateMigDistributionPolicyTargetShapeFlag(target_shape, group_ref):
