@@ -34,6 +34,18 @@ DEFAULT_LIST_FORMAT = """\
       ipCidrRange:label=RANGE
     )"""
 
+DEFAULT_LIST_FORMAT_WITH_IPV6_FIELD = """\
+    table(
+      name,
+      region.basename(),
+      network.basename(),
+      ipCidrRange:label=RANGE,
+      stackType,
+      ipv6AccessType,
+      ipv6CidrRange,
+      externalIpv6Prefix
+    )"""
+
 
 class SubnetworksCompleter(compute_completers.ListCommandCompleter):
 
@@ -55,19 +67,50 @@ def SubnetworkArgument(required=True, plural=False):
       region_explanation=compute_flags.REGION_PROPERTY_EXPLANATION)
 
 
+def SubnetworkArgumentForRouter(required=False, operation_type='added'):
+  return compute_flags.ResourceArgument(
+      resource_name='subnetwork',
+      name='--subnetwork',
+      completer=SubnetworksCompleter,
+      plural=False,
+      required=required,
+      regional_collection='compute.subnetworks',
+      short_help='The subnetwork of the interface being {0}.'.format(
+          operation_type),
+      region_explanation=(
+          'If not specified it will be set to the region of the router.'))
+
+
+def SubnetworkArgumentForServiceAttachment(required=True):
+  return compute_flags.ResourceArgument(
+      resource_name='subnetwork',
+      name='--nat-subnets',
+      completer=SubnetworksCompleter,
+      plural=True,
+      required=required,
+      regional_collection='compute.subnetworks',
+      short_help='The subnetworks provided by service producer to use for NAT',
+      region_explanation=(
+          'If not specified, it will be set to the region of the service '
+          'attachment.'))
+
+
 def SubnetworkResolver():
   return compute_flags.ResourceResolver.FromMap(
       'subnetwork', {compute_scope.ScopeEnum.REGION: 'compute.subnetworks'})
 
 
 def AddUpdateArgs(parser, include_alpha_logging,
-                  include_l7_internal_load_balancing, api_version):
+                  include_l7_internal_load_balancing, include_stack_type,
+                  include_ipv6_access_type, api_version):
   """Add args to the parser for subnet update.
 
   Args:
     parser: The argparse parser.
     include_alpha_logging: Include alpha-specific logging args.
     include_l7_internal_load_balancing: Include Internal HTTP(S) LB args.
+    include_stack_type: Include stack type args.
+    include_ipv6_access_type: Include IPv6 access type args.
     api_version: The api version of the request.
   """
   messages = apis.GetMessagesModule('compute',
@@ -188,6 +231,33 @@ def AddUpdateArgs(parser, include_alpha_logging,
         only applicable when the [--role=ACTIVE] flag is being used.
         """)
 
+  if include_stack_type:
+    parser.add_argument(
+        '--stack-type',
+        choices={
+            'IPV4_ONLY':
+                'New VMs in this subnet will only be assigned IPv4 addresses',
+            'IPV4_IPV6':
+                'New VMs in this subnet can have both IPv4 and IPv6 addresses'
+        },
+        type=arg_utils.ChoiceToEnumName,
+        help=('The stack type for this subnet to identify whether the IPv6 '
+              'feature is enabled or not.'))
+
+  if include_ipv6_access_type:
+    parser.add_argument(
+        '--ipv6-access-type',
+        choices={
+            'INTERNAL': 'VMs in this subnet can have internal IPv6.',
+            'EXTERNAL': 'VMs in this subnet can have external IPv6.'
+        },
+        type=arg_utils.ChoiceToEnumName,
+        help=('The access type of IPv6 address this subnet holds. It\'s '
+              'immutable and can only be specified during creation or the '
+              'time the subnet is updated into IPV4_IPV6 dual stack. If the '
+              'ipv6 access type is EXTERNAL then this subnet cannot enable '
+              'direct path.'))
+
   messages = apis.GetMessagesModule('compute', api_version)
   GetPrivateIpv6GoogleAccessTypeFlagMapper(messages).choice_arg.AddToParser(
       updated_field)
