@@ -22,11 +22,15 @@ from __future__ import unicode_literals
 import abc
 import copy
 
+from googlecloudsdk.api_lib.run import container_resource
+from googlecloudsdk.api_lib.run import job
 from googlecloudsdk.api_lib.run import k8s_object
 from googlecloudsdk.api_lib.run import revision
 from googlecloudsdk.api_lib.run import service
+from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.run import exceptions
 from googlecloudsdk.command_lib.run import name_generator
+from googlecloudsdk.command_lib.run import platforms
 from googlecloudsdk.command_lib.util.args import labels_util
 from googlecloudsdk.command_lib.util.args import repeated
 
@@ -50,11 +54,39 @@ class ConfigChanger(six.with_metaclass(abc.ABCMeta, object)):
     return resource
 
 
+def WithChanges(resource, changes):
+  """Apply ConfigChangers to resource.
+
+  It's undefined whether the input resource is modified.
+
+  Args:
+    resource: KubernetesObject, probably a Service.
+    changes: List of ConfigChangers.
+
+  Returns:
+    Changed resource.
+  """
+  for config_change in changes:
+    resource = config_change.Adjust(resource)
+  return resource
+
+
+def _AssertValidSecretKey(key, platform):
+  if platform == platforms.PLATFORM_MANAGED:
+    if not (key.isdigit() or key == 'latest'):
+      raise exceptions.ConfigurationError(
+          "Secret key must be an integer or 'latest'.")
+
+
 class LabelChanges(ConfigChanger):
   """Represents the user intent to modify metadata labels."""
 
-  def __init__(self, diff):
+  LABELS_NOT_ALLOWED_IN_REVISION = ([service.ENDPOINT_VISIBILITY])
+
+  def __init__(self, diff, copy_to_revision=True):
+    super(LabelChanges, self).__init__()
     self._diff = diff
+    self._copy_to_revision = copy_to_revision
 
   def Adjust(self, resource):
     # Currently assumes all "system"-owned labels are applied by the control
@@ -65,13 +97,17 @@ class LabelChanges(ConfigChanger):
     maybe_new_labels = update_result.GetOrNone()
     if maybe_new_labels:
       resource.metadata.labels = maybe_new_labels
-      # Service labels are the source of truth and *overwrite* revision labels.
-      # See run-labels-prd for deets.
-      # However, we need to preserve the nonce if there is one.
-      nonce = resource.template.labels.get(revision.NONCE_LABEL)
-      resource.template.metadata.labels = copy.deepcopy(maybe_new_labels)
-      if nonce:
-        resource.template.labels[revision.NONCE_LABEL] = nonce
+      if self._copy_to_revision and hasattr(resource.template, 'labels'):
+        # Service labels are the source of truth and *overwrite* revision labels
+        # See go/run-labels-prd for deets.
+        # However, we need to preserve the nonce if there is one.
+        nonce = resource.template.labels.get(revision.NONCE_LABEL)
+        resource.template.metadata.labels = copy.deepcopy(maybe_new_labels)
+        for label_to_remove in self.LABELS_NOT_ALLOWED_IN_REVISION:
+          if label_to_remove in resource.template.labels:
+            del resource.template.labels[label_to_remove]
+        if nonce:
+          resource.template.labels[revision.NONCE_LABEL] = nonce
     return resource
 
 
@@ -79,6 +115,7 @@ class ReplaceServiceChange(ConfigChanger):
   """Represents the user intent to replace the service."""
 
   def __init__(self, new_service):
+    super(ReplaceServiceChange, self).__init__()
     self._service = new_service
 
   def Adjust(self, resource):
@@ -95,15 +132,17 @@ class ReplaceServiceChange(ConfigChanger):
       self._service.metadata.resourceVersion = resource.metadata.resourceVersion
       # Knative will complain if you try to edit (incl remove) serving annots.
       # So replicate them here.
-      if not resource.is_managed:
-        for k, v in resource.annotations.items():
-          if k.startswith(k8s_object.SERVING_GROUP):
-            self._service.annotations[k] = v
+      for k, v in resource.annotations.items():
+        if k.startswith(k8s_object.SERVING_GROUP):
+          self._service.annotations[k] = v
     return self._service
 
 
 class EndpointVisibilityChange(LabelChanges):
-  """Represents the user intent to modify the endpoint visibility."""
+  """Represents the user intent to modify the endpoint visibility.
+
+  Only applies to Cloud Run for Anthos.
+  """
 
   def __init__(self, endpoint_visibility):
     """Determine label changes for modifying endpoint visibility.
@@ -118,20 +157,48 @@ class EndpointVisibilityChange(LabelChanges):
           additions={service.ENDPOINT_VISIBILITY: service.CLUSTER_LOCAL})
     else:
       diff = labels_util.Diff(subtractions=[service.ENDPOINT_VISIBILITY])
-    super(EndpointVisibilityChange, self).__init__(diff)
+    # Don't copy this label to the revision because it's not supported there.
+    # See b/154664962.
+    super(EndpointVisibilityChange, self).__init__(diff, False)
+
+
+class SetAnnotationChange(ConfigChanger):
+  """Represents the user intent to set an annotation."""
+
+  def __init__(self, key, value):
+    super(SetAnnotationChange, self).__init__()
+    self._key = key
+    self._value = value
+
+  def Adjust(self, resource):
+    resource.annotations[self._key] = self._value
+    return resource
+
+
+class DeleteAnnotationChange(ConfigChanger):
+  """Represents the user intent to delete an annotation."""
+
+  def __init__(self, key):
+    super(DeleteAnnotationChange, self).__init__()
+    self._key = key
+
+  def Adjust(self, resource):
+    annotations = resource.annotations
+    if self._key in annotations:
+      del annotations[self._key]
+    return resource
 
 
 class SetTemplateAnnotationChange(ConfigChanger):
   """Represents the user intent to set a template annotation."""
 
   def __init__(self, key, value):
+    super(SetTemplateAnnotationChange, self).__init__()
     self._key = key
     self._value = value
 
   def Adjust(self, resource):
-    annotations = k8s_object.AnnotationsFromMetadata(
-        resource.MessagesModule(), resource.template.metadata)
-    annotations[self._key] = self._value
+    resource.template.annotations[self._key] = self._value
     return resource
 
 
@@ -139,27 +206,77 @@ class DeleteTemplateAnnotationChange(ConfigChanger):
   """Represents the user intent to delete a template annotation."""
 
   def __init__(self, key):
+    super(DeleteTemplateAnnotationChange, self).__init__()
     self._key = key
 
   def Adjust(self, resource):
-    annotations = k8s_object.AnnotationsFromMetadata(
-        resource.MessagesModule(), resource.template.metadata)
+    annotations = resource.template.annotations
     if self._key in annotations:
       del annotations[self._key]
     return resource
 
 
+class SetLaunchStageAnnotationChange(ConfigChanger):
+  """Sets a VPC connector annotation on the service."""
+
+  def __init__(self, launch_stage):
+    super(SetLaunchStageAnnotationChange, self).__init__()
+    self._launch_stage = launch_stage
+
+  def Adjust(self, resource):
+    if self._launch_stage == base.ReleaseTrack.GA:
+      return resource
+    else:
+      resource.annotations[
+          k8s_object.LAUNCH_STAGE_ANNOTATION] = self._launch_stage.id
+      return resource
+
+
+class SetClientNameAndVersionAnnotationChange(ConfigChanger):
+  """Sets the client name and version annotations."""
+
+  def __init__(self, client_name, client_version):
+    super(SetClientNameAndVersionAnnotationChange, self).__init__()
+    self._client_name = client_name
+    self._client_version = client_version
+
+  def Adjust(self, resource):
+    if self._client_name is not None:
+      resource.annotations[
+          k8s_object.CLIENT_NAME_ANNOTATION] = self._client_name
+      resource.template.annotations[
+          k8s_object.CLIENT_NAME_ANNOTATION] = self._client_name
+    if self._client_version is not None:
+      resource.annotations[
+          k8s_object.CLIENT_VERSION_ANNOTATION] = self._client_version
+      resource.template.annotations[
+          k8s_object.CLIENT_VERSION_ANNOTATION] = self._client_version
+    return resource
+
+
+class SandboxChange(ConfigChanger):
+  """Sets a sandbox annotation on the service."""
+
+  def __init__(self, sandbox):
+    super(SandboxChange, self).__init__()
+    self._sandbox = sandbox
+
+  def Adjust(self, resource):
+    resource.template.annotations[
+        container_resource.SANDBOX_ANNOTATION] = self._sandbox
+    return resource
+
+
 class VpcConnectorChange(ConfigChanger):
   """Sets a VPC connector annotation on the service."""
 
   def __init__(self, connector_name):
+    super(VpcConnectorChange, self).__init__()
     self._connector_name = connector_name
 
   def Adjust(self, resource):
-    annotations = k8s_object.AnnotationsFromMetadata(resource.MessagesModule(),
-                                                     resource.metadata)
-    annotations['run.googleapis.com/vpc-access-connector'] = (
-        self._connector_name)
+    resource.template.annotations[
+        container_resource.VPC_ACCESS_ANNOTATION] = self._connector_name
     return resource
 
 
@@ -167,10 +284,11 @@ class ClearVpcConnectorChange(ConfigChanger):
   """Clears a VPC connector annotation on the service."""
 
   def Adjust(self, resource):
-    annotations = k8s_object.AnnotationsFromMetadata(resource.MessagesModule(),
-                                                     resource.metadata)
-    if 'run.googleapis.com/vpc-access-connector' in annotations:
-      del annotations['run.googleapis.com/vpc-access-connector']
+    annotations = resource.template.annotations
+    if container_resource.VPC_ACCESS_ANNOTATION in annotations:
+      del annotations[container_resource.VPC_ACCESS_ANNOTATION]
+    if container_resource.EGRESS_SETTINGS_ANNOTATION in annotations:
+      del annotations[container_resource.EGRESS_SETTINGS_ANNOTATION]
     return resource
 
 
@@ -180,36 +298,44 @@ class ImageChange(ConfigChanger):
   deployment_type = 'container'
 
   def __init__(self, image):
+    super(ImageChange, self).__init__()
     self.image = image
 
   def Adjust(self, resource):
-    resource.annotations[revision.USER_IMAGE_ANNOTATION] = (
+    resource.annotations[container_resource.USER_IMAGE_ANNOTATION] = (
         self.image)
-    resource.template.annotations[revision.USER_IMAGE_ANNOTATION] = (
-        self.image)
-    resource.template.image = self.image
+    if hasattr(resource.template, 'annotations'):
+      resource.template.annotations[
+          container_resource.USER_IMAGE_ANNOTATION] = (
+              self.image)
+    resource.image = self.image
     return resource
 
 
+def _PruneMapping(mapping, keys_to_remove, clear_others):
+  if clear_others:
+    mapping.clear()
+  else:
+    for var_or_path in keys_to_remove:
+      if var_or_path in mapping:
+        del mapping[var_or_path]
+
+
 class EnvVarLiteralChanges(ConfigChanger):
   """Represents the user intent to modify environment variables string literals."""
 
-  def __init__(self, env_vars_to_update=None,
-               env_vars_to_remove=None, clear_others=False):
+  def __init__(self, updates, removes, clear_others):
     """Initialize a new EnvVarLiteralChanges object.
 
     Args:
-      env_vars_to_update: {str, str}, Update env var names and values.
-      env_vars_to_remove: [str], List of env vars to remove.
+      updates: {str, str}, Update env var names and values.
+      removes: [str], List of env vars to remove.
       clear_others: bool, If true, clear all non-updated env vars.
     """
-    self._to_update = None
-    self._to_remove = None
+    super(EnvVarLiteralChanges, self).__init__()
+    self._updates = updates
+    self._removes = removes
     self._clear_others = clear_others
-    if env_vars_to_update:
-      self._to_update = {k.strip(): v for k, v in env_vars_to_update.items()}
-    if env_vars_to_remove:
-      self._to_remove = [k.lstrip() for k in env_vars_to_remove]
 
   def Adjust(self, resource):
     """Mutates the given config's env vars to match the desired changes.
@@ -226,64 +352,50 @@ class EnvVarLiteralChanges(ConfigChanger):
         (e.g. env var's secret source can't be replaced with a config map
         source).
     """
-    if self._clear_others:
-      resource.template.env_vars.literals.clear()
-    elif self._to_remove:
-      for env_var in self._to_remove:
-        if env_var in resource.template.env_vars.literals:
-          del resource.template.env_vars.literals[env_var]
+    _PruneMapping(resource.template.env_vars.literals, self._removes,
+                  self._clear_others)
 
-    if self._to_update:
-      try:
-        resource.template.env_vars.literals.update(self._to_update)
-      except KeyError as e:
-        raise exceptions.ConfigurationError(
-            'Cannot update environment variable [{}] to string literal '
-            'because it has already been set with a different type.'.format(
-                e.args[0]))
+    try:
+      resource.template.env_vars.literals.update(self._updates)
+    except KeyError as e:
+      raise exceptions.ConfigurationError(
+          'Cannot update environment variable [{}] to string literal '
+          'because it has already been set with a different type.'.format(
+              e.args[0]))
     return resource
 
 
-class EnvVarSourceChanges(ConfigChanger):
-  """Represents the user intent to modify environment variables sources."""
+class SecretEnvVarChanges(ConfigChanger):
+  """Represents the user intent to modify environment variable secrets."""
 
-  def __init__(self, env_vars_to_update=None,
-               env_vars_to_remove=None, clear_others=False):
-    """Initialize a new EnvVarSourceChanges object.
+  def __init__(self, updates, removes, clear_others):
+    """Initialize a new SecretEnvVarChanges object.
 
     Args:
-      env_vars_to_update: {str, str}, Update env var names and values.
-      env_vars_to_remove: [str], List of env vars to remove.
+      updates: {str, str}, Update env var names and values.
+      removes: [str], List of env vars to remove.
       clear_others: bool, If true, clear all non-updated env vars.
 
     Raises:
       ConfigurationError if a key hasn't been provided for a source.
     """
-    self._to_update = None
-    self._to_remove = None
+    super(SecretEnvVarChanges, self).__init__()
+    self._updates = {}
+    for name, v in updates.items():
+      # Split the given values into 2 parts:
+      #    [env var source name, source data item key]
+      value = v.split(':', 1)
+      if len(value) < 2:
+        value.append(self._OmittedSecretKeyDefault(name))
+      self._updates[name] = value
+    self._removes = removes
     self._clear_others = clear_others
-    if env_vars_to_update:
-      self._to_update = {}
-      for k, v in env_vars_to_update.items():
-        name = k.strip()
-        # Split the given values into 2 parts:
-        #    [env var source name, source data item key]
-        value = v.split(':', 1)
-        if len(value) < 2:
-          raise exceptions.ConfigurationError(
-              'Missing required item key for environment variable [{}].'
-              .format(name))
-        self._to_update[name] = value
-    if env_vars_to_remove:
-      self._to_remove = [k.lstrip() for k in env_vars_to_remove]
 
-  @abc.abstractmethod
-  def _MakeEnvVarSource(self, messages, name, key):
-    """Returns an instance of an EnvVarSource."""
-
-  @abc.abstractmethod
-  def _GetEnvVars(self, resource):
-    """Returns a k8s_object.ListAsDictionaryWrapper to manage env vars with a source."""
+  def _OmittedSecretKeyDefault(self, name):
+    if platforms.GetPlatform() == platforms.PLATFORM_MANAGED:
+      return 'latest'
+    raise exceptions.ConfigurationError(
+        'Missing required item key for environment variable [{}].'.format(name))
 
   def Adjust(self, resource):
     """Mutates the given config's env vars to match the desired changes.
@@ -300,58 +412,99 @@ class EnvVarSourceChanges(ConfigChanger):
         (e.g. env var's secret source can't be replaced with a config map
         source).
     """
-    env_vars = self._GetEnvVars(resource)
+    env_vars = resource.template.env_vars.secrets
+    _PruneMapping(env_vars, self._removes, self._clear_others)
 
-    if self._clear_others:
-      env_vars.clear()
-    elif self._to_remove:
-      for env_var in self._to_remove:
-        if env_var in env_vars:
-          del env_vars[env_var]
-
-    if self._to_update:
-      for name, (source_name, source_key) in self._to_update.items():
-        try:
-          env_vars[name] = self._MakeEnvVarSource(
-              resource.MessagesModule(), source_name, source_key)
-        except KeyError:
-          raise exceptions.ConfigurationError(
-              'Cannot update environment variable [{}] to the given type '
-              'because it has already been set with a different type.'.format(
-                  name))
+    for name, (source_name, source_key) in self._updates.items():
+      try:
+        env_vars[name] = self._MakeEnvVarSource(resource.MessagesModule(),
+                                                source_name, source_key)
+      except KeyError:
+        raise exceptions.ConfigurationError(
+            'Cannot update environment variable [{}] to the given type '
+            'because it has already been set with a different type.'.format(
+                name))
     return resource
 
-
-class SecretEnvVarChanges(EnvVarSourceChanges):
-  """Represents the user intent to modify environment variable secrets."""
-
   def _MakeEnvVarSource(self, messages, name, key):
+    _AssertValidSecretKey(key, platforms.GetPlatform())
     return messages.EnvVarSource(
-        secretKeyRef=messages.SecretKeySelector(
-            name=name,
-            key=key))
-
-  def _GetEnvVars(self, resource):
-    return resource.template.env_vars.secrets
+        secretKeyRef=messages.SecretKeySelector(name=name, key=key))
 
 
-class ConfigMapEnvVarChanges(EnvVarSourceChanges):
+class ConfigMapEnvVarChanges(ConfigChanger):
   """Represents the user intent to modify environment variable config maps."""
 
+  def __init__(self, updates, removes, clear_others):
+    """Initialize a new ConfigMapEnvVarChanges object.
+
+    Args:
+      updates: {str, str}, Update env var names and values.
+      removes: [str], List of env vars to remove.
+      clear_others: bool, If true, clear all non-updated env vars.
+
+    Raises:
+      ConfigurationError if a key hasn't been provided for a source.
+    """
+    super(ConfigMapEnvVarChanges, self).__init__()
+    self._updates = {}
+    for name, v in updates.items():
+      # Split the given values into 2 parts:
+      #    [env var source name, source data item key]
+      value = v.split(':', 1)
+      if len(value) < 2:
+        value.append(self._OmittedSecretKeyDefault(name))
+      self._updates[name] = value
+    self._removes = removes
+    self._clear_others = clear_others
+
+  def _OmittedSecretKeyDefault(self, name):
+    if platforms.GetPlatform() == platforms.PLATFORM_MANAGED:
+      return 'latest'
+    raise exceptions.ConfigurationError(
+        'Missing required item key for environment variable [{}].'.format(name))
+
+  def Adjust(self, resource):
+    """Mutates the given config's env vars to match the desired changes.
+
+    Args:
+      resource: k8s_object to adjust
+
+    Returns:
+      The adjusted resource
+
+    Raises:
+      ConfigurationError if there's an attempt to replace the source of an
+        existing environment variable whose source is of a different type
+        (e.g. env var's secret source can't be replaced with a config map
+        source).
+    """
+    env_vars = resource.template.env_vars.config_maps
+    _PruneMapping(env_vars, self._removes, self._clear_others)
+
+    for name, (source_name, source_key) in self._updates.items():
+      try:
+        env_vars[name] = self._MakeEnvVarSource(resource.MessagesModule(),
+                                                source_name, source_key)
+      except KeyError:
+        raise exceptions.ConfigurationError(
+            'Cannot update environment variable [{}] to the given type '
+            'because it has already been set with a different type.'.format(
+                name))
+    return resource
+
   def _MakeEnvVarSource(self, messages, name, key):
     return messages.EnvVarSource(
         configMapKeyRef=messages.ConfigMapKeySelector(
             name=name,
             key=key))
 
-  def _GetEnvVars(self, resource):
-    return resource.template.env_vars.config_maps
-
 
 class ResourceChanges(ConfigChanger):
   """Represents the user intent to update resource limits."""
 
   def __init__(self, memory=None, cpu=None):
+    super(ResourceChanges, self).__init__()
     self._memory = memory
     self._cpu = cpu
 
@@ -375,6 +528,7 @@ class CloudSQLChanges(ConfigChanger):
       region: Region to use as the default region for Cloud SQL instances
       args: Args to the command.
     """
+    super(CloudSQLChanges, self).__init__()
     self._project = project
     self._region = region
     self._args = args
@@ -407,7 +561,7 @@ class CloudSQLChanges(ConfigChanger):
   def Adjust(self, resource):
     def GetCurrentInstances():
       annotation_val = resource.template.annotations.get(
-          revision.CLOUDSQL_ANNOTATION)
+          container_resource.CLOUDSQL_ANNOTATION)
       if annotation_val:
         return annotation_val.split(',')
       return []
@@ -416,7 +570,7 @@ class CloudSQLChanges(ConfigChanger):
         self, 'cloudsql-instances', GetCurrentInstances)
     if instances is not None:
       resource.template.annotations[
-          revision.CLOUDSQL_ANNOTATION] = ','.join(instances)
+          container_resource.CLOUDSQL_ANNOTATION] = ','.join(instances)
     return resource
 
   def _Augment(self, instance_str):
@@ -444,6 +598,7 @@ class ConcurrencyChanges(ConfigChanger):
   """Represents the user intent to update concurrency preference."""
 
   def __init__(self, concurrency):
+    super(ConcurrencyChanges, self).__init__()
     self._concurrency = None if concurrency == 'default' else int(concurrency)
 
   def Adjust(self, resource):
@@ -456,6 +611,7 @@ class TimeoutChanges(ConfigChanger):
   """Represents the user intent to update request duration."""
 
   def __init__(self, timeout):
+    super(TimeoutChanges, self).__init__()
     self._timeout = timeout
 
   def Adjust(self, resource):
@@ -468,6 +624,7 @@ class ServiceAccountChanges(ConfigChanger):
   """Represents the user intent to change service account for the revision."""
 
   def __init__(self, service_account):
+    super(ServiceAccountChanges, self).__init__()
     self._service_account = service_account
 
   def Adjust(self, resource):
@@ -476,15 +633,21 @@ class ServiceAccountChanges(ConfigChanger):
     return resource
 
 
+_MAX_RESOURCE_NAME_LENGTH = 63
+
+
 class RevisionNameChanges(ConfigChanger):
   """Represents the user intent to change revision name."""
 
   def __init__(self, revision_suffix):
+    super(RevisionNameChanges, self).__init__()
     self._revision_suffix = revision_suffix
 
   def Adjust(self, resource):
     """Mutates the given config's revision name to match what's desired."""
-    resource.template.name = '{}-{}'.format(resource.name,
+    max_prefix_length = (
+        _MAX_RESOURCE_NAME_LENGTH - len(self._revision_suffix) - 1)
+    resource.template.name = '{}-{}'.format(resource.name[:max_prefix_length],
                                             self._revision_suffix)
     return resource
 
@@ -494,48 +657,61 @@ def _GenerateVolumeName(prefix):
   return name_generator.GenerateName(sections=3, separator='-', prefix=prefix)
 
 
-class VolumeChanges(ConfigChanger):
-  """Represents the user intent to modify volumes and mounts."""
+def _UniqueVolumeName(source_name, existing_volumes):
+  """Generate unique volume name.
 
-  def __init__(self,
-               mounts_to_update=None,
-               mounts_to_remove=None,
-               clear_others=False):
-    """Initialize a new VolumeChanges object.
+  The names that connect volumes and mounts must be unique even if their
+  source volume names match.
+
+  Args:
+    source_name: (Potentially clashing) name.
+    existing_volumes: Names in use.
+
+  Returns:
+    Unique name.
+  """
+  volume_name = None
+  while volume_name is None or volume_name in existing_volumes:
+    volume_name = _GenerateVolumeName(source_name)
+  return volume_name
+
+
+def _PruneVolumes(volume_mounts, volumes):
+  """Delete all volumes no longer being mounted.
+
+  Args:
+    volume_mounts: resource.template.volume_mounts
+    volumes: resource.template.volumes
+  """
+  for volume in list(volumes):
+    if volume not in volume_mounts.values():
+      del volumes[volume]
+
+
+class SecretVolumeChanges(ConfigChanger):
+  """Represents the user intent to change volumes with secret source types."""
+
+  def __init__(self, updates, removes, clear_others):
+    """Initialize a new SecretVolumeChanges object.
 
     Args:
-      mounts_to_update: {str, str}, Update mount path and volume fields.
-      mounts_to_remove: [str], List of mount paths to remove.
+      updates: {str, str}, Update mount path and volume fields.
+      removes: [str], List of mount paths to remove.
       clear_others: bool, If true, clear all non-updated volumes and mounts of
         the given [volume_type].
     """
-    self._to_update = None
-    self._to_remove = None
+    super(SecretVolumeChanges, self).__init__()
+    self._updates = {}
+    for k, v in updates.items():
+      # Split the given values into 2 parts:
+      #    [volume source name, data item key]
+      update_value = v.split(':', 1)
+      # Pad with None if no data item key specified
+      if len(update_value) < 2:
+        update_value.append(None)
+      self._updates[k] = update_value
+    self._removes = removes
     self._clear_others = clear_others
-    if mounts_to_update:
-      self._to_update = {}
-      for k, v in mounts_to_update.items():
-        # Split the given values into 2 parts:
-        #    [volume source name, data item key]
-        update_value = v.split(':', 1)
-        # Pad with None if no data item key specified
-        if len(update_value) < 2:
-          update_value.append(None)
-        self._to_update[k.strip()] = update_value
-    if mounts_to_remove:
-      self._to_remove = [k.lstrip() for k in mounts_to_remove]
-
-  @abc.abstractmethod
-  def _MakeVolumeSource(self, messages, name, key=None):
-    """Returns an instance of a volume source."""
-
-  @abc.abstractmethod
-  def _GetVolumes(self, resource):
-    """Returns a k8s_object.ListAsDictionaryWrapper to manage volumes."""
-
-  @abc.abstractmethod
-  def _GetVolumeMounts(self, resource):
-    """Returns a k8s_object.ListAsDictionaryWrapper to manage volume mounts."""
 
   def Adjust(self, resource):
     """Mutates the given config's volumes to match the desired changes.
@@ -552,73 +728,105 @@ class VolumeChanges(ConfigChanger):
         the new volume (e.g. mount that points to a volume with a secret source
         can't be replaced with a volume that has a config map source).
     """
-    volume_mounts = self._GetVolumeMounts(resource)
-    volumes = self._GetVolumes(resource)
+    volume_mounts = resource.template.volume_mounts.secrets
+    volumes = resource.template.volumes.secrets
 
-    if self._clear_others:
-      volume_mounts.clear()
-    elif self._to_remove:
-      for path in self._to_remove:
-        if path in volume_mounts:
-          del volume_mounts[path]
+    _PruneMapping(volume_mounts, self._removes, self._clear_others)
 
-    if self._to_update:
-      for path, (source_name, source_key) in self._to_update.items():
-        # Generate unique volume name so that volume source configurations
-        # can be unique (e.g. different items) even if the source name matches
-        volume_name = None
-        while volume_name is None or volume_name in resource.template.volumes:
-          volume_name = _GenerateVolumeName(source_name)
+    for file_path, (source_name, source_key) in self._updates.items():
+      volume_name = _UniqueVolumeName(source_name, resource.template.volumes)
 
-        # Set the mount and volume
-        try:
-          volume_mounts[path] = volume_name
-        except KeyError:
-          raise exceptions.ConfigurationError(
-              'Cannot update mount [{}] because its mounted volume '
-              'is of a different source type.'.format(path))
-        volumes[volume_name] = self._MakeVolumeSource(
-            resource.MessagesModule(), source_name, source_key)
-
-    # Delete all volumes no longer being mounted
-    for volume in list(volumes):
-      if volume not in volume_mounts.values():
-        del volumes[volume]
+      # volume_mounts is a special mapping that filters for the current kind
+      # of mount and KeyErrors on existing keys with other types.
+      try:
+        volume_mounts[file_path] = volume_name
+      except KeyError:
+        raise exceptions.ConfigurationError(
+            'Cannot update mount [{}] because its mounted volume '
+            'is of a different source type.'.format(file_path))
+      volumes[volume_name] = self._MakeVolumeSource(resource.MessagesModule(),
+                                                    source_name, source_key)
 
+    _PruneVolumes(volume_mounts, volumes)
     return resource
 
-
-class SecretVolumeChanges(VolumeChanges):
-  """Represents the user intent to change volumes with secret source types."""
-
   def _MakeVolumeSource(self, messages, name, key=None):
     source = messages.SecretVolumeSource(secretName=name)
     if key is not None:
+      _AssertValidSecretKey(key, platforms.GetPlatform())
       source.items.append(messages.KeyToPath(key=key, path=key))
     return source
 
-  def _GetVolumes(self, resource):
-    return resource.template.volumes.secrets
 
-  def _GetVolumeMounts(self, resource):
-    return resource.template.volume_mounts.secrets
-
-
-class ConfigMapVolumeChanges(VolumeChanges):
+class ConfigMapVolumeChanges(ConfigChanger):
   """Represents the user intent to change volumes with config map source types."""
 
+  def __init__(self, updates, removes, clear_others):
+    """Initialize a new ConfigMapVolumeChanges object.
+
+    Args:
+      updates: {str, [str, str]}, Update mount path and volume fields.
+      removes: [str], List of mount paths to remove.
+      clear_others: bool, If true, clear all non-updated volumes and mounts of
+        the given [volume_type].
+    """
+    super(ConfigMapVolumeChanges, self).__init__()
+    self._updates = {}
+    for k, v in updates.items():
+      # Split the given values into 2 parts:
+      #    [volume source name, data item key]
+      update_value = v.split(':', 1)
+      # Pad with None if no data item key specified
+      if len(update_value) < 2:
+        update_value.append(None)
+      self._updates[k] = update_value
+    self._removes = removes
+    self._clear_others = clear_others
+
+  def Adjust(self, resource):
+    """Mutates the given config's volumes to match the desired changes.
+
+    Args:
+      resource: k8s_object to adjust
+
+    Returns:
+      The adjusted resource
+
+    Raises:
+      ConfigurationError if there's an attempt to replace the volume a mount
+        points to whose existing volume has a source of a different type than
+        the new volume (e.g. mount that points to a volume with a secret source
+        can't be replaced with a volume that has a config map source).
+    """
+    volume_mounts = resource.template.volume_mounts.config_maps
+    volumes = resource.template.volumes.config_maps
+
+    _PruneMapping(volume_mounts, self._removes, self._clear_others)
+
+    for path, (source_name, source_key) in self._updates.items():
+      volume_name = _UniqueVolumeName(source_name, resource.template.volumes)
+
+      # volume_mounts is a special mapping that filters for the current kind
+      # of mount and KeyErrors on existing keys with other types.
+      try:
+        volume_mounts[path] = volume_name
+      except KeyError:
+        raise exceptions.ConfigurationError(
+            'Cannot update mount [{}] because its mounted volume '
+            'is of a different source type.'.format(path))
+      volumes[volume_name] = self._MakeVolumeSource(resource.MessagesModule(),
+                                                    source_name, source_key)
+
+    _PruneVolumes(volume_mounts, volumes)
+
+    return resource
+
   def _MakeVolumeSource(self, messages, name, key=None):
     source = messages.ConfigMapVolumeSource(name=name)
     if key is not None:
       source.items.append(messages.KeyToPath(key=key, path=key))
     return source
 
-  def _GetVolumes(self, resource):
-    return resource.template.volumes.config_maps
-
-  def _GetVolumeMounts(self, resource):
-    return resource.template.volume_mounts.config_maps
-
 
 class NoTrafficChange(ConfigChanger):
   """Represents the user intent to block traffic for a new revision."""
@@ -635,19 +843,61 @@ class NoTrafficChange(ConfigChanger):
       raise exceptions.ConfigurationError(
           '--no-traffic not supported when creating a new service.')
 
-    resource.traffic.ZeroLatestTraffic(resource.status.latestReadyRevisionName)
+    resource.spec_traffic.ZeroLatestTraffic(
+        resource.status.latestReadyRevisionName)
     return resource
 
 
 class TrafficChanges(ConfigChanger):
-  """Represents the user intent to change a services traffic assignments."""
+  """Represents the user intent to change a service's traffic assignments."""
 
-  def __init__(self, new_percentages):
+  def __init__(self,
+               new_percentages,
+               by_tag=False,
+               tags_to_update=None,
+               tags_to_remove=None,
+               clear_other_tags=False):
+    super(TrafficChanges, self).__init__()
     self._new_percentages = new_percentages
+    self._by_tag = by_tag
+    self._tags_to_update = tags_to_update or {}
+    self._tags_to_remove = tags_to_remove or []
+    self._clear_other_tags = clear_other_tags
 
   def Adjust(self, resource):
-    """Mutates the given services traffic assignments."""
-    resource.traffic.UpdateTraffic(self._new_percentages)
+    """Mutates the given service's traffic assignments."""
+    if self._tags_to_update or self._tags_to_remove or self._clear_other_tags:
+      resource.spec_traffic.UpdateTags(self._tags_to_update,
+                                       self._tags_to_remove,
+                                       self._clear_other_tags)
+    if self._new_percentages:
+      if self._by_tag:
+        tag_to_key = resource.spec_traffic.TagToKey()
+        percentages = {}
+        for tag in self._new_percentages:
+          try:
+            percentages[tag_to_key[tag]] = self._new_percentages[tag]
+          except KeyError:
+            raise exceptions.ConfigurationError(
+                'There is no revision tagged with [{}] in the traffic allocation for [{}].'
+                .format(tag, resource.name))
+      else:
+        percentages = self._new_percentages
+      resource.spec_traffic.UpdateTraffic(percentages)
+    return resource
+
+
+class TagOnDeployChange(ConfigChanger):
+  """The intent to provide a tag for the revision you're currently deploying."""
+
+  def __init__(self, tag):
+    super(TagOnDeployChange, self).__init__()
+    self._tag = tag
+
+  def Adjust(self, resource):
+    """Gives the revision that's being created the given tag."""
+    tags_to_update = {self._tag: resource.template.name}
+    resource.spec_traffic.UpdateTags(tags_to_update, [], False)
     return resource
 
 
@@ -655,6 +905,7 @@ class ContainerCommandChange(ConfigChanger):
   """Represents the user intent to change the 'command' for the container."""
 
   def __init__(self, command):
+    super(ContainerCommandChange, self).__init__()
     self._commands = command
 
   def Adjust(self, resource):
@@ -666,6 +917,7 @@ class ContainerArgsChange(ConfigChanger):
   """Represents the user intent to change the 'args' for the container."""
 
   def __init__(self, args):
+    super(ContainerArgsChange, self).__init__()
     self._args = args
 
   def Adjust(self, resource):
@@ -689,6 +941,7 @@ class ContainerPortChange(ConfigChanger):
       use_http2: bool, True to set the port name for http/2, False to unset it,
         or None to not modify the port name.
     """
+    super(ContainerPortChange, self).__init__()
     self._port = port
     self._http2 = use_http2
 
@@ -718,3 +971,33 @@ class ContainerPortChange(ConfigChanger):
     else:
       resource.template.container.reset('ports')
     return resource
+
+
+class SpecChange(ConfigChanger):
+  """Represents the user intent to update field in the resource's spec."""
+
+  def __init__(self, field, value):
+    super(SpecChange, self).__init__()
+    self._field = field
+    self._value = value
+
+  def Adjust(self, resource):
+    setattr(resource.spec, self._field, self._value)
+    return resource
+
+
+class JobMaxAttemptsChange(ConfigChanger):
+  """Represents the user intent to update a job's restart policy."""
+
+  def __init__(self, max_attempts):
+    super(JobMaxAttemptsChange, self).__init__()
+    self._max_attempts = max_attempts
+
+  def Adjust(self, resource):
+    if self._max_attempts == 1:
+      resource.template.restart_policy = job.RestartPolicy.NEVER
+      resource.backoff_limit = 0
+    else:
+      resource.template.restart_policy = job.RestartPolicy.ON_FAILURE
+      resource.backoff_limit = self._max_attempts - 1
+    return resource
