@@ -19,6 +19,7 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import unicode_literals
 
+
 import contextlib
 import errno
 import os
@@ -26,20 +27,28 @@ import re
 import signal
 import subprocess
 import sys
+import threading
 import time
 
+
+from googlecloudsdk.core import argv_utils
 from googlecloudsdk.core import config
 from googlecloudsdk.core import exceptions
 from googlecloudsdk.core import log
 from googlecloudsdk.core import properties
 from googlecloudsdk.core.configurations import named_configs
 from googlecloudsdk.core.util import encoding
+from googlecloudsdk.core.util import parallel
 from googlecloudsdk.core.util import platforms
 
 import six
 from six.moves import map
 
 
+class OutputStreamProcessingException(exceptions.Error):
+  """Error class for errors raised during output stream processing."""
+
+
 class PermissionError(exceptions.Error):
   """User does not have execute permissions."""
 
@@ -103,7 +112,7 @@ def _GetShellExecutable():
   """
   shells = ['/bin/bash', '/bin/sh']
 
-  user_shell = os.getenv('SHELL')
+  user_shell = encoding.GetEncodedValue(os.environ, 'SHELL')
   if user_shell and os.path.basename(user_shell) in _BORNE_COMPATIBLE_SHELLS:
     shells.insert(0, user_shell)
 
@@ -221,7 +230,7 @@ def ArgsForGcloud():
   if not sys.executable:
     # In hermetic par/stub files sys.executable is None. In regular installs,
     # and in classic par/stub files it is a non-empty string.
-    return _GetToolArgs(None, None, sys.argv[0])
+    return _GetToolArgs(None, None, argv_utils.GetDecodedArgv()[0])
   return ArgsForPythonTool(config.GcloudPath())
 
 
@@ -261,6 +270,54 @@ def _ReplaceSignal(signo, handler):
     signal.signal(signo, old_handler)
 
 
+def _Exec(args,
+          process_holder,
+          env=None,
+          out_func=None,
+          err_func=None,
+          in_str=None,
+          **extra_popen_kwargs):
+  """See Exec docstring."""
+  if out_func:
+    extra_popen_kwargs['stdout'] = subprocess.PIPE
+  if err_func:
+    extra_popen_kwargs['stderr'] = subprocess.PIPE
+  if in_str:
+    extra_popen_kwargs['stdin'] = subprocess.PIPE
+  try:
+    if args and isinstance(args, list):
+      # On Python 2.x on Windows, the first arg can't be unicode. We encode
+      # encode it anyway because there is really nothing else we can do if
+      # that happens.
+      # https://bugs.python.org/issue19264
+      args = [encoding.Encode(a) for a in args]
+    p = subprocess.Popen(args, env=_GetToolEnv(env=env), **extra_popen_kwargs)
+  except OSError as err:
+    if err.errno == errno.EACCES:
+      raise PermissionError(err.strerror)
+    elif err.errno == errno.ENOENT:
+      raise InvalidCommandError(args[0])
+    raise
+  process_holder.process = p
+
+  if process_holder.signum is not None:
+    # This covers the small possibility that process_holder handled a
+    # signal when the process was starting but not yet set to
+    # process_holder.process.
+    if p.poll() is None:
+      p.terminate()
+
+  if isinstance(in_str, six.text_type):
+    in_str = in_str.encode('utf-8')
+  stdout, stderr = list(map(encoding.Decode, p.communicate(input=in_str)))
+
+  if out_func:
+    out_func(stdout)
+  if err_func:
+    err_func(stderr)
+  return p.returncode
+
+
 def Exec(args,
          env=None,
          no_exit=False,
@@ -301,15 +358,111 @@ def Exec(args,
   # started and the original is killed.  When running in a shell, the prompt
   # returns as soon as the parent is killed even though the child is still
   # running.  subprocess waits for the new process to finish before returning.
-  env = _GetToolEnv(env=env)
+  process_holder = _ProcessHolder()
 
+  # pylint:disable=protected-access
+  # Python 3 has a cleaner way to check if on main thread, but must support PY2.
+  if isinstance(threading.current_thread(), threading._MainThread):
+    # pylint:enable=protected-access
+    # Signal replacement is not allowed by Python on non-main threads.
+    # https://bugs.python.org/issue38904
+    with _ReplaceSignal(signal.SIGTERM, process_holder.Handler):
+      with _ReplaceSignal(signal.SIGINT, process_holder.Handler):
+        ret_val = _Exec(args, process_holder, env, out_func, err_func, in_str,
+                        **extra_popen_kwargs)
+  else:
+    ret_val = _Exec(args, process_holder, env, out_func, err_func, in_str,
+                    **extra_popen_kwargs)
+
+  if no_exit and process_holder.signum is None:
+    return ret_val
+  sys.exit(ret_val)
+
+
+def _ProcessStreamHandler(proc, err=False, handler=log.Print):
+  """Process output stream from a running subprocess in realtime."""
+  stream = proc.stderr if err else proc.stdout
+  stream_reader = stream.readline
+  while True:
+    line = stream_reader() or b''
+    if not line and proc.poll() is not None:
+      try:
+        stream.close()
+      except OSError:
+        pass  # This is thread cleanup so we should just
+        # exit so runner can Join()
+      break
+    line_str = line.decode('utf-8')
+    line_str = line_str.rstrip('\r\n')
+    if line_str:
+      handler(line_str)
+
+
+def _KillProcIfRunning(proc):
+  """Kill process and close open streams."""
+  if proc:
+    if proc.poll() is None:
+      proc.terminate()
+    try:
+      if not proc.stdin.closed:
+        proc.stdin.close()
+      if not proc.stdout.closed:
+        proc.stdout.close()
+      if not proc.stderr.closed:
+        proc.stderr.close()
+    except OSError:
+      pass  # Clean Up
+
+
+def ExecWithStreamingOutput(args,
+                            env=None,
+                            no_exit=False,
+                            out_func=None,
+                            err_func=None,
+                            in_str=None,
+                            **extra_popen_kwargs):
+  """Emulates the os.exec* set of commands, but uses subprocess.
+
+  This executes the given command, waits for it to finish, and then exits this
+  process with the exit code of the child process. Allows realtime processing of
+  stderr and stdout from subprocess using threads.
+
+  Args:
+    args: [str], The arguments to execute.  The first argument is the command.
+    env: {str: str}, An optional environment for the child process.
+    no_exit: bool, True to just return the exit code of the child instead of
+      exiting.
+    out_func: str->None, a function to call with each line of the stdout of the
+      executed process. This can be e.g. log.file_only_logger.debug or
+      log.out.write.
+    err_func: str->None, a function to call with each line of the stderr of
+      the executed process. This can be e.g. log.file_only_logger.debug or
+      log.err.write.
+    in_str: bytes or str, input to send to the subprocess' stdin.
+    **extra_popen_kwargs: Any additional kwargs will be passed through directly
+      to subprocess.Popen
+
+  Returns:
+    int, The exit code of the child if no_exit is True, else this method does
+    not return.
+
+  Raises:
+    PermissionError: if user does not have execute permission for cloud sdk bin
+    files.
+    InvalidCommandError: if the command entered cannot be found.
+  """
+  log.debug('Executing command: %s', args)
+  # We use subprocess instead of execv because windows does not support process
+  # replacement.  The result of execv on windows is that a new processes is
+  # started and the original is killed.  When running in a shell, the prompt
+  # returns as soon as the parent is killed even though the child is still
+  # running.  subprocess waits for the new process to finish before returning.
+  env = _GetToolEnv(env=env)
   process_holder = _ProcessHolder()
   with _ReplaceSignal(signal.SIGTERM, process_holder.Handler):
     with _ReplaceSignal(signal.SIGINT, process_holder.Handler):
-      if out_func:
-        extra_popen_kwargs['stdout'] = subprocess.PIPE
-      if err_func:
-        extra_popen_kwargs['stderr'] = subprocess.PIPE
+      out_handler_func = out_func or log.Print
+      err_handler_func = err_func or log.status.Print
       if in_str:
         extra_popen_kwargs['stdin'] = subprocess.PIPE
       try:
@@ -319,7 +472,34 @@ def Exec(args,
           # that happens.
           # https://bugs.python.org/issue19264
           args = [encoding.Encode(a) for a in args]
-        p = subprocess.Popen(args, env=env, **extra_popen_kwargs)
+        p = subprocess.Popen(args, env=env, stderr=subprocess.PIPE,
+                             stdout=subprocess.PIPE, **extra_popen_kwargs)
+
+        if in_str:
+          in_str = six.text_type(in_str).encode('utf-8')
+          try:
+            p.stdin.write(in_str)
+            p.stdin.close()
+          except OSError as exc:
+            if (exc.errno == errno.EPIPE or
+                exc.errno == errno.EINVAL):
+              pass  # Obey same conventions as subprocess.communicate()
+            else:
+              _KillProcIfRunning(p)
+              raise OutputStreamProcessingException(exc)
+
+        try:
+          with parallel.GetPool(2) as pool:
+            std_out_future = pool.ApplyAsync(_ProcessStreamHandler,
+                                             (p, False, out_handler_func))
+            std_err_future = pool.ApplyAsync(_ProcessStreamHandler,
+                                             (p, True, err_handler_func))
+            std_out_future.Get()
+            std_err_future.Get()
+        except Exception as e:
+          _KillProcIfRunning(p)
+          raise  OutputStreamProcessingException(e)
+
       except OSError as err:
         if err.errno == errno.EACCES:
           raise PermissionError(err.strerror)
@@ -332,17 +512,8 @@ def Exec(args,
         # This covers the small possibility that process_holder handled a
         # signal when the process was starting but not yet set to
         # process_holder.process.
-        if p.poll() is None:
-          p.terminate()
+        _KillProcIfRunning(p)
 
-      if isinstance(in_str, six.text_type):
-        in_str = in_str.encode('utf-8')
-      stdout, stderr = list(map(encoding.Decode, p.communicate(input=in_str)))
-
-      if out_func:
-        out_func(stdout)
-      if err_func:
-        err_func(stderr)
       ret_val = p.returncode
 
   if no_exit and process_holder.signum is None:
@@ -417,7 +588,7 @@ def KillSubprocess(p):
   if platforms.OperatingSystem.Current() == platforms.OperatingSystem.WINDOWS:
     # Consume stdout so it doesn't show in the shell
     taskkill_process = subprocess.Popen(
-        ['taskkill', '/F', '/T', '/PID', str(p.pid)],
+        ['taskkill', '/F', '/T', '/PID', six.text_type(p.pid)],
         stdout=subprocess.PIPE,
         stderr=subprocess.PIPE)
     (stdout, stderr) = taskkill_process.communicate()
