@@ -54,6 +54,7 @@ class Import(base.CreateCommand):
     instances_flags.AddPrivateNetworkIpArgs(parser)
     instances_flags.AddDeletionProtectionFlag(parser)
     instances_flags.AddNetworkTierArgs(parser, instance=True)
+    instances_flags.AddNoAddressArg(parser)
     labels_util.AddCreateLabelsFlags(parser)
     daisy_utils.AddCommonDaisyArgs(parser)
     daisy_utils.AddExtraCommonDaisyArgs(parser)
@@ -169,7 +170,8 @@ class Import(base.CreateCommand):
         output_filter=_OUTPUT_FILTER,
         release_track=
         self.ReleaseTrack().id.lower() if self.ReleaseTrack() else None,
-        hostname=getattr(args, 'hostname', None)
+        hostname=getattr(args, 'hostname', None),
+        no_address=getattr(args, 'no_address', False),
     )
 
 
