@@ -35,7 +35,9 @@
 #include <endian.h>
 #define __BYTE_ORDER BYTE_ORDER
 #if BYTE_ORDER == LITTLE_ENDIAN
+#ifndef __LITTLE_ENDIAN__
 #define __LITTLE_ENDIAN__
+#endif /* __LITTLE_ENDIAN__ */
 #else
 #define __BIG_ENDIAN__
 #endif/* BYTE_ORDER */
@@ -179,6 +181,8 @@
 #define NDPI_JABBER_FT_TIMEOUT				         5
 #define NDPI_SOULSEEK_CONNECTION_IP_TICK_TIMEOUT               600
 
+#include "ndpi_config.h"	/* To have access to NDPI_ENABLE_DEBUG_MESSAGES */
+
 #ifdef NDPI_ENABLE_DEBUG_MESSAGES
  #define NDPI_LOG(proto, m, log_level, args...)		                                 \
   {								                         \
@@ -187,7 +191,7 @@
       (*(mod->ndpi_debug_printf))(proto, mod, log_level, __FILE__, __FUNCTION__, __LINE__, args); \
   }
 
- /* We must define NDPI_CURRENT_PROTO before include ndpi_main.h !!! 
+ /* We must define NDPI_CURRENT_PROTO before include ndpi_main.h !!!
   *
   * #include "ndpi_protocol_ids.h"
   * #define NDPI_CURRENT_PROTO NDPI_PROTOCOL_XXXX
@@ -196,7 +200,7 @@
   */
 
  #ifndef NDPI_CURRENT_PROTO
- #define NDPI_CURRENT_PROTO NDPI_PROTO_UNKNOWN
+ #define NDPI_CURRENT_PROTO NDPI_PROTOCOL_UNKNOWN
  #endif
 
  #define NDPI_LOG_ERR(mod, args...)		                                 \
@@ -274,6 +278,12 @@
 #define NDPI_BITMASK_SET_ALL(a)   NDPI_ONE(&a)
 #define NDPI_BITMASK_SET(a, b)    { memcpy(&a, &b, sizeof(NDPI_PROTOCOL_BITMASK)); }
 
+#define NDPI_SET_BIT(num, n)    num |= 1UL << ( n )
+#define NDPI_CLR_BIT(num, n)    num &= ~(1UL << ( n ))
+#define NDPI_CLR_BIT(num, n)    num &= ~(1UL << ( n ))
+#define NDPI_ISSET_BIT(num, n)  (num & (1UL << ( n )))
+#define NDPI_ZERO_BIT(num)      num = 0
+
 /* this is a very very tricky macro *g*,
  * the compiler will remove all shifts here if the protocol is static...
  */
@@ -304,10 +314,10 @@
 #define NDPI_ICMPV6_PROTOCOL_TYPE  0x3a
 
 /* the get_uXX will return raw network packet bytes !! */
-#define get_u_int8_t(X,O)  (*(u_int8_t *)(((u_int8_t *)X) + O))
-#define get_u_int16_t(X,O)  (*(u_int16_t *)(((u_int8_t *)X) + O))
-#define get_u_int32_t(X,O)  (*(u_int32_t *)(((u_int8_t *)X) + O))
-#define get_u_int64_t(X,O)  (*(u_int64_t *)(((u_int8_t *)X) + O))
+#define get_u_int8_t(X,O)   (*(u_int8_t  *)((&(((u_int8_t *)X)[O]))))
+#define get_u_int16_t(X,O)  (*(u_int16_t *)((&(((u_int8_t *)X)[O]))))
+#define get_u_int32_t(X,O)  (*(u_int32_t *)((&(((u_int8_t *)X)[O]))))
+#define get_u_int64_t(X,O)  (*(u_int64_t *)((&(((u_int8_t *)X)[O]))))
 
 /* new definitions to get little endian from network bytes */
 #define get_ul8(X,O) get_u_int8_t(X,O)
@@ -328,14 +338,14 @@
 #define match_first_bytes(payload,st) (memcmp((payload),(st),(sizeof(st)-1))==0)
 
 #if defined(WIN32) && !defined(snprintf)
-#define snprintf _snprintf 
+#define snprintf _snprintf
 #endif
 
 #define NDPI_MAX_DNS_REQUESTS                   16
 #define NDPI_MIN_NUM_STUN_DETECTION             8
 
 #define NDPI_MAJOR                              3
-#define NDPI_MINOR                              0
+#define NDPI_MINOR                              4
 #define NDPI_PATCH                              0
 
 /* IMPORTANT: order according to its severity */
@@ -343,6 +353,10 @@
 #define NDPI_CIPHER_WEAK                        1
 #define NDPI_CIPHER_INSECURE                    2
 
+#define NDPI_OPTIMAL_HLL_NUM_BUCKETS           16
+
+#define NDPI_MAX_NUM_TLS_APPL_BLOCKS            8
+
 #ifdef __APPLE__
 
 #include <libkern/OSByteOrder.h>
