@@ -55,12 +55,12 @@ extern "C" {
 #define PKG_STATIC_NAME	"pkg-static"
 #endif
 
-#define PKGVERSION "1.15.10"
+#define PKGVERSION "1.16.3"
 
 /* PORTVERSION equivalent for proper pkg-static->ports-mgmt/pkg
  * version comparison in pkgdb_query_newpkgversion() */
 
-#define PKG_PORTVERSION "1.15.10"
+#define PKG_PORTVERSION "1.16.3"
 
 /* The OS major version at the time of compilation */
 #ifdef __FreeBSD__
@@ -85,10 +85,10 @@ extern "C" {
 #endif
 
 /* Special exit status for worker processes indicating that a restart
- * is desired -- eg. after a child has updated pkg(8) itself.  Don't
- * clash with any of the sysexits values */
+ * is desired -- eg. after a child has updated pkg(8) itself.
+ */
 
-#define EX_NEEDRESTART	(EX__MAX + 1)
+#define EX_NEEDRESTART	4
 
 struct pkg;
 struct pkg_dep;
@@ -753,6 +753,7 @@ void pkg_manifest_parser_free(struct pkg_manifest_parser *p);
 #define PKG_MANIFEST_EMIT_PRETTY (0x1 << 2)
 #define PKG_MANIFEST_EMIT_JSON (0x1 << 3)
 #define PKG_MANIFEST_EMIT_UCL (0x1 << 4)
+#define PKG_MANIFEST_EMIT_LOCAL_METADATA (0x1 << 5)
 
 /**
  * Emit a manifest according to the attributes of pkg.
@@ -1291,6 +1292,9 @@ typedef enum {
 	PKG_EVENT_CLEANUP_CALLBACK_REGISTER,
 	PKG_EVENT_CLEANUP_CALLBACK_UNREGISTER,
 	PKG_EVENT_CONFLICTS,
+	PKG_EVENT_TRIGGERS_BEGIN,
+	PKG_EVENT_TRIGGER,
+	PKG_EVENT_TRIGGERS_FINISHED
 } pkg_event_t;
 
 struct pkg_event {
@@ -1456,6 +1460,10 @@ struct pkg_event {
 			struct pkg *p2;
 			const char *path;
 		} e_conflicts;
+		struct {
+			char *name;
+			bool cleanup;
+		} e_trigger;
 	};
 };
 
@@ -1600,29 +1608,6 @@ int pkg_asprintf(char **ret, const char * restrict format, ...);
  */
 int pkg_vasprintf(char **ret, const char * restrict format, va_list ap);
 
-#if defined(UTSTRING_H)
-/**
- * store data from pkg into sbuf as indicated by the format code format.
- * @param sbuf contains the result
- * @param ... Varargs list of struct pkg etc. supplying the data
- * @param format String with embedded %-escapes indicating what to output
- * @return count of the number of characters in the result
- */
-UT_string *pkg_utstring_printf(UT_string * restrict sbuf,
-	const char * restrict format, ...);
-
-/**
- * store data from pkg into sbuf as indicated by the format code format.
- * This is the core function called by all the other pkg_printf() family.
- * @param sbuf contains the result
- * @param ap Arglist with struct pkg etc. supplying the data
- * @param format String with embedded %-escapes indicating what to output
- * @return count of the number of characters in the result
- */
-UT_string *pkg_utstring_vprintf(UT_string * restrict sbuf,
-	const char * restrict format, va_list ap);
-#endif
-
 bool pkg_has_message(struct pkg *p);
 bool pkg_is_locked(const struct pkg * restrict p);
 
@@ -1644,47 +1629,6 @@ bool pkg_is_locked(const struct pkg * restrict p);
  */
 #define PKG_FILE_CKSUM_CHARS 10
 
-struct pkg_audit;
-
-/**
- * Creates new pkg_audit structure
- */
-struct pkg_audit * pkg_audit_new(void);
-
-/**
- * Fetch and extract audit file from url `src` to the file `dest`
- * If no update is required then this function returns `EPKG_UPTODATE`
- * @return error code
- */
-int pkg_audit_fetch(const char *src, const char *dest);
-
-/**
- * Load audit file into memory
- * @return error code
- */
-int pkg_audit_load(struct pkg_audit *audit, const char *fname);
-
-/**
- * Process loaded audit structure.
- * Can and should be executed after cap_enter(3) or another sandboxing call
- * @return error code
- */
-int pkg_audit_process(struct pkg_audit *audit);
-
-#if defined(UTSTRING_H)
-/**
- * Check whether `pkg` is vulnerable against processed `audit` structure.
- * If a package is vulnerable, then `result` is set to sbuf describing the
- * vulnerability. If `quiet` is true, then this function produces reduced output
- * just returning a name of vulnerable package.
- * It's caller responsibility to free `result` after use
- * @return true and `*result` is set if a package is vulnerable
- */
-bool pkg_audit_is_vulnerable(struct pkg_audit *audit, struct pkg *pkg,
-		bool quiet, UT_string **result, int *affected);
-#endif
-
-void pkg_audit_free (struct pkg_audit *audit);
 char *pkg_utils_tokenize(char **);
 int pkg_utils_count_spaces(const char *);
 int pkg_add_port(struct pkgdb *db, struct pkg *pkg, const char *root, \
@@ -1707,6 +1651,7 @@ void pkg_create_set_overwrite(struct pkg_create *, bool);
 void pkg_create_set_rootdir(struct pkg_create *, const char *);
 void pkg_create_set_output_dir(struct pkg_create *, const char *);
 void pkg_create_set_timestamp(struct pkg_create *, time_t);
+void pkg_create_set_expand_manifest(struct pkg_create *, bool);
 int pkg_create(struct pkg_create *, const char *, const char *, bool);
 int pkg_create_i(struct pkg_create *, struct pkg *, bool);
 /* deprecated */
