@@ -14,7 +14,7 @@ extern "C" {
 #ifndef RTRLIB_H
 #define RTRLIB_H
 
-#define RTRLIB_HAVE_LIBSSH
+/* #undef RTRLIB_HAVE_LIBSSH */
 #define RTRLIB_VERSION_MAJOR 0
 #define RTRLIB_VERSION_MINOR 6
 #define RTRLIB_VERSION_PATCH 3
