@@ -10,7 +10,7 @@ use strict;
 use warnings;
 our ( %Config, $VERSION );
 
-$VERSION = "5.030003";
+$VERSION = "5.032001";
 
 # Skip @Config::EXPORT because it only contains %Config, which we special
 # case below as it's not a function. @Config::EXPORT won't change in the
@@ -56,11 +56,11 @@ sub import {
     return;
 }
 
-die "$0: Perl lib version (5.30.3) doesn't match executable '$^X' version ($])"
+die "$0: Perl lib version (5.32.1) doesn't match executable '$^X' version ($])"
     unless $^V;
 
-$^V eq 5.30.3
-    or die sprintf "%s: Perl lib version (5.30.3) doesn't match executable '$^X' version (%vd)", $0, $^V;
+$^V eq 5.32.1
+    or die sprintf "%s: Perl lib version (5.32.1) doesn't match executable '$^X' version (%vd)", $0, $^V;
 
 
 sub FETCH {
@@ -84,7 +84,7 @@ sub AUTOLOAD {
 
 # tie returns the object, so the value returned to require will be true.
 tie %Config, 'Config', {
-    archlibexp => '/usr/local/lib/perl5/5.30/mach',
+    archlibexp => '/usr/local/lib/perl5/5.32/mach',
     archname => 'amd64-freebsd-thread-multi',
     cc => 'cc',
     d_readlink => 'define',
@@ -98,14 +98,14 @@ tie %Config, 'Config', {
     ldlibpthname => 'LD_LIBRARY_PATH',
     libpth => '/usr/lib /usr/local/lib /usr/lib/clang/8.0.1/lib /usr/lib',
     osname => 'freebsd',
-    osvers => '12.1--hbsd',
+    osvers => '12.1-release-p18-hbsd',
     path_sep => ':',
-    privlibexp => '/usr/local/lib/perl5/5.30',
+    privlibexp => '/usr/local/lib/perl5/5.32',
     scriptdir => '/usr/local/bin',
-    sitearchexp => '/usr/local/lib/perl5/site_perl/mach/5.30',
+    sitearchexp => '/usr/local/lib/perl5/site_perl/mach/5.32',
     sitelibexp => '/usr/local/lib/perl5/site_perl',
     so => 'so',
     useithreads => 'define',
     usevendorprefix => undef,
-    version => '5.30.3',
+    version => '5.32.1',
 };
