@@ -28,14 +28,15 @@
  *    POSSIBILITY OF SUCH DAMAGE.
  *
  */
+
 namespace OPNsense\AcmeClient\Api;
 
-use \OPNsense\Base\ApiMutableModelControllerBase;
-use \OPNsense\Core\Backend;
-use \OPNsense\Cron\Cron;
-use \OPNsense\Core\Config;
-use \OPNsense\Base\UIModelGrid;
-use \OPNsense\AcmeClient\AcmeClient;
+use OPNsense\Base\ApiMutableModelControllerBase;
+use OPNsense\Core\Backend;
+use OPNsense\Cron\Cron;
+use OPNsense\Core\Config;
+use OPNsense\Base\UIModelGrid;
+use OPNsense\AcmeClient\AcmeClient;
 
 /**
  * Class SettingsController
@@ -85,6 +86,8 @@ class SettingsController extends ApiMutableModelControllerBase
                     Config::getInstance()->save();
                     // Refresh the crontab
                     $backend->configdRun('template reload OPNsense/Cron');
+                    // (res)start daemon
+                    $backend->configdRun("cron restart");
                     $result['result'] = "new";
                     $result['uuid'] = $cron_uuid;
                 } else {
@@ -98,7 +101,7 @@ class SettingsController extends ApiMutableModelControllerBase
             ) {
                 // Get UUID, clean existin entry
                 $cron_uuid = (string)$mdlAcme->settings->UpdateCron;
-                $mdlAcme->settings->UpdateCron = null;
+                $mdlAcme->settings->UpdateCron = "";
                 $mdlCron = new Cron();
                 // Delete the cronjob item
                 if ($mdlCron->jobs->job->del($cron_uuid)) {
@@ -108,6 +111,8 @@ class SettingsController extends ApiMutableModelControllerBase
                     Config::getInstance()->save();
                     // Regenerate the crontab
                     $backend->configdRun('template reload OPNsense/Cron');
+                    // (res)start daemon
+                    $backend->configdRun("cron restart");
                     $result['result'] = "deleted";
                 } else {
                     $result['result'] = "unable to delete cron";
@@ -383,7 +388,7 @@ class SettingsController extends ApiMutableModelControllerBase
     }
 
     /**
-     * Check wether the Google Cloud plugin is installed.
+     * Check whether the Google Cloud plugin is installed.
      * @return array status action
      */
     public function getGcloudPluginStatusAction()
@@ -401,7 +406,7 @@ class SettingsController extends ApiMutableModelControllerBase
     }
 
     /**
-     * Check wether the BIND plugin is installed.
+     * Check whether the BIND plugin is installed.
      * @return array status action
      */
     public function getBindPluginStatusAction()
