@@ -28,9 +28,8 @@
 
 namespace OPNsense\IDS\Api;
 
-use Phalcon\Filter;
+use Phalcon\Filter\FilterFactory;
 use OPNsense\Base\ApiMutableModelControllerBase;
-use OPNsense\Base\Filters\QueryFilter;
 use OPNsense\Core\Backend;
 use OPNsense\Core\Config;
 use OPNsense\Base\UIModelGrid;
@@ -71,9 +70,10 @@ class SettingsController extends ApiMutableModelControllerBase
         if ($this->request->isPost()) {
             $this->sessionClose();
             // create filter to sanitize input data
-            $filter = new Filter();
-            $filter->add('query', new QueryFilter());
-
+            $filter = (new FilterFactory())->newInstance();
+            $filter->set('query', function ($value) {
+                return preg_replace("/[^0-9,a-z,A-Z, ,*,\-,_,.,\#]/", "", $value);
+            });
 
             // fetch query parameters (limit results to prevent out of memory issues)
             $itemsPerPage = $this->request->getPost('rowCount', 'int', 9999);
@@ -108,17 +108,11 @@ class SettingsController extends ApiMutableModelControllerBase
             foreach ($_POST as $key => $value) {
                 $key = $filter->sanitize($key, "string");
                 $value = $filter->sanitize($value, "string");
-                if (!in_array($key, ['current', 'rowCount', 'sort', 'searchPhrase', 'action'])) {
-                    $searchPhrase .= " {$key}/{$value} ";
+                if (!in_array($key, ['current', 'rowCount', 'sort', 'searchPhrase'])) {
+                    $searchPhrase .= " {$key}/\"{$value}\" ";
                 }
             }
 
-            // add filter for action
-            if ($this->request->getPost("action", "string", '') != "") {
-                $searchTag = $filter->sanitize($this->request->getPost('action'), "query");
-                $searchPhrase .= " installed_action/" . $searchTag . ' ';
-            }
-
             // request list of installed rules
             $backend = new Backend();
             $response = $backend->configdpRun("ids query rules", array($itemsPerPage,
@@ -130,13 +124,6 @@ class SettingsController extends ApiMutableModelControllerBase
             if ($data != null && array_key_exists("rows", $data)) {
                 $result = array();
                 $result['rows'] = $data['rows'];
-                // update rule status with own administration
-                foreach ($result['rows'] as &$row) {
-                    $row['enabled_default'] = $row['enabled'];
-                    $row['enabled'] = $this->getModel()->getRuleStatus($row['sid'], $row['enabled']);
-                    $row['action'] = $this->getModel()->getRuleAction($row['sid'], $row['action'], true);
-                }
-
                 $result['rowCount'] = empty($result['rows']) || !is_array($result['rows']) ? 0 : count($result['rows']);
                 $result['total'] = $data['total_rows'];
                 $result['parameters'] = $data['parameters'];
@@ -172,8 +159,6 @@ class SettingsController extends ApiMutableModelControllerBase
         if ($data != null && array_key_exists("rows", $data) && !empty($data['rows'])) {
             $row = $data['rows'][0];
             // set current enable status (default + registered offset)
-            $row['enabled_default'] = $row['enabled'];
-            $row['action_default'] = $row['action'];
             $row['enabled'] = $this->getModel()->getRuleStatus($row['sid'], $row['enabled']);
             $row['action'] = $this->getModel()->getRuleAction($row['sid'], $row['action']);
             //
@@ -194,7 +179,7 @@ class SettingsController extends ApiMutableModelControllerBase
                             substr($ref, 8), $item_html);
                         $item_html = str_replace("%ref%", "bugtraq " . substr($ref, 8), $item_html);
                     } elseif (substr($ref, 0, 4) == "cve,") {
-                        $item_html = str_replace("%url%", "http://cve.mitre.org/cgi-bin/cvename.cgi?name=" .
+                        $item_html = str_replace("%url%", "https://cve.mitre.org/cgi-bin/cvename.cgi?name=" .
                             substr($ref, 4), $item_html);
                         $item_html = str_replace("%ref%", substr($ref, 4), $item_html);
                     } elseif (substr($ref, 0, 7) == "nessus,") {
@@ -229,6 +214,12 @@ class SettingsController extends ApiMutableModelControllerBase
         $response = (new Backend())->configdRun("ids list rulemetadata");
         $data = json_decode($response, true);
         if ($data != null) {
+            $data['matched_policy'] = ['__manual__'];
+            foreach ($this->getModel()->policies->policy->iterateItems() as $policy) {
+                if (!empty((string)$policy->enabled) && !empty((string)$policy->description)) {
+                    $data['matched_policy'][] = (string)$policy->description;
+                }
+            }
             return $data;
         } else {
             return array();
@@ -271,8 +262,6 @@ class SettingsController extends ApiMutableModelControllerBase
                 // retrieve status from model
                 $fileNode = $this->getModel()->getFileNode($fileinfo['filename']);
                 $item['enabled'] = (string)$fileNode->enabled;
-                $item['filter'] = $fileNode->filter->getNodeData(); // filter (option list)
-                $item['filter_str'] = (string)$fileNode->filter; // filter current value
                 $result[] = $item;
             }
         }
@@ -451,12 +440,6 @@ class SettingsController extends ApiMutableModelControllerBase
                     $node = $this->getModel()->getFileNode($filename);
                     if ($enabled == "0" || $enabled == "1") {
                         $node->enabled = (string)$enabled;
-                    } elseif ($enabled == "drop") {
-                        $node->enabled = "1";
-                        $node->filter = "drop";
-                    } elseif ($enabled == "clear") {
-                        $node->enabled = "1";
-                        $node->filter = "";
                     } elseif ((string)$node->enabled == "1") {
                         $node->enabled = "0";
                     } else {
@@ -519,10 +502,7 @@ class SettingsController extends ApiMutableModelControllerBase
                     } else {
                         $new_state = 0;
                     }
-                    if ($ruleinfo['enabled_default'] == $new_state && $current_action == $ruleinfo['action_default']) {
-                        // if we're switching back to default, remove alter rule
-                        $this->getModel()->removeRule($sid);
-                    } elseif ($new_state == 1) {
+                    if ($new_state == 1) {
                         $this->getModel()->enableRule($sid)->action = $current_action;
                     } else {
                         $this->getModel()->disableRule($sid)->action = $current_action;
@@ -557,16 +537,7 @@ class SettingsController extends ApiMutableModelControllerBase
             $newAction = $this->request->getPost("action", "striptags", null);
             if (!empty($ruleinfo)) {
                 $mdlIDS = $this->getModel();
-                if (
-                    $ruleinfo['enabled_default'] == $ruleinfo['enabled'] &&
-                    $ruleinfo['action_default'] == $newAction
-                ) {
-                    // if we're switching back to default, remove alter rule
-                    $mdlIDS->removeRule($sid);
-                } else {
-                    $mdlIDS->setAction($sid, $newAction);
-                }
-
+                $mdlIDS->setAction($sid, $newAction);
                 $validations = $mdlIDS->validate();
                 if (!empty($validations)) {
                     $result['validations'] = $validations;
@@ -646,4 +617,166 @@ class SettingsController extends ApiMutableModelControllerBase
     {
         return $this->toggleBase("userDefinedRules.rule", $uuid, $enabled);
     }
+
+    /**
+     * Search policy
+     * @return array list of found user rules
+     * @throws \ReflectionException when not bound to model
+     */
+    public function searchPolicyAction()
+    {
+        return $this->searchBase("policies.policy", array("enabled", "prio", "description"), "description");
+    }
+
+    /**
+     * Update policy
+     * @param string $uuid internal id
+     * @return array save result + validation output
+     * @throws \Phalcon\Validation\Exception when field validations fail
+     * @throws \ReflectionException when not bound to model
+     */
+    public function setPolicyAction($uuid)
+    {
+        return $this->setBase("policy", "policies.policy", $uuid);
+    }
+
+    /**
+     * Add new policy
+     * @return array save result + validation output
+     * @throws \Phalcon\Validation\Exception when field validations fail
+     * @throws \ReflectionException when not bound to model
+     */
+    public function addPolicyAction()
+    {
+        return $this->addBase("policy", "policies.policy");
+    }
+
+    /**
+     * Get properties of a policy
+     * @param null|string $uuid user rule internal id
+     * @return array user defined properties
+     * @throws \ReflectionException when not bound to model
+     */
+    public function getPolicyAction($uuid = null)
+    {
+        return $this->getBase("policy", "policies.policy", $uuid);
+    }
+
+    /**
+     * Delete policy item
+     * @param string $uuid user rule internal id
+     * @return array save status
+     * @throws \Phalcon\Validation\Exception when field validations fail
+     * @throws \ReflectionException when not bound to model
+     */
+    public function delPolicyAction($uuid)
+    {
+        return  $this->delBase("policies.policy", $uuid);
+    }
+
+    /**
+     * Toggle policy by uuid (enable/disable)
+     * @param $uuid user defined rule internal id
+     * @param $enabled desired state enabled(1)/disabled(1), leave empty for toggle
+     * @return array save result
+     * @throws \Phalcon\Validation\Exception when field validations fail
+     * @throws \ReflectionException when not bound to model
+     */
+    public function togglePolicyAction($uuid, $enabled = null)
+    {
+        return $this->toggleBase("policies.policy", $uuid, $enabled);
+    }
+
+    /**
+     * Search policy rule adjustment
+     * @return array list of found user rules
+     * @throws \ReflectionException when not bound to model
+     */
+    public function searchPolicyRuleAction()
+    {
+        return $this->searchBase("rules.rule", array("sid", "enabled", "action"), "sid");
+    }
+
+    /**
+     * Update policy rule adjustment
+     * @param string $uuid internal id
+     * @return array save result + validation output
+     * @throws \Phalcon\Validation\Exception when field validations fail
+     * @throws \ReflectionException when not bound to model
+     */
+    public function setPolicyRuleAction($uuid)
+    {
+        return $this->setBase("rule", "rules.rule", $uuid);
+    }
+
+    /**
+     * Add new policy rule adjustment
+     * @return array save result + validation output
+     * @throws \Phalcon\Validation\Exception when field validations fail
+     * @throws \ReflectionException when not bound to model
+     */
+    public function addPolicyRuleAction()
+    {
+        return $this->addBase("rule", "rules.rule");
+    }
+
+    /**
+     * Get properties of a policy rule adjustment
+     * @param null|string $uuid internal id
+     * @return array user defined properties
+     * @throws \ReflectionException when not bound to model
+     */
+    public function getPolicyRuleAction($uuid = null)
+    {
+        return $this->getBase("rule", "rules.rule", $uuid);
+    }
+
+    /**
+     * Delete policy rule adjustment item
+     * @param string $uuid internal id
+     * @return array save status
+     * @throws \Phalcon\Validation\Exception when field validations fail
+     * @throws \ReflectionException when not bound to model
+     */
+    public function delPolicyRuleAction($uuid)
+    {
+        return  $this->delBase("rules.rule", $uuid);
+    }
+
+    /**
+     * Toggle policy rule adjustment by uuid (enable/disable)
+     * @param $uuid user internal id
+     * @param $enabled desired state enabled(1)/disabled(1), leave empty for toggle
+     * @return array save result
+     * @throws \Phalcon\Validation\Exception when field validations fail
+     * @throws \ReflectionException when not bound to model
+     */
+    public function togglePolicyRuleAction($uuid, $enabled = null)
+    {
+        return $this->toggleBase("rules.rule", $uuid, $enabled);
+    }
+
+    /**
+     * return then number of custom defined policy rules
+     */
+    public function checkPolicyRuleAction()
+    {
+        $result = ['status' => 'ok'];
+        $mdlIDS = $this->getModel();
+        $result['count'] = count(iterator_to_array($mdlIDS->rules->rule->iterateItems()));
+        if ($result['count'] > 100) {
+            // changing some rules by sid doesn't matter, a lot inflates the config.xml beyond reasonable limits.
+            $result['status'] = 'warning';
+            $result['message'] = sprintf(
+                gettext("We strongly advise to use policies instead of " .
+                "single rule based changes to limit the size of the configuration. " .
+                "A list of all manual changes can be revised in the policy editor (available %s here %s)"),
+                "<a href='/ui/ids/policy#rules'>",
+                "</a>"
+            );
+        }
+        // return unescaped content
+        $this->response->setContentType('application/json', 'UTF-8');
+        $this->response->setContent(json_encode($result));
+    }
 }
