@@ -28,9 +28,10 @@
 
 namespace OPNsense\Proxy\Api;
 
-use \OPNsense\Base\ApiMutableServiceControllerBase;
-use \OPNsense\Core\Backend;
-use \OPNsense\Proxy\Proxy;
+use OPNsense\Base\ApiMutableServiceControllerBase;
+use OPNsense\Base\UserException;
+use OPNsense\Core\Backend;
+use OPNsense\Proxy\Proxy;
 
 /**
  * Class ServiceController
@@ -56,6 +57,26 @@ class ServiceController extends ApiMutableServiceControllerBase
             (!empty((string)$mdlProxy->general->cache->local->enabled) != $prev_cache_active);
     }
 
+    private function hookStartErrorHandler($result)
+    {
+        if (preg_match('/__ok__$/', $result['response'])) {
+            $result['response'] = "ok";
+        } else {
+            throw new UserException($result['response'], gettext("proxy load error"));
+        }
+        return $result;
+    }
+
+    public function startAction()
+    {
+        return $this->hookStartErrorHandler(parent::startAction());
+    }
+
+    public function restartAction()
+    {
+        return $this->hookStartErrorHandler(parent::restartAction());
+    }
+
     /**
      * reload template only (for example PAC does not need to change squid configuration)
      * @return array
