@@ -187,6 +187,7 @@ class AuthenticationFactory
      */
     public function authenticate($service_name, $username, $password)
     {
+        openlog("audit", LOG_ODELAY, LOG_AUTH);
         $service = $this->getService($service_name);
         if ($service !== null) {
             $service->setUserName($username);
@@ -197,12 +198,13 @@ class AuthenticationFactory
                     if ($authenticator->authenticate($service->getUserName(), $password)) {
                         if ($service->checkConstraints()) {
                             syslog(LOG_NOTICE, sprintf(
-                                "user %s authenticated successfully for %s [using %s + %s]\n",
+                                "user %s authenticated successfully for %s [using %s + %s]",
                                 $username,
                                 $service_name,
                                 get_class($service),
                                 get_class($authenticator)
                             ));
+                            closelog();
                             return true;
                         } else {
                             // since checkConstraints() is defined on the service, who doesn't know about the
@@ -214,6 +216,7 @@ class AuthenticationFactory
                                 get_class($service),
                                 get_class($authenticator)
                             ));
+                            closelog();
                             return false;
                         }
                     } else {
@@ -229,12 +232,13 @@ class AuthenticationFactory
             }
         }
         syslog(LOG_WARNING, sprintf(
-            "user %s could not authenticate for %s. [using %s + %s]\n",
+            "user %s could not authenticate for %s. [using %s + %s]",
             $username,
             $service_name,
             !empty($service) ? get_class($service) : '-',
             !empty($authenticator) ? get_class($authenticator) : '-'
         ));
+        closelog();
         return false;
     }
 
