@@ -29,6 +29,7 @@
 namespace OPNsense\Auth;
 
 use OPNsense\Core\Config;
+use OPNsense\Core\Backend;
 
 /**
  * Class LDAP connector
@@ -193,7 +194,12 @@ class LDAP extends Base implements IAuthConnector
         $error_string = "";
         if ($this->ldapHandle !== false) {
             ldap_get_option($this->ldapHandle, LDAP_OPT_ERROR_STRING, $error_string);
-            syslog(LOG_ERR, sprintf($message . " [%s,%s]", $error_string, ldap_error($this->ldapHandle)));
+            $error_string = str_replace(array("\n","\r","\t"), ' ', $error_string);
+            syslog(LOG_ERR, sprintf($message . " [%s; %s]", $error_string, ldap_error($this->ldapHandle)));
+            if (!empty($error_string)) {
+                $this->lastAuthErrors['error'] = $error_string;
+            }
+            $this->lastAuthErrors['ldap_error'] = ldap_error($this->ldapHandle);
         } else {
             syslog(LOG_ERR, $message);
         }
@@ -279,6 +285,26 @@ class LDAP extends Base implements IAuthConnector
         if (!empty($config['ldap_port'])) {
             $this->ldapBindURL .= ":{$config['ldap_port']}";
         }
+        if (!empty($config['caseInSensitiveUsernames'])) {
+            $this->caseInSensitiveUsernames = true;
+        }
+    }
+
+    /**
+     * retrieve configuration options
+     * @return array
+     */
+    public function getConfigurationOptions()
+    {
+        $options = array();
+        $options["caseInSensitiveUsernames"] = array();
+        $options["caseInSensitiveUsernames"]["name"] = gettext("Match case insensitive");
+        $options["caseInSensitiveUsernames"]["help"] = gettext("Allow mixed case input when gathering local user settings.");
+        $options["caseInSensitiveUsernames"]["type"] = "checkbox";
+        $options["caseInSensitiveUsernames"]["validate"] = function ($value) {
+            return array();
+        };
+        return $options;
     }
 
     /**
@@ -375,8 +401,8 @@ class LDAP extends Base implements IAuthConnector
                                 'name' => $searchResults[$i][$ldapAttr][0],
                                 'dn' => $searchResults[$i]['dn']
                             );
-                            if (!empty($searchResults[$i]['displayName'][0])) {
-                                $user['fullname'] = $searchResults[$i]['displayName'][0];
+                            if (!empty($searchResults[$i]['displayname'][0])) {
+                                $user['fullname'] = $searchResults[$i]['displayname'][0];
                             } elseif (!empty($searchResults[$i]['cn'][0])) {
                                 $user['fullname'] = $searchResults[$i]['cn'][0];
                             } elseif (!empty($searchResults[$i]['name'][0])) {
@@ -410,11 +436,14 @@ class LDAP extends Base implements IAuthConnector
         if ($this->ldapHandle !== false) {
             $searchResults = $this->search("(|(ou=*)(cn=Users))");
             if ($searchResults !== false) {
+                $this->logLdapError("LDAP containers search result count: " . $searchResults["count"]);
                 for ($i = 0; $i < $searchResults["count"]; $i++) {
                     $result[] = $searchResults[$i]['dn'];
                 }
 
                 return $result;
+            } else {
+                   $this->logLdapError("LDAP containers search returned no results");
             }
         }
 
@@ -444,10 +473,10 @@ class LDAP extends Base implements IAuthConnector
             $cnf = Config::getInstance()->object();
             if (isset($cnf->system->group)) {
                 foreach ($cnf->system->group as $group) {
-                    $known_groups[] = (string)$group->name;
+                    $known_groups[] = strtolower((string)$group->name);
                     // when user is known, collect current groups
                     if ($user != null && in_array((string)$user->uid, (array)$group->member)) {
-                        $user_groups[] = (string)$group->name;
+                        $user_groups[] = strtolower((string)$group->name);
                     }
                 }
             }
@@ -456,12 +485,12 @@ class LDAP extends Base implements IAuthConnector
             $ldap_groups = array();
             foreach (explode("\n", $this->lastAuthProperties['memberof']) as $member) {
                 if (stripos($member, "cn=") === 0) {
-                    $ldap_groups[explode(",", substr($member, 3))[0]] = $member;
+                    $ldap_groups[strtolower(explode(",", substr($member, 3))[0])] = $member;
                 }
             }
             // list of enabled groups (all when empty), so we can ignore some local groups if needed
             if (!empty($this->ldapSyncMemberOfLimit)) {
-                $sync_groups = explode(",", $this->ldapSyncMemberOfLimit);
+                $sync_groups = explode(",", strtolower($this->ldapSyncMemberOfLimit));
             } else {
                 $sync_groups = $known_groups;
             }
@@ -483,10 +512,11 @@ class LDAP extends Base implements IAuthConnector
                 // if returned by ldap.
                 $cnf = Config::getInstance()->lock(true)->object();
                 foreach ($cnf->system->group as $group) {
-                    if (in_array((string)$group->name, $sync_groups)) {
+                    $lc_groupname = strtolower((string)$group->name);
+                    if (in_array($lc_groupname, $sync_groups)) {
                         if (
                             in_array((string)$user->uid, (array)$group->member)
-                              && empty($ldap_groups[(string)$group->name])
+                              && empty($ldap_groups[$lc_groupname])
                         ) {
                             unset($group->member[array_search((string)$user->uid, (array)$group->member)]);
                             syslog(LOG_NOTICE, sprintf(
@@ -496,19 +526,20 @@ class LDAP extends Base implements IAuthConnector
                             ));
                         } elseif (
                             !in_array((string)$user->uid, (array)$group->member)
-                              && !empty($ldap_groups[(string)$group->name])
+                              && !empty($ldap_groups[$lc_groupname])
                         ) {
                             syslog(LOG_NOTICE, sprintf(
                                 'User: policy change for %s link group %s [%s]',
                                 $username,
                                 (string)$group->name,
-                                $ldap_groups[(string)$group->name]
+                                $ldap_groups[$lc_groupname]
                             ));
                             $group->addChild('member', (string)$user->uid);
                         }
                     }
                 }
                 Config::getInstance()->save();
+                (new Backend())->configdpRun("auth user changed", array($username));
             }
         }
     }
@@ -538,6 +569,8 @@ class LDAP extends Base implements IAuthConnector
                 if ($result !== false && count($result) > 0) {
                     $user_dn = $result[0]['dn'];
                     $ldap_is_connected = $this->connect($this->ldapBindURL, $result[0]['dn'], $password);
+                } else {
+                    $this->lastAuthErrors['error'] = "User DN not found";
                 }
             }
         }
@@ -545,7 +578,7 @@ class LDAP extends Base implements IAuthConnector
         if ($ldap_is_connected) {
             $this->lastAuthProperties['dn'] = $user_dn;
             if ($this->ldapReadProperties) {
-                $sr = @ldap_read($this->ldapHandle, $user_dn, '(objectclass=*)');
+                $sr = @ldap_read($this->ldapHandle, $user_dn, '(objectclass=*)', ['*', 'memberOf']);
                 $info = @ldap_get_entries($this->ldapHandle, $sr);
                 if ($info['count'] != 0) {
                     foreach ($info[0] as $ldap_key => $ldap_value) {
