@@ -55,7 +55,7 @@ class Nextcloud extends Base implements IBackupProvider
                 "name" => "url",
                 "type" => "text",
                 "label" => gettext("URL"),
-                "help" => gettext("The Base URL to Nextcloud. For example: https://cloud.example.com"),
+                "help" => gettext("The Base URL to Nextcloud without trailing slash. For example: https://cloud.example.com"),
                 "value" => null
             ),
             array(
@@ -82,7 +82,7 @@ class Nextcloud extends Base implements IBackupProvider
             array(
                 "name" => "backupdir",
                 "type" => "text",
-                "label" => gettext("Directory Name"),
+                "label" => gettext("Directory Name without leading slash, starting from user's root"),
                 "value" => 'OPNsense-Backup'
             )
         );
@@ -145,12 +145,15 @@ class Nextcloud extends Base implements IBackupProvider
             if (!empty($crypto_password)) {
                 $confdata = $this->encrypt($confdata, $crypto_password);
             }
+            // Check if destination directory exists, create (full path) if not
             try {
                 $internal_username = $this->getInternalUsername($url, $username, $password);
-                $directories = $this->listFiles($url, $username, $password, $internal_username, '/');
-                if (!in_array("/$backupdir/", $directories)) {
-                    $this->create_directory($url, $username, $password, $internal_username, $backupdir);
-                }
+                $this->create_directory($url, $username, $password, $internal_username, $backupdir);
+            } catch (\Exception $e) {
+                return array();
+            }
+
+            try {
                 $this->upload_file_content(
                     $url,
                     $username,
@@ -191,7 +194,7 @@ class Nextcloud extends Base implements IBackupProvider
             $username,
             $password,
             'PROPFIND',
-            "Error while fetching filelist from Nextcloud"
+            "Error while fetching filelist from Nextcloud '{$directory}' path"
         );
         // workaround - simplexml seems to be broken when using namespaces - remove them.
         $xml = simplexml_load_string(str_replace(['<d:', '</d:'], ['<', '</'], $result['response']));
@@ -238,7 +241,7 @@ class Nextcloud extends Base implements IBackupProvider
     }
 
     /**
-     * create new remote directory
+     * create new remote directory if doesn't exist
      * @param string $url remote location
      * @param string $username remote user
      * @param string $password password to use
@@ -247,8 +250,27 @@ class Nextcloud extends Base implements IBackupProvider
      */
     public function create_directory($url, $username, $password, $internal_username, $backupdir)
     {
+        $parent_path = dirname($backupdir);
+        try {
+            $directories = $this->listFiles($url, $username, $password, $internal_username, "/{$parent_path}");
+        } catch (\Exception $e) {
+            if ($backupdir == ".") {
+                // We cannot create root, if we reached here there's some other problem
+                syslog(LOG_ERR, "Check Nextcloud configuration parameters");
+                return false;
+            }
+            // If error assume dir doesn't exist. Create parent folder
+            if ($this->create_directory($url, $username, $password, $internal_username, $parent_path) === false) {
+                throw new \Exception();
+            }
+        }
+        // if path exists ok
+        if (in_array("/{$backupdir}/", $directories)) {
+            return;
+        }
+
         $this->curl_request(
-            $url . "/remote.php/dav/files/$internal_username/$backupdir",
+            $url . "/remote.php/dav/files/{$internal_username}/{$backupdir}",
             $username,
             $password,
             'MKCOL',
