@@ -123,8 +123,12 @@ class FilterRule extends Rule
             // reply-to gateway set, when found map to reply attribute, otherwise skip keyword
             if (!empty($this->gatewayMapping[$rule['reply-to']])) {
                 $if = $this->gatewayMapping[$rule['reply-to']]['interface'];
-                $gw = $this->gatewayMapping[$rule['reply-to']]['gateway'];
-                $rule['reply'] = "reply-to ( {$if} {$gw} ) ";
+                if (!empty($this->gatewayMapping[$rule['reply-to']]['gateway'])) {
+                    $gw = $this->gatewayMapping[$rule['reply-to']]['gateway'];
+                    $rule['reply'] = "reply-to ( {$if} {$gw} ) ";
+                } else {
+                    $rule['reply'] = "reply-to {$if} ";
+                }
             }
         } elseif (!isset($rule['disablereplyto']) && $rule['direction'] != 'any') {
             $proto = $rule['ipprotocol'];
@@ -205,7 +209,7 @@ class FilterRule extends Rule
                 }
             }
             // restructure state settings for easier output parsing
-            if (!empty($rule['statetype']) && $rule['type'] == 'pass') {
+            if (!empty($rule['statetype']) && ($rule['type'] == 'pass' || empty($rule['type']))) {
                 $rule['state'] = array('type' => 'keep', 'options' => array());
                 switch ($rule['statetype']) {
                     case 'none':
