@@ -10,8 +10,10 @@ for directory in ${ACME_DIRS}; do
     chmod -R 750 ${directory}
 done
 
-if [ ! -L /var/etc/acme-client/home/dns_opnsense.sh ]; then
-    ln -s /usr/local/opnsense/scripts/OPNsense/AcmeClient/dns_opnsense.sh /var/etc/acme-client/home/dns_opnsense.sh
+# Remove symlink in order to use upstream version
+# see https://github.com/opnsense/plugins/pull/1888
+if [ -L /var/etc/acme-client/home/dns_opnsense.sh ]; then
+    unlink /var/etc/acme-client/home/dns_opnsense.sh
 fi
 
 # Setting owner and mode for base and immediate children (non recursive)
