@@ -1,5 +1,5 @@
 """
-    Copyright (c) 2017-2019 Ad Schellevis <ad@opnsense.org>
+    Copyright (c) 2017-2021 Ad Schellevis <ad@opnsense.org>
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -35,6 +35,8 @@ import dns.resolver
 import syslog
 from hashlib import md5
 from . import geoip
+from . import net_wildcard_iterator
+from .arpcache import ArpCache
 
 class Alias(object):
     def __init__(self, elem, known_aliases=[], ttl=-1, ssl_no_verify=False, timeout=120):
@@ -90,10 +92,18 @@ class Alias(object):
             :return: boolean
         """
         address = address.strip()
-        if address.find('/') > -1:
+        if address.find('/') > -1 and not address.split('/')[-1].isdigit():
+            # wildcard netmask
+            for idx, item in enumerate(net_wildcard_iterator(address.lstrip('!'))):
+                if idx > 65535:
+                    # overflow
+                    syslog.syslog(syslog.LOG_ERR, 'alias table %s overflow' % self._name)
+                    break
+                yield "!%s" % item if address.startswith('!') else str(item)
+        elif address.find('/') > -1:
             # provided address could be a network
             try:
-                ipaddress.ip_network(str(address), strict=False)
+                ipaddress.ip_network(str(address.lstrip('!')), strict=False)
                 yield address
                 return
             except (ipaddress.AddressValueError, ValueError):
@@ -102,13 +112,14 @@ class Alias(object):
             # check if address is an ipv4/6 address or range
             try:
                 tmp = str(address).split('-')
-                addr1 = ipaddress.ip_address(tmp[0])
                 if len(tmp) > 1:
+                    addr1 = ipaddress.ip_address(tmp[0])
                     # address range (from-to)
                     addr2 = ipaddress.ip_address(tmp[1])
                     for addr in ipaddress.summarize_address_range(addr1, addr2):
                         yield str(addr)
                 else:
+                    ipaddress.ip_address(tmp[0].lstrip('!'))
                     yield address
                 return
             except (ipaddress.AddressValueError, ValueError):
@@ -147,9 +158,7 @@ class Alias(object):
                 # only handle content if response is correct
                 req.raw.decode_content = True
                 lines = req.raw.read().decode().splitlines()
-                if len(lines) > 100:
-                    # when larger alias lists are downloaded, make sure we log before handling.
-                    syslog.syslog(syslog.LOG_ERR, 'fetch alias url %s (lines: %s)' % (url, len(lines)))
+                syslog.syslog(syslog.LOG_NOTICE, 'fetch alias url %s (lines: %s)' % (url, len(lines)))
                 for line in lines:
                     raw_address = re.split(r'[\s,;|#]+', line)[0]
                     if raw_address and not raw_address.startswith('//'):
@@ -269,6 +278,8 @@ class Alias(object):
             return self._fetch_url
         elif self._type == 'geoip':
             return self._fetch_geo
+        elif self._type == 'mac':
+            return ArpCache().iter_addresses
         else:
             return None
 
