@@ -67,7 +67,7 @@ class RuleCache(object):
             rule_filename = os.path.basename(filename)
             for rule in self.list_rules(filename):
                 if rule['metadata']:
-                    rule['rulesets'] = rule_filename
+                    rule['metadata']['rulesets'] = rule_filename
                     rule_orig_action = rule['metadata']['action']
                     if 'enabled' not in rule['metadata'] or not rule['metadata']['enabled']:
                         rule['metadata']['action'] = 'disable'
@@ -82,7 +82,14 @@ class RuleCache(object):
                                     # step into metadata
                                     src_pointer = src_pointer['metadata']
                                     src_section = section_name.split('metadata_', 1)[1]
-                                rule_data = src_pointer[src_section] if src_section in src_pointer else None
+
+                                if  src_section in src_pointer:
+                                    rule_data = src_pointer[src_section]
+                                elif src_section in rule['metadata']:
+                                    # metadata field is actually a rule field (category)
+                                    rule_data = rule['metadata'][src_section]
+                                else:
+                                    rule_data = None
                                 if rule_data not in configured_policies[policy_id][section_name]:
                                     is_matched = False
                         if is_matched:
@@ -91,7 +98,8 @@ class RuleCache(object):
                     if policy_match:
                         local_change = {
                             'mtime': policy_config_mtime,
-                            'policy_id': configured_policies[policy_match]['__policy_id__']
+                            'policy_id': configured_policies[policy_match]['__policy_id__'],
+                            'policy': configured_policies[policy_match]['__policy_description__']
                         }
                         local_change['action'] = rule_orig_action
                         if configured_policies[policy_match]['__target_action__'] == 'disable':
@@ -109,11 +117,13 @@ class RuleCache(object):
             cnf = ConfigParser()
             cnf.read(rule_config_fn)
             for section in cnf.sections():
-                if section[0:5] == 'rule_':
-                    sid = section[5:]
-                    rule_updates[sid] = {'mtime': policy_config_mtime, 'policy_id': None}
+                if section[0:5] == 'rule_' and cnf.has_option(section, 'sid'):
+                    sid = cnf.get(section, 'sid')
+                    # mark rule policies as __manual__ so we can filter them easily
+                    rule_updates[sid] = {'mtime': policy_config_mtime, 'policy_id': None, 'policy': "__manual__"}
                     for rule_item in cnf.items(section):
                         rule_updates[sid][rule_item[0]] = rule_item[1]
+                    rule_updates[sid]['enabled'] = rule_updates[sid]['enabled'] == '1'
         return rule_updates
 
     @staticmethod
@@ -310,7 +320,7 @@ class RuleCache(object):
                 cur.execute(
                     'insert into local_rule_changes(sid, action, enabled, last_mtime, policy) values (?,?,?,?,?)',
                     (sid, local_changes[sid]['action'], local_changes[sid]['enabled'], local_changes[sid]['mtime'],
-                     local_changes[sid]['policy_id'])
+                     local_changes[sid]['policy'])
                 )
             db.commit()
             # release lock
@@ -334,7 +344,7 @@ class RuleCache(object):
             sql_parameters = {}
             sql_filters = []
             prop_values = []
-            rule_search_fields = ['msg', 'sid', 'source', 'installed_action', 'installed_status']
+            rule_search_fields = ['msg', 'sid', 'source', 'action', 'status', 'matched_policy']
             for filtertag in shlex.split(filter_txt):
                 fieldnames, searchcontent = filtertag.split('/', maxsplit=1)
                 sql_item = []
@@ -364,13 +374,17 @@ class RuleCache(object):
 
             sql = """select *
                      from (
-                         select rules.*,
-                                case when rc.action is null then rules.action else rc.action end installed_action,
+                         select rules.sid, rules.rev, rules.gid, rules.msg, rules.reference, rules.source,
+                                case when rc.enabled is null then rules.enabled else rc.enabled end enabled,
+                                case when rc.action is null then rules.action else rc.action end action,
                                 case
                                     when case when rc.enabled is null then rules.enabled else rc.enabled end = true
                                     then 'enabled'
                                     else  'disabled'
-                                end installed_status
+                                end status,
+                                rules.enabled enabled_default,
+                                rules.action action_default,
+                                rc.policy matched_policy
                          from rules
                   """
 
@@ -440,7 +454,7 @@ class RuleCache(object):
         if os.path.exists(self.cachefile):
             db = sqlite3.connect(self.cachefile)
             cur = db.cursor()
-            cur.execute('SELECT property, value FROM metadata_histogram order by property, value')
+            cur.execute("SELECT property, value FROM metadata_histogram")
             for record in cur.fetchall():
                 if record[0] not in result:
                     result[record[0]] = list()
