@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 """
-Copyright (c) 2014-2020 Maltrail developers (https://github.com/stamparm/maltrail/)
+Copyright (c) 2014-2021 Maltrail developers (https://github.com/stamparm/maltrail/)
 See the file 'LICENSE' for copying permission
 """
 from __future__ import print_function
@@ -29,7 +29,6 @@ from core.common import get_regex
 from core.common import ipcat_lookup
 from core.common import worst_asns
 from core.compat import xrange
-from core.datatype import LRUDict
 from core.enums import HTTP_HEADER
 from core.settings import config
 from core.settings import CONTENT_EXTENSIONS_EXCLUSIONS
@@ -40,7 +39,6 @@ from core.settings import HTML_DIR
 from core.settings import HTTP_TIME_FORMAT
 from core.settings import IS_WIN
 from core.settings import MAX_NOFILE
-from core.settings import MAX_RESULT_CACHE_ENTRIES
 from core.settings import NAME
 from core.settings import PING_RESPONSE
 from core.settings import SESSION_COOKIE_NAME
@@ -70,7 +68,8 @@ try:
 except:
     pass
 
-_result_cache = LRUDict(MAX_RESULT_CACHE_ENTRIES)
+_fail2ban_cache = None
+_fail2ban_key = None
 
 def start_httpd(address=None, port=None, join=False, pem=None):
     """
@@ -91,15 +90,28 @@ def start_httpd(address=None, port=None, join=False, pem=None):
 
     class SSLThreadingServer(ThreadingServer):
         def __init__(self, server_address, pem, HandlerClass):
-            import OpenSSL  # python-openssl
+            if six.PY2:
+                import OpenSSL  # pyopenssl
 
-            ThreadingServer.__init__(self, server_address, HandlerClass)
-            ctx = OpenSSL.SSL.Context(OpenSSL.SSL.TLSv1_2_METHOD)
-            ctx.use_privatekey_file(pem)
-            ctx.use_certificate_file(pem)
-            self.socket = OpenSSL.SSL.Connection(ctx, socket.socket(self.address_family, self.socket_type))
-            self.server_bind()
-            self.server_activate()
+                ThreadingServer.__init__(self, server_address, HandlerClass)
+                for method in ("TLSv1_2_METHOD", "TLSv1_1_METHOD", "TLSv1_METHOD", "TLS_METHOD", "SSLv23_METHOD", "SSLv2_METHOD"):
+                    if hasattr(OpenSSL.SSL, method):
+                        ctx = OpenSSL.SSL.Context(getattr(OpenSSL.SSL, method))
+                        break
+                ctx.use_privatekey_file(pem)
+                ctx.use_certificate_file(pem)
+                self.socket = OpenSSL.SSL.Connection(ctx, socket.socket(self.address_family, self.socket_type))
+                self.server_bind()
+                self.server_activate()
+            else:
+                import ssl
+
+                ThreadingServer.__init__(self, server_address, ReqHandler)
+                ctx = ssl.SSLContext(ssl.PROTOCOL_TLS_SERVER)
+                ctx.load_cert_chain(pem, pem)
+                self.socket = ctx.wrap_socket(socket.socket(self.address_family, self.socket_type), server_side=True)
+                self.server_bind()
+                self.server_activate()
 
         def shutdown_request(self, request):
             try:
@@ -141,10 +153,10 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                     path = "%s.html" % path
 
                 if any((config.IP_ALIASES,)) and self.path.split('?')[0] == "/js/main.js":
-                    content = open(path, "rb").read()
+                    content = open(path, 'r').read()
                     content = re.sub(r"\bvar IP_ALIASES =.+", "var IP_ALIASES = {%s};" % ", ".join('"%s": "%s"' % (_.split(':', 1)[0].strip(), _.split(':', 1)[-1].strip()) for _ in config.IP_ALIASES), content)
-                    self.send_response(_http_client.OK)
-                elif ".." not in os.path.relpath(path, HTML_DIR) and os.path.isfile(path) and (extension not in DISABLED_CONTENT_EXTENSIONS or os.path.split(path)[-1] in CONTENT_EXTENSIONS_EXCLUSIONS):
+
+                if ".." not in os.path.relpath(path, HTML_DIR) and os.path.isfile(path) and (extension not in DISABLED_CONTENT_EXTENSIONS or os.path.split(path)[-1] in CONTENT_EXTENSIONS_EXCLUSIONS):
                     mtime = time.gmtime(os.path.getmtime(path))
                     if_modified_since = self.headers.get(HTTP_HEADER.IF_MODIFIED_SINCE)
 
@@ -156,7 +168,7 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                             skip = True
 
                     if not skip:
-                        content = open(path, "rb").read()
+                        content = content or open(path, "rb").read()
                         last_modified = time.strftime(HTTP_TIME_FORMAT, mtime)
                         self.send_response(_http_client.OK)
                         self.send_header(HTTP_HEADER.CONNECTION, "close")
@@ -164,9 +176,11 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                         self.send_header(HTTP_HEADER.LAST_MODIFIED, last_modified)
 
                         # For CSP policy directives see: https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Security-Policy/
-                        self.send_header(HTTP_HEADER.CONTENT_SECURITY_POLICY, "default-src 'self'; style-src 'self' 'unsafe-inline'; img-src * blob:; " +
-                                                                              "script-src 'self' 'unsafe-eval' https://stat.ripe.net; " +
-                                                                              "frame-src *; object-src 'none'; block-all-mixed-content;")
+                        self.send_header(HTTP_HEADER.CONTENT_SECURITY_POLICY, "default-src 'self'; style-src 'self' 'unsafe-inline'; img-src * blob:; script-src 'self' 'unsafe-eval' https://stat.ripe.net; frame-src *; object-src 'none'; block-all-mixed-content;")
+
+                        if os.path.basename(path) == "index.html":
+                            content = re.sub(b'\s*<script[^>]+src="js/demo.js"></script>', b'', content)
+
                         if extension not in (".htm", ".html"):
                             self.send_header(HTTP_HEADER.EXPIRES, "Sun, 17-Jan-2038 19:14:07 GMT")        # Reference: http://blog.httpwatch.com/2007/12/10/two-simple-rules-for-http-caching/
                             self.send_header(HTTP_HEADER.CACHE_CONTROL, "max-age=3600, must-revalidate")  # Reference: http://stackoverflow.com/a/5084555
@@ -186,7 +200,7 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                     name = match.group(1).decode(UNICODE_ENCODING)
                     _ = getattr(self, "_%s" % name.lower(), None)
                     if _:
-                        content = self._format(content, **{ name: _() })
+                        content = self._format(content, **{name: _()})
 
                 if "gzip" in self.headers.get(HTTP_HEADER.ACCEPT_ENCODING, ""):
                     self.send_header(HTTP_HEADER.CONTENT_ENCODING, "gzip")
@@ -443,12 +457,15 @@ def start_httpd(address=None, port=None, join=False, pem=None):
             return PING_RESPONSE
 
         def _fail2ban(self, params):
+            global _fail2ban_cache
+            global _fail2ban_key
+
             self.send_response(_http_client.OK)
             self.send_header(HTTP_HEADER.CONNECTION, "close")
             self.send_header(HTTP_HEADER.CONTENT_TYPE, "text/plain")
 
             content = ""
-            key = "fail2ban|%d" % (int(time.time()) >> 3)
+            key = int(time.time()) >> 3
 
             if config.FAIL2BAN_REGEX:
                 try:
@@ -456,8 +473,8 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                 except re.error:
                     content = "invalid regular expression used in option FAIL2BAN_REGEX"
                 else:
-                    if key in _result_cache:
-                        content = _result_cache[key]
+                    if key == _fail2ban_key:
+                        content = _fail2ban_cache
                     else:
                         result = set()
                         _ = os.path.join(config.LOG_DIR, "%s.log" % datetime.datetime.now().strftime("%Y-%m-%d"))
@@ -467,7 +484,9 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                                     result.add(line.split()[3])
 
                         content = "\n".join(result)
-                        _result_cache[key] = content
+
+                        _fail2ban_cache = content
+                        _fail2ban_key = key
             else:
                 content = "configuration option FAIL2BAN_REGEX not set"
 
@@ -674,8 +693,8 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                         with open(filepath, "rb") as f:
                             content = f.read(io.DEFAULT_BUFFER_SIZE)
                             if size >= io.DEFAULT_BUFFER_SIZE:
-                                total = 1.0 * content.count(b'\n') * size / io.DEFAULT_BUFFER_SIZE
-                                counts[timestamp] = int(round(total / 100) * 100)
+                                total = 1.0 * (1 + content.count(b'\n')) * size / io.DEFAULT_BUFFER_SIZE
+                                counts[timestamp] = int(round(total / 100.0) * 100)
                             else:
                                 counts[timestamp] = content.count(b'\n')
 
@@ -718,7 +737,7 @@ def start_httpd(address=None, port=None, join=False, pem=None):
 
     print("[i] starting HTTP%s server at http%s://%s:%d/" % ('S' if pem else "", 's' if pem else "", server.server_address[0], server.server_address[1]))
 
-    print("[o] running...")
+    print("[^] running...")
 
     if join:
         server.serve_forever()
