@@ -4,9 +4,17 @@
 
 local dirs = ntop.getDirs()
 package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path
-
+package.path = dirs.installdir .. "/scripts/lua/modules/pools/?.lua;" .. package.path
 require "lua_utils"
-local host_pools_utils = require "host_pools_utils"
+
+local host_pools_nedge
+if ntop.isnEdge() then
+   host_pools_nedge = require "host_pools_nedge"
+end
+local host_pools = require "host_pools"
+-- Instantiate host pools
+local host_pools_instance = host_pools:create()
+
 local page_utils = require("page_utils")
 local custom_column_utils = require("custom_column_utils")
 local discover = require("discover_utils")
@@ -47,11 +55,138 @@ if ((mode ~= "all") or (not isEmptyString(pool))) then
    hosts_filter = '<span class="fas fa-filter"></span>'
 end
 
+function getPageTitle(protocol_name, traffic_type_title, network_name, cidr, ipver_title, os_, country, asninfo, mac, pool_, vlan_title)
+   local mode_label = ""
+
+   if mode == "remote" then
+      mode_label = i18n("hosts_stats.remote")
+   elseif mode == "local" then
+      mode_label = i18n("hosts_stats.local")
+   elseif mode == "filtered" then
+      mode_label = i18n("hosts_stats.filtered")
+   elseif mode == "blacklisted" then
+      mode_label = i18n("hosts_stats.blacklisted")
+   elseif mode == "dhcp" then
+      mode_label = i18n("nedge.network_conf_dhcp")
+   end
+
+   -- Note: we must use the empty string as fallback. Multiple spaces will be collapsed into one automatically.
+   return i18n("hosts_stats.hosts_page_title", {
+        all = isEmptyString(mode_label) and i18n("hosts_stats.all") or "",
+        traffic_type = traffic_type_title or "",
+        local_remote = mode_label,
+        protocol = protocol_name or "",
+        network = not isEmptyString(network_name) and i18n("hosts_stats.in_network", {network=network_name}) or "",
+        network_cidr = not isEmptyString(cidr) and i18n("hosts_stats.in_network", {network=cidr}) or "",
+        ip_version = ipver_title or "",
+        ["os"] = discover.getOsName(os_),
+        country_asn_or_mac = country or asninfo or mac or pool_ or "",
+        vlan = vlan_title or "",
+   }) .. charts_icon
+end
+
 dofile(dirs.installdir .. "/scripts/lua/inc/menu.lua")
 
-prefs = ntop.getPrefs()
+local ifstats = interface.getStats()
 
-ifstats = interface.getStats()
+-- Parameters necessary for the page title
+local protocol_name = nil
+local network_name = nil
+local traffic_type_title = nil
+local ipver_title = nil
+local asninfo = nil
+local os_title = nil
+local country_title = nil
+local mac_title = nil
+local vlan_title = nil
+local pool_title = nil
+
+if((protocol ~= nil) and (protocol ~= "")) then
+   protocol_name = interface.getnDPIProtoName(tonumber(protocol))
+end
+
+if(protocol_name == nil) then protocol_name = protocol end
+
+local traffic_type_title
+if not isEmptyString(traffic_type) then
+   page_params["traffic_type"] = traffic_type
+
+   if traffic_type == "one_way" then
+      traffic_type_title = i18n("hosts_stats.traffic_type_one_way")
+   elseif traffic_type == "bidirectional" then
+      traffic_type_title = i18n("hosts_stats.traffic_type_two_ways")
+   end
+else
+   traffic_type_title = ""
+end
+
+if(network ~= nil) then
+   local network_key = ntop.getNetworkNameById(tonumber(network))
+   network_name = getLocalNetworkAlias(network_key)
+
+   if isEmptyString(network_name) then
+      network_name = i18n("hosts_stats.remote")
+   end
+else
+   network_name = ""
+end
+
+if not isEmptyString(ipversion) then
+   ipver_title = i18n("hosts_stats.ipver_title", {version_num=ipversion})
+else
+   ipver_title = ""
+end
+
+if(asn ~= nil) then
+   asninfo = " " .. i18n("hosts_stats.asn_title",{asn=asn}) ..
+      "<small>&nbsp;<i class='fas fa-info-circle fa-sm' aria-hidden='true'></i> <A HREF='https://stat.ripe.net/AS"..
+      asn .. "'><i class='fas fa-external-link-alt fa-sm' title=\\\"".. i18n("hosts_stats.more_info_about_as_popup_msg") ..
+      "\\\"></i></A></small>"
+end
+
+if(os_ ~= nil) then
+   os_title = " " .. os_
+end
+
+if(country ~= nil) then
+   country_title = " " .. i18n("hosts_stats.country_title", {country=country})
+end
+
+if(mac ~= nil) then
+   mac_title = " " .. i18n("hosts_stats.mac_title", {mac=mac})
+end
+
+if(vlan ~= nil) then
+   vlan_title = " [".. i18n("hosts_stats.vlan_title", {vlan=vlan}) .."]"
+end
+
+if(pool ~= nil) then
+   local charts_available = areHostPoolsTimeseriesEnabled(ifstats.id)
+   local pool_edit = ""
+   local pool_link
+   local title
+
+   if(tonumber(pool) ~= host_pools_instance.DEFAULT_POOL_ID) or (have_nedge) then
+      if have_nedge then
+	 pool_link = "/lua/pro/nedge/admin/nf_edit_user.lua?username=" ..
+	 ternary(pool == host_pools_nedge.DEFAULT_POOL_ID, "", host_pools_nedge.poolIdToUsername(pool))
+	 title = i18n("nedge.edit_user")
+      else
+	 pool_link = "/lua/admin/manage_host_members.lua?pool="..pool
+	 title = i18n("host_pools.manage_pools")
+      end
+
+      pool_edit = "&nbsp; <A HREF='"..ntop.getHttpPrefix()..pool_link.."'><i class='fas fa-cog fa-sm' title='"..title .. "'></i></A>"
+   end
+
+   pool_title = " "..i18n(ternary(have_nedge, "hosts_stats.user_title", "hosts_stats.pool_title"),
+		     {poolname=host_pools_instance:get_pool_name(pool)})
+      .."<small>".. pool_edit ..
+      ternary(charts_available, "&nbsp; <A HREF='"..ntop.getHttpPrefix().."/lua/pool_details.lua?page=historical&pool="..pool.."'><i class='fas fa-chart-area fa-sm' title='"..i18n("chart") .. "'></i></A>", "")..
+      "</small>"
+end
+
+page_utils.print_page_title(getPageTitle(protocol_name, traffic_type_title, network_name, cidr, ipver_title, os_title, country_title, asninfo, mac_title, pool_title, vlan_title))
 
 if (_GET["page"] ~= "historical") then
    if(asn ~= nil) then
@@ -105,7 +240,6 @@ if (_GET["page"] ~= "historical") then
    if(network ~= nil) then
       page_params["network"] = network
       local network_key = ntop.getNetworkNameById(tonumber(network))
-      network_name = getLocalNetworkAlias(network_key)
 
       if not isEmptyString(network_name) then
          local charts_available = areInterfaceTimeseriesEnabled(ifstats.id)
@@ -119,32 +253,11 @@ if (_GET["page"] ~= "historical") then
          end
 
          charts_icon = charts_icon.."</small>"
-      else
-         network_name = i18n("hosts_stats.remote")
       end
-   else
-      network_name = ""
    end
 
-   local ipver_title
    if not isEmptyString(ipversion) then
       page_params["version"] = ipversion
-      ipver_title = i18n("hosts_stats.ipver_title",{version_num=ipversion})
-   else
-      ipver_title = ""
-   end
-
-   local traffic_type_title
-   if not isEmptyString(traffic_type) then
-      page_params["traffic_type"] = traffic_type
-
-      if traffic_type == "one_way" then
-	 traffic_type_title = i18n("hosts_stats.traffic_type_one_way")
-      elseif traffic_type == "bidirectional" then
-	 traffic_type_title = i18n("hosts_stats.traffic_type_two_ways")
-      end
-   else
-      traffic_type_title = ""
    end
 
    custom_column_utils.updateCustomColumn()
@@ -171,102 +284,13 @@ if (_GET["page"] ~= "historical") then
 
    if(protocol == nil) then protocol = "" end
 
-   if(asn ~= nil) then 
-      asninfo = " " .. i18n("hosts_stats.asn_title",{asn=asn}) ..
-	 "<small>&nbsp;<i class='fas fa-info-circle fa-sm' aria-hidden='true'></i> <A HREF='https://stat.ripe.net/AS"..
-	 asn .. "'><i class='fas fa-external-link-alt fa-sm' title=\\\"".. i18n("hosts_stats.more_info_about_as_popup_msg") ..
-	 "\\\"></i></A></small>"
-   end
-
-   if(_GET["country"] ~= nil) then 
-      country = " " .. i18n("hosts_stats.country_title",{country=_GET["country"]})
-   end
-
-   if(_GET["mac"] ~= nil) then 
-      mac = " " .. i18n("hosts_stats.mac_title",{mac=_GET["mac"]})
-   end
-
-   if(_GET["os"] ~= nil) then 
-      os_ = " ".._GET["os"] 
-   end
-
-   if(_GET["pool"] ~= nil) then
-      local charts_available = areHostPoolsTimeseriesEnabled(ifstats.id)
-      local pool_edit = ""
-
-      if (_GET["pool"] ~= host_pools_utils.DEFAULT_POOL_ID) or (have_nedge) then
-	 local pool_link
-    local title
-
-	 if have_nedge then
-	    pool_link = "/lua/pro/nedge/admin/nf_edit_user.lua?username=" ..
-         ternary(_GET["pool"] == host_pools_utils.DEFAULT_POOL_ID, "", host_pools_utils.poolIdToUsername(_GET["pool"]))
-       title = i18n("nedge.edit_user")
-	 else
-	    pool_link = "/lua/if_stats.lua?page=pools&pool=".._GET["pool"]
-       title = i18n("host_pools.manage_pools")
-	 end
-
-	 pool_edit = "&nbsp; <A HREF='"..ntop.getHttpPrefix()..pool_link.."'><i class='fas fa-cog fa-sm' title='"..title .. "'></i></A>"
-
-      end
-
-      pool_ = " "..i18n(ternary(have_nedge, "hosts_stats.user_title", "hosts_stats.pool_title"),
-			{poolname=host_pools_utils.getPoolName(ifstats.id, _GET["pool"])})
-	 .."<small>".. pool_edit ..
-	 ternary(charts_available, "&nbsp; <A HREF='"..ntop.getHttpPrefix().."/lua/pool_details.lua?page=historical&pool=".._GET["pool"].."'><i class='fas fa-chart-area fa-sm' title='"..i18n("chart") .. "'></i></A>", "")..
-	 "</small>"
-   end
-
-   if(_GET["vlan"] ~= nil) then
-      vlan_title = " ["..i18n("hosts_stats.vlan_title",{vlan=_GET["vlan"]}).."]"
-   end
-
-   local protocol_name = nil
-
-   if((protocol ~= nil) and (protocol ~= "")) then
-      protocol_name = interface.getnDPIProtoName(tonumber(protocol))
-   end
-
-   if(protocol_name == nil) then protocol_name = protocol end
-
    if not isEmptyString(protocol_name) then
       charts_icon = " <a href='".. ntop.getHttpPrefix() .."/lua/if_stats.lua?ifid="..
          ifstats.id .. "&page=historical&ts_schema=iface:ndpi&protocol=" .. protocol_name..
          "'><i class='fas fa-sm fa-chart-area'></i></a>"
    end
 
-   function getPageTitle()
-      local mode_label = ""
-
-      if mode == "remote" then
-	 mode_label = i18n("hosts_stats.remote")
-      elseif mode == "local" then
-	 mode_label = i18n("hosts_stats.local")
-      elseif mode == "filtered" then
-	 mode_label = i18n("hosts_stats.filtered")
-      elseif mode == "blacklisted" then
-	 mode_label = i18n("hosts_stats.blacklisted")
-      elseif mode == "dhcp" then
-	 mode_label = i18n("nedge.network_conf_dhcp")
-      end
-
-      -- Note: we must use the empty string as fallback. Multiple spaces will be collapsed into one automatically.
-      return i18n("hosts_stats.hosts_page_title", {
-		     all = isEmptyString(mode_label) and i18n("hosts_stats.all") or "",
-		     traffic_type = traffic_type_title or "",
-		     local_remote = mode_label,
-		     protocol = protocol_name or "",
-		     network = not isEmptyString(network_name) and i18n("hosts_stats.in_network", {network=network_name}) or "",
-		     network_cidr = not isEmptyString(cidr) and i18n("hosts_stats.in_network", {network=cidr}) or "",
-		     ip_version = ipver_title or "",
-		     ["os"] = discover.getOsName(os_),
-		     country_asn_or_mac = country or asninfo or mac or pool_ or "",
-		     vlan = vlan_title or "",
-      }) .. charts_icon
-   end
-
-   print('title: "'..getPageTitle()..'",\n')
+   print('title: "",\n')
    print ('rowCallback: function ( row ) { return host_table_setID(row); },')
 
    print [[
@@ -286,20 +310,13 @@ if (_GET["page"] ~= "historical") then
 
    print('buttons: [ ')
 
-   
-   --[[ if((page_params.network ~= nil) and (page_params.network ~= "-1")) then
-      print('\'<div class="btn-group float-right"><A HREF="'..ntop.getHttpPrefix()..'/lua/network_details.lua?page=historical&network='..network..'"><i class=\"fas fa-chart-area fa-lg\"></i></A></div>\', ')
-      elseif (page_params.pool ~= nil) and (isAdministrator()) and (pool ~= host_pools_utils.DEFAULT_POOL_ID) then
-      print('\'<div class="btn-group float-right"><A HREF="'..ntop.getHttpPrefix()..'/lua/if_stats.lua?page=pools&pool='..pool..'#manage"><i class=\"fas fa-users fa-lg\"></i></A></div>\', ')
-      end]]
-
-   -- Ip version selector
+   -- IP version selector
    print[['<div class="btn-group float-right">]]
    custom_column_utils.printCustomColumnDropdown(base_url, page_params)
    print[[</div>']]
-   
 
-   -- Ip version selector
+
+   -- IP version selector
    print[[, '<div class="btn-group float-right">]]
    printIpVersionDropdown(base_url, page_params)
    print[[</div>']]
@@ -316,6 +333,7 @@ if (_GET["page"] ~= "historical") then
    print[[</div>']]
 
    -- Hosts filter
+   -- table.clone needed to modify some parameters while keeping the original unchanged
    local hosts_filter_params = table.clone(page_params)
 
    print(', \'<div class="btn-group"><button class="btn btn-link dropdown-toggle" data-toggle="dropdown">'..i18n("hosts_stats.filter_hosts")..hosts_filter..'<span class="caret"></span></button> <ul class="dropdown-menu scrollable-dropdown" role="menu" style="min-width: 90px;"><li"><a class="dropdown-item" href="')
@@ -370,10 +388,10 @@ if (_GET["page"] ~= "historical") then
       hosts_filter_params.mode = nil
       hosts_filter_params.pool = nil
       print('<li role="separator" class="divider"></li>')
-      for _, _pool in ipairs(host_pools_utils.getPoolsList(ifstats.id)) do
-	 hosts_filter_params.pool = _pool.id
+      for _, _pool in ipairs(host_pools_instance:get_all_pools()) do
+	 hosts_filter_params.pool = _pool.pool_id
 	 print('<li ')
-	 print('"><a class="dropdown-item ' .. ternary(pool == _pool.id, "active", "") ..'" href="')
+	 print('"><a class="dropdown-item ' .. ternary(pool == _pool.pool_id, "active", "") ..'" href="')
 	 print(getPageUrl(base_url, hosts_filter_params)..'">'
 		  ..i18n(ternary(have_nedge, "hosts_stats.user", "hosts_stats.host_pool"),
 			 {pool_name=string.gsub(_pool.name, "'", "\\'")}) ..'</li>')
@@ -428,16 +446,16 @@ if (_GET["page"] ~= "historical") then
 			     title: "]] print(i18n("hosts_stats.location")) print[[",
 				 field: "column_location",
 				 sortable: false,
-	 	             css: { 
+	 	             css: {
 			        textAlign: 'center'
 			     }
 
-				 },			     
+				 },
 			     {
 			     title: "]] print(i18n("flows")) print[[",
 				 field: "column_num_flows",
 				 sortable: true,
-	 	             css: { 
+	 	             css: {
 			        textAlign: 'center'
 			     }
 
@@ -452,7 +470,7 @@ if (_GET["page"] ~= "historical") then
       print("true")
    end
    print[[,
-	 	             css: { 
+	 	             css: {
 			        textAlign: 'center'
 			     }
 
@@ -481,7 +499,7 @@ if (_GET["page"] ~= "historical") then
 			     title: "]] print(i18n("seen_since")) print[[",
 				 field: "column_since",
 				 sortable: true,
-	 	             css: { 
+	 	             css: {
 			        textAlign: 'center'
 			     }
 				 }, {
@@ -496,7 +514,7 @@ if (_GET["page"] ~= "historical") then
 			     title: "]] print(i18n("throughput")) print[[",
 				 field: "column_thpt",
 				 sortable: true,
-	 	             css: { 
+	 	             css: {
 			        textAlign: 'right'
 			     }
 				 },
@@ -504,7 +522,7 @@ if (_GET["page"] ~= "historical") then
 			     title: "]] print(i18n("flows_page.total_bytes")) print[[",
 				 field: "column_traffic",
 				 sortable: true,
-	 	             css: { 
+	 	             css: {
 			        textAlign: 'right'
 			     }
 				 }
@@ -519,7 +537,7 @@ if (_GET["page"] ~= "historical") then
    if(have_nedge) then
       print[[
 <script>
-  var block_host_csrf = "]] print(ntop.getRandomCSRFValue()) print[[";
+  const block_host_csrf = "]] print(ntop.getRandomCSRFValue()) print[[";
 
   function block_host(host_key, host_url) {
     var url = "]] print(ntop.getHttpPrefix()) print[[/lua/pro/nedge/toggle_block_host.lua?" + host_url;
@@ -532,14 +550,13 @@ if (_GET["page"] ~= "historical") then
       },
       success: function(content) {
         var data = jQuery.parseJSON(content);
-        block_host_csrf = data.csrf;
         if (data.status == "BLOCKED") {
           $('#'+host_key+'_info').find('.block-badge')
-            .removeClass('badge-secondary').addClass('badge-danger');
+            .removeClass('btn-secondary').addClass('btn-danger');
           $('#'+host_key+'_ip').find('a').css("text-decoration", "line-through");
         } else if (data.status == "UNBLOCKED") {
           $('#'+host_key+'_info').find('.block-badge')
-            .removeClass('badge-danger').addClass('badge-secondary');
+            .removeClass('btn-danger').addClass('btn-secondary');
           $('#'+host_key+'_ip').find('a').css("text-decoration", "none");
         }
       },
