@@ -7,7 +7,6 @@ package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path
 
 require "template"
 require "lua_utils"
-require "flow_aggregation_utils"
 
 local db_debug = false
 
@@ -54,12 +53,6 @@ local function flowsTableName(version, force_raw)
 
    local tblname = "flowsv"..version
 
-   if force_raw ~= true and ntop.isEnterprise() == true then
-      if useAggregatedFlows() == true then
-	 tblname = "aggr"..tblname
-      end
-   end
-
    -- return "flowsv"..version -- FIXX: remove this line when ready
    return tblname
 end
@@ -163,7 +156,7 @@ end
 
 -- ########################################################
 
-function getInterfaceTopFlows(interface_id, version, host_or_profile, peer, l7proto, l4proto, port, vlan, profile, info, begin_epoch, end_epoch, offset, max_num_flows, sort_column, sort_order, aggregated_flows)
+function getInterfaceTopFlows(interface_id, version, host_or_profile, peer, l7proto, l4proto, port, vlan, profile, info, begin_epoch, end_epoch, offset, max_num_flows, sort_column, sort_order)
    -- CONVERT(UNCOMPRESS(JSON) USING 'utf8') AS JSON
 
    if(version == 4) then
@@ -782,10 +775,6 @@ end
 
 function db_utils.harverstExpiredMySQLFlows(ifname, mysql_retention, verbose)
    local dbtables = {"flowsv4", "flowsv6"}
-   if useAggregatedFlows() then
-      dbtables[#dbtables+1] = "aggrflowsv4"
-      dbtables[#dbtables+1] = "aggrflowsv6"
-   end
 
    if tonumber(ifname) == nil then
       ifname = getInterfaceId(ifname)
