@@ -157,6 +157,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         foreach (glob("/tmp/packetcapture_*.cap") as $filename) {
             $bfilename = basename($filename);
             if ($_GET['download'] === $bfilename) {
+                ob_end_clean(); // disable legacy csrf output buffering
                 header("Content-Type: application/octet-stream");
                 header("Content-Disposition: attachment; filename={$bfilename}");
                 header("Content-Length: ".filesize($filename));
@@ -367,7 +368,7 @@ include("fbegin.inc");
                     <td>
                       <input name="promiscuous" type="checkbox" <?= !empty($pconfig['promiscuous']) ? " checked=\"checked\"" : ""; ?> />
                       <div class="hidden" data-for="help_for_promiscuous">
-                        <?=gettext("If checked, the");?> <a target="_blank" href="http://www.freebsd.org/cgi/man.cgi?query=tcpdump&amp;apropos=0&amp;sektion=0&amp;manpath=FreeBSD+8.3-stable&amp;arch=default&amp;format=html"><?= gettext("packet capture")?></a> <?= gettext("will be performed using promiscuous mode.");?>
+                        <?=gettext("If checked, the");?> <a target="_blank" href="https://www.freebsd.org/cgi/man.cgi?query=tcpdump&amp;apropos=0&amp;sektion=0&amp;manpath=FreeBSD+8.3-stable&amp;arch=default&amp;format=html"><?= gettext("packet capture")?></a> <?= gettext("will be performed using promiscuous mode.");?>
                         <br /><b><?=gettext("Note");?>: </b><?=gettext("Some network adapters do not support or work well in promiscuous mode.");?>
                       </div>
                   </td>
@@ -446,7 +447,7 @@ include("fbegin.inc");
                   <tr>
                     <td><a id="help_for_count" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext("Count");?></td>
                     <td>
-                      <input type="text" name="count" class="formfld unknown" id="count" size="5" value="<?=$pconfig['count'];?>" />
+                      <input type="text" name="count" id="count" size="5" value="<?= $pconfig['count']; ?>" />
                       <div class="hidden" data-for="help_for_count">
                         <?=gettext("This is the number of packets the packet capture will grab. Default value is 100.") . "<br />" . gettext("Enter 0 (zero) for no count limit.");?>
                       </div>
