@@ -1,31 +1,31 @@
 <?php
 
 /*
-    Copyright (C) 2014 Deciso B.V.
-    Copyright (C) 2003-2004 Manuel Kasper <mk@neon1.net>
-    All rights reserved.
-
-    Redistribution and use in source and binary forms, with or without
-    modification, are permitted provided that the following conditions are met:
-
-    1. Redistributions of source code must retain the above copyright notice,
-       this list of conditions and the following disclaimer.
-
-    2. Redistributions in binary form must reproduce the above copyright
-       notice, this list of conditions and the following disclaimer in the
-       documentation and/or other materials provided with the distribution.
-
-    THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
-    INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
-    AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
-    AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
-    OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-    SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-    INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-    CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-    ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-    POSSIBILITY OF SUCH DAMAGE.
-*/
+ * Copyright (C) 2014 Deciso B.V.
+ * Copyright (C) 2003-2004 Manuel Kasper <mk@neon1.net>
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright notice,
+ *    this list of conditions and the following disclaimer.
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ *
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
+ * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
+ * AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ */
 
 require_once("guiconfig.inc");
 require_once("interfaces.inc");
@@ -101,10 +101,6 @@ legacy_html_escape_form_data($a_1to1);
 
 include("head.inc");
 
-$main_buttons = array(
-    array('label' => gettext('Add'), 'href' => 'firewall_nat_1to1_edit.php'),
-);
-
 ?>
 <body>
   <script>
@@ -220,11 +216,43 @@ $main_buttons = array(
 
     // watch scroll position and set to last known on page load
     watchScrollPosition();
+
+    // move category block
+    $("#category_block").detach().appendTo($(".page-content-head > .container-fluid > .list-inline"));
+    $("#category_block").addClass("pull-right");
+
+    // our usual zebra striping doesn't respect hidden rows, hook repaint on .opnsense-rules change() and fire initially
+    $(".opnsense-rules > tbody > tr").each(function(){
+        // save zebra color
+        let tr_color = $(this).children(0).css("background-color");
+        if (tr_color != 'transparent' && !tr_color.includes('(0, 0, 0')) {
+            $("#fw_category").data('stripe_color', tr_color);
+        }
+    });
+    $(".opnsense-rules").removeClass("table-striped");
+    $(".opnsense-rules").change(function(){
+        $(".opnsense-rules > tbody > tr:visible").each(function (index) {
+            $(this).css("background-color", "inherit");
+            if ( index % 2 == 0) {
+                $(this).css("background-color", $("#fw_category").data('stripe_color'));
+            }
+        });
+    });
+
+    // hook category functionality
+    hook_firewall_categories();
+
   });
   </script>
 
 
 <?php include("fbegin.inc"); ?>
+  <div class="hidden">
+    <div id="category_block" style="z-index:-100;">
+        <select class="selectpicker hidden-xs hidden-sm hidden-md" data-live-search="true" data-size="5"  multiple title="<?=gettext("Select category");?>" id="fw_category">
+        </select>
+    </div>
+  </div>
   <section class="page-content-main">
     <div class="container-fluid">
       <div class="row">
@@ -242,7 +270,7 @@ $main_buttons = array(
             <form method="post" name="iform" id="iform">
               <input type="hidden" id="id" name="id" value="" />
               <input type="hidden" id="action" name="action" value="" />
-              <table class="table table-striped">
+              <table class="table table-striped table-condensed opnsense-rules">
                 <thead>
                   <tr>
                     <th><input type="checkbox" id="selectAll"></th>
@@ -252,7 +280,25 @@ $main_buttons = array(
                     <th><?=gettext("Internal IP"); ?></th>
                     <th><?=gettext("Destination IP"); ?></th>
                     <th><?=gettext("Description"); ?></th>
-                    <th></th>
+                    <th class="text-nowrap">
+                      <a href="firewall_nat_1to1_edit.php" class="btn btn-primary btn-xs" data-toggle="tooltip" title="<?= html_safe(gettext('Add')) ?>">
+                        <i class="fa fa-plus fa-fw"></i>
+                      </a>
+<?php if (count($a_1to1)): ?>
+                      <button id="move_<?= count($a_1to1) ?>" name="move_<?= count($a_1to1) ?>_x" data-toggle="tooltip" title="<?= html_safe(gettext('Move selected rules to end')) ?>" class="act_move btn btn-default btn-xs">
+                        <i class="fa fa-arrow-left fa-fw"></i>
+                      </button>
+                      <button id="del_x" title="<?= html_safe(gettext('Delete selected')) ?>" data-toggle="tooltip" class="act_delete btn btn-default btn-xs">
+                        <i class="fa fa-trash fa-fw"></i>
+                      </button>
+                      <button title="<?= html_safe(gettext('Enable selected')) ?>" data-toggle="tooltip" class="act_toggle_enable btn btn-default btn-xs">
+                        <i class="fa fa-check-square-o fa-fw"></i>
+                      </button>
+                      <button title="<?= html_safe(gettext('Disable selected')) ?>" data-toggle="tooltip" class="act_toggle_disable btn btn-default btn-xs">
+                        <i class="fa fa-square-o fa-fw"></i>
+                      </button>
+                    </th>
+<?php endif ?>
                   </tr>
                 </thead>
                 <tbody>
@@ -260,7 +306,7 @@ $main_buttons = array(
                 $i = 0;
                 foreach ($a_1to1 as $natent):
 ?>
-                  <tr <?=isset($natent['disabled'])?"class=\"text-muted\"":"";?> ondblclick="document.location='firewall_nat_1to1_edit.php?id=<?=$i;?>';">
+                  <tr class="rule <?=isset($natent['disabled'])?"text-muted":"";?>" data-category="<?=!empty($natent['category']) ? $natent['category'] : "";?>" ondblclick="document.location='firewall_nat_1to1_edit.php?id=<?=$i;?>';">
                     <td>
                       <input class="rule_select" type="checkbox" name="rule[]" value="<?=$i;?>" />
                     </td>
@@ -308,7 +354,7 @@ $main_buttons = array(
                         <?=htmlspecialchars(pprint_address($natent['destination']));?>
 <?php                 endif; ?>
                     </td>
-                    <td>
+                    <td class="rule-description">
                       <?=$natent['descr'];?> &nbsp;
                     </td>
                     <td>
@@ -330,25 +376,6 @@ $main_buttons = array(
                   $i++;
                 endforeach;
 ?>
-<?php if ($i != 0): ?>
-                  <tr>
-                    <td colspan="7"></td>
-                    <td>
-                      <button id="move_<?=$i;?>" name="move_<?=$i;?>_x" data-toggle="tooltip" title="<?= html_safe(gettext('Move selected rules to end')) ?>" class="act_move btn btn-default btn-xs">
-                        <i class="fa fa-arrow-left fa-fw"></i>
-                      </button>
-                      <button id="del_x" title="<?= html_safe(gettext('Delete selected')) ?>" data-toggle="tooltip" class="act_delete btn btn-default btn-xs">
-                        <i class="fa fa-trash fa-fw"></i>
-                      </button>
-                      <button title="<?= html_safe(gettext('Enable selected')) ?>" data-toggle="tooltip" class="act_toggle_enable btn btn-default btn-xs">
-                          <i class="fa fa-check-square-o fa-fw"></i>
-                      </button>
-                      <button title="<?= html_safe(gettext('Disable selected')) ?>" data-toggle="tooltip" class="act_toggle_disable btn btn-default btn-xs">
-                          <i class="fa fa-square-o fa-fw"></i>
-                      </button>
-                    </td>
-                  </tr>
-<?php endif ?>
                 </tbody>
                 <tfoot>
                   <tr>
