@@ -55,6 +55,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
     $pconfig['dstbeginport'] = 80 ;
     $pconfig['dstendport'] = 80 ;
     $pconfig['local-port'] = 80;
+    $pconfig['associated-rule-id'] = true;
     if (isset($configId)) {
         // copy 1-on-1
         foreach (array('protocol','target','local-port','descr','interface','associated-rule-id','nosync','log',
@@ -212,8 +213,8 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         if ($pconfig['protocol'] != 'any') {
             $natent['protocol'] = $pconfig['protocol'];
         }
-        $natent['interface'] = implode(",", $pconfig['interface']);
-        $natent['category'] = implode(",", $pconfig['category']);
+        $natent['interface'] = !empty($pconfig['interface']) ? implode(",", $pconfig['interface']) : null;
+        $natent['category'] = !empty($pconfig['category']) ? implode(",", $pconfig['category']) : null;
         $natent['ipprotocol'] = $pconfig['ipprotocol'];
         $natent['descr'] = $pconfig['descr'];
         $natent['tag'] = $pconfig['tag'];
@@ -989,7 +990,7 @@ $( document ).ready(function() {
 <?php
                     foreach ((new OPNsense\Firewall\Category())->iterateCategories() as $category):
                       $catname = htmlspecialchars($category['name'], ENT_QUOTES | ENT_HTML401);?>
-                      <option value="<?=$catname;?>" <?=in_array($catname, $pconfig['category']) ? 'selected="selected"' : '';?> ><?=$catname;?></option>
+                      <option value="<?=$catname;?>" <?=!empty($pconfig['category']) && in_array($catname, $pconfig['category']) ? 'selected="selected"' : '';?> ><?=$catname;?></option>
 <?php
                     endforeach;?>
                     </select>
@@ -1000,9 +1001,9 @@ $( document ).ready(function() {
                 <tr>
                   <td><a id="help_for_descr" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext("Description"); ?></td>
                   <td>
-                    <input name="descr" type="text" class="formfld unknown" id="descr" size="40" value="<?=$pconfig['descr'];?>" />
+                    <input name="descr" type="text" id="descr" size="40" value="<?=$pconfig['descr'];?>" />
                     <div class="hidden" data-for="help_for_descr">
-                      <?=gettext("You may enter a description here " ."for your reference (not parsed)."); ?>
+                      <?=gettext("You may enter a description here for your reference (not parsed)."); ?>
                     </div>
                 </tr>
                 <tr>
@@ -1077,10 +1078,10 @@ $( document ).ready(function() {
                   <td><a id="help_for_fra" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext("Filter rule association"); ?></td>
                   <td>
                     <select name="filter-rule-association">
-                      <option value=""><?=gettext("None"); ?></option>
-                      <option value="add-associated" selected="selected"><?=gettext("Add associated filter rule"); ?></option>
+                      <option value="" <?= empty($pconfig['associated-rule-id']) ? " selected=\"selected\"" : ""; ?>><?=gettext("None"); ?></option>
+                      <option value="add-associated" <?= !empty($pconfig['associated-rule-id']) && $pconfig['associated-rule-id'] != "pass" ? " selected=\"selected\"" : ""; ?>><?=gettext("Add associated filter rule"); ?></option>
                       <option value="add-unassociated"><?=gettext("Add unassociated filter rule"); ?></option>
-                      <option value="pass"><?=gettext("Pass"); ?></option>
+                      <option value="pass" <?= $pconfig['associated-rule-id'] == "pass" ? " selected=\"selected\"" : ""; ?>><?=gettext("Pass"); ?></option>
                     </select>
                     <div class="hidden" data-for="help_for_fra">
                       <?=gettext("NOTE: The \"pass\" selection does not work properly with Multi-WAN. It will only work on an interface containing the default gateway.")?>
