@@ -48,7 +48,7 @@ $gateways = new \OPNsense\Routing\Gateways(legacy_interfaces_details());
  */
 function FormSetAdvancedOptions(&$item) {
     foreach (array("max", "max-src-nodes", "max-src-conn", "max-src-states","nopfsync", "statetimeout"
-                  ,"max-src-conn-rate","max-src-conn-rates", "tag", "tagged", "allowopts", "disablereplyto","tcpflags1"
+                  ,"max-src-conn-rate","max-src-conn-rates", "tag", "tagged", "allowopts", "reply-to","tcpflags1"
                   ,"tcpflags2") as $fieldname) {
 
         if (!empty($item[$fieldname])) {
@@ -97,6 +97,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         'direction',
         'disabled',
         'disablereplyto',
+        'reply-to',
         'floating',
         'gateway',
         'icmptype',
@@ -172,6 +173,8 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
             $pconfig[$fieldname] = null;
         }
     }
+    // replyto switch
+    $pconfig['reply-to'] = !empty($pconfig['disablereplyto']) ? "__disable__" : $pconfig['reply-to'];
 } elseif ($_SERVER['REQUEST_METHOD'] === 'POST') {
     $input_errors = array();
     $pconfig = $_POST;
@@ -238,6 +241,16 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
             $input_errors[] = gettext('You can not assign the IPv6 Gateway to an IPv4 filter rule.');
         }
     }
+    if ($pconfig['ipprotocol'] == "inet46" && !empty($pconfig['reply-to']) && $pconfig['reply-to'] != '__disable__') {
+        $input_errors[] = gettext("You can not assign a reply-to destination to a rule that applies to IPv4 and IPv6");
+    } elseif (!empty($pconfig['reply-to']) && is_ipaddr($gateways->getAddress($pconfig['reply-to']))) {
+        if ($pconfig['ipprotocol'] == "inet6" && !is_ipaddrv6($gateways->getAddress($pconfig['reply-to']))) {
+            $input_errors[] = gettext('You can not assign the IPv4 reply-to destination to an IPv6 filter rule.');
+        }
+        if ($pconfig['ipprotocol'] == "inet" && !is_ipaddrv4($gateways->getAddress($pconfig['reply-to']))) {
+            $input_errors[] = gettext('You can not assign the IPv6 reply-to destination to an IPv4 filter rule.');
+        }
+    }
     if ($pconfig['protocol'] == "icmp" && !empty($pconfig['icmptype']) && $pconfig['ipprotocol'] == "inet46") {
         $input_errors[] =  gettext('You can not assign an ICMP type to a rule that applies to IPv4 and IPv6.');
     } elseif ($pconfig['protocol'] == "ipv6-icmp" && !empty($pconfig['icmp6-type']) && $pconfig['ipprotocol'] == "inet46") {
@@ -339,18 +352,27 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
           $input_errors[] = gettext("You can only specify the state timeout (advanced option) for TCP protocol.");
     }
     if ($pconfig['type'] != 'pass') {
-      if (!empty($pconfig['max']))
+      if (!empty($pconfig['max'])) {
           $input_errors[] = gettext("You can only specify the maximum state entries (advanced option) for Pass type rules.");
-      if (!empty($pconfig['max-src-nodes']))
+      }
+      if (!empty($pconfig['max-src-nodes'])) {
           $input_errors[] = gettext("You can only specify the maximum number of unique source hosts (advanced option) for Pass type rules.");
-      if (!empty($pconfig['max-src-conn']))
+      }
+      if (!empty($pconfig['max-src-conn'])) {
           $input_errors[] = gettext("You can only specify the maximum number of established connections per host (advanced option) for Pass type rules.");
-      if (!empty($pconfig['max-src-states']))
+      }
+      if (!empty($pconfig['max-src-states'])) {
           $input_errors[] = gettext("You can only specify the maximum state entries per host (advanced option) for Pass type rules.");
-      if (!empty($pconfig['max-src-conn-rate']) || !empty($pconfig['max-src-conn-rates']))
+      }
+      if (!empty($pconfig['max-src-conn-rate']) || !empty($pconfig['max-src-conn-rates'])) {
           $input_errors[] = gettext("You can only specify the maximum new connections per host / per second(s) (advanced option) for Pass type rules.");
-      if (!empty($pconfig['statetimeout']))
+      }
+      if (!empty($pconfig['statetimeout'])) {
           $input_errors[] = gettext("You can only specify the state timeout (advanced option) for Pass type rules.");
+      }
+      if (!empty($pconfig['allowopts'])) {
+          $input_errors[] = gettext("You can only specify allow options (advanced option) for Pass type rules.");
+      }
     }
     if ($pconfig['statetype'] == "none") {
       if (!empty($pconfig['max']))
@@ -457,8 +479,10 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         if (!empty($pconfig['allowopts'])) {
             $filterent['allowopts'] = true;
         }
-        if (!empty($pconfig['disablereplyto'])) {
+        if ($pconfig['reply-to'] == "__disable__") {
             $filterent['disablereplyto'] = true;
+        } elseif (!empty($pconfig['reply-to'])) {
+            $filterent['reply-to'] = $pconfig['reply-to'];
         }
         if(!empty($pconfig['nopfsync'])) {
             $filterent['nopfsync'] = true;
@@ -830,8 +854,13 @@ include("head.inc");
                       endforeach; ?>
                       </select>
                       <div class="hidden" data-for="help_for_direction">
-                        <?=gettext("Direction of the traffic. The default policy is to filter inbound traffic, ".
-                                   "which sets the policy to the interface originally receiving the traffic.") ?>
+                        <?=gettext("Direction of the traffic. Traffic IN is coming into the firewall interface, ".
+                                   "while traffic OUT is going out of the firewall interface. In visual terms: ".
+                                   "[Source] -> IN -> [Firewall] -> OUT -> [Destination]. The default policy is to ".
+                                   "filter inbound traffic, which means the policy applies to the interface on which ".
+                                   "the traffic is originally received by the firewall from the source. This is more ".
+                                   "efficient from a traffic processing perspective. In most cases, the default ".
+                                   "policy will be the most appropriate.") ?>
                       </div>
                     </td>
                   <tr>
@@ -1219,7 +1248,7 @@ include("head.inc");
 <?php
                       foreach ((new OPNsense\Firewall\Category())->iterateCategories() as $category):
                         $catname = htmlspecialchars($category['name'], ENT_QUOTES | ENT_HTML401);?>
-                        <option value="<?=$catname;?>" <?=in_array($catname, $pconfig['category']) ? 'selected="selected"' : '';?> ><?=$catname;?></option>
+                        <option value="<?=$catname;?>" <?=!empty($pconfig['category']) && in_array($catname, $pconfig['category']) ? 'selected="selected"' : '';?> ><?=$catname;?></option>
 <?php
                       endforeach;?>
                       </select>
@@ -1230,7 +1259,7 @@ include("head.inc");
                   <tr>
                     <td><a id="help_for_descr" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext("Description"); ?></td>
                     <td>
-                      <input name="descr" type="text" class="formfld unknown" id="descr" size="40" value="<?=$pconfig['descr'];?>" />
+                      <input name="descr" type="text" id="descr" size="40" value="<?=$pconfig['descr'];?>" />
                       <div class="hidden" data-for="help_for_descr">
                         <?=gettext("You may enter a description here for your reference (not parsed)."); ?>
                       </div>
@@ -1335,11 +1364,26 @@ include("head.inc");
                       </td>
                   </tr>
                   <tr class="opt_advanced hidden">
-                      <td><a id="help_for_disable_replyto" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a>  <?=gettext("disable reply-to");?> </td>
+                      <td><a id="help_for_replyto" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a>  <?=gettext("reply-to");?> </td>
                       <td>
-                        <input type="checkbox" value="yes" name="disablereplyto"<?= !empty($pconfig['disablereplyto']) ? " checked=\"checked\"" :""; ?> />
-                        <div class="hidden" data-for="help_for_disable_replyto">
-                          <?=gettext("This will disable auto generated reply-to for this rule.");?>
+                        <select name="reply-to" class="selectpicker" data-live-search="true" data-size="5" data-width="auto">
+                          <option value="" ><?=gettext("default");?></option>
+                          <option value="__disable__" <?= "__disable__" == $pconfig['reply-to'] ? " selected=\"selected\"" : "";?> ><?=gettext("disable");?></option>
+<?php
+                        foreach($gateways->gatewaysIndexedByName(true, true, true) as $gwname => $gw):?>
+                          <option value="<?=$gwname;?>" <?=$gwname == $pconfig['reply-to'] ? " selected=\"selected\"" : "";?>>
+                            <?=$gw['name'];?>
+                            <?=empty($gw['gateway']) ? "" : " - " . $gw['gateway'];?>
+                          </option>
+<?php
+endforeach;?>
+                        </select>
+                        <div class="hidden" data-for="help_for_replyto">
+                          <?=gettext(
+                            "Determines how packets route back in the opposite direction (replies), ".
+                            "when set to default, packets on WAN type interfaces reply to their connected gateway on the interface (unless globally disabled). " .
+                            "A specific gateway may be chosen as well here. This setting is only relevant in the context of a state, " .
+                            "for stateless rules there is no defined opposite direction.");?>
                         </div>
                       </td>
                   </tr>
