@@ -53,6 +53,7 @@ function lagg_inuse($lagg_intf)
 $a_laggs = &config_read_array('laggs', 'lagg');
 
 if ($_SERVER['REQUEST_METHOD'] === 'POST') {
+    $input_errors = array();
     if (!empty($a_laggs[$_POST['id']])) {
         $id = $_POST['id'];
     }
@@ -70,15 +71,11 @@ if ($_SERVER['REQUEST_METHOD'] === 'POST') {
     }
 }
 
-
 include("head.inc");
+
 legacy_html_escape_form_data($a_laggs);
-$main_buttons = array(
-  array('href'=>'interfaces_lagg_edit.php', 'label'=>gettext('Add')),
-);
 
 ?>
-
 <body>
   <script>
   $( document ).ready(function() {
@@ -126,7 +123,11 @@ $main_buttons = array(
                         <th><?=gettext("Members");?></th>
                         <th><?=gettext("Protocol");?></th>
                         <th><?=gettext("Description");?></th>
-                        <th class="text-nowrap"></th>
+                        <th class="text-nowrap">
+                           <a href="interfaces_lagg_edit.php" class="btn btn-primary btn-xs" data-toggle="tooltip" title="<?= html_safe(gettext('Add')) ?>">
+                            <i class="fa fa-plus fa-fw"></i>
+                          </a>
+                        </th>
                       </tr>
                     </thead>
                     <tbody>
@@ -150,11 +151,6 @@ $main_buttons = array(
 <?php
                     $i++;
                     endforeach; ?>
-                      <tr>
-                        <td colspan="5">
-                           <?=gettext("LAGG allows for link aggregation, bonding and fault tolerance. Only unassigned interfaces can be added to LAGG."); ?>
-                        </td>
-                      </tr>
                     </tbody>
                   </table>
                 </div>
