@@ -248,7 +248,7 @@ include("head.inc");
                     <td>
                       <input name="descr" type="text" value="<?=$pconfig['descr'];?>" />
                       <div class="hidden" data-for="help_for_descr">
-                        <?=gettext("You may enter a description here "."for your reference (not parsed).");?>
+                        <?=gettext("You may enter a description here for your reference (not parsed).");?>
                       </div>
                     </td>
                   </tr>
