@@ -1,7 +1,7 @@
 <?php
 
 /*
- * Copyright (C) 2014-2016 Deciso B.V.
+ * Copyright (C) 2014-2021 Deciso B.V.
  * Copyright (C) 2004-2009 Scott Ullrich <sullrich@gmail.com>
  * Copyright (C) 2003-2004 Manuel Kasper <mk@neon1.net>
  * All rights reserved.
@@ -201,7 +201,6 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
     /* remove the old array */
     unset($lease_content);
 
-    /* remove duplicate items by mac address */
     if (count($leases) > 0) {
         $leases = remove_duplicate($leases,"ip");
     }
@@ -217,40 +216,46 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
             $macs[$this_lease['mac']] = $i;
         }
     }
-    foreach ($interfaces as $ifname => $ifarr) {
-        if (isset($config['dhcpd'][$ifname]['staticmap'])) {
-            foreach($config['dhcpd'][$ifname]['staticmap'] as $static) {
-                $slease = array();
-                $slease['ip'] = $static['ipaddr'];
-                $slease['type'] = "static";
-                $slease['mac'] = $static['mac'];
-                $slease['start'] = '';
-                $slease['end'] = '';
-                $slease['hostname'] = $static['hostname'];
-                $slease['descr'] = $static['descr'];
-                $slease['act'] = "static";
-                $slease['online'] = in_array(strtolower($slease['mac']), $arpdata_mac) ? 'online' : 'offline';
-                if (isset($macs[$slease['mac']])) {
-                    // update lease with static data
-                    foreach ($slease as $key => $value) {
-                        if (!empty($value)) {
-                            $leases[$macs[$slease['mac']]][$key] = $slease[$key];
-                        }
-                    }
-                } else {
-                    $leases[] = $slease;
+
+    foreach (dhcpd_staticmap() as $static) {
+        if (!isset($static['ipaddr'])) {
+            continue;
+        }
+
+        $slease = array();
+        $slease['ip'] = $static['ipaddr'];
+        $slease['type'] = 'static';
+        $slease['mac'] = $static['mac'];
+        $slease['start'] = '';
+        $slease['end'] = '';
+        $slease['hostname'] = $static['hostname'];
+        $slease['descr'] = $static['descr'];
+        $slease['act'] = 'static';
+        $slease['online'] = in_array(strtolower($slease['mac']), $arpdata_mac) ? 'online' : 'offline';
+
+        if (isset($macs[$slease['mac']])) {
+            /* update lease with static data */
+            foreach ($slease as $key => $value) {
+                if (!empty($value)) {
+                    $leases[$macs[$slease['mac']]][$key] = $slease[$key];
                 }
             }
+        } else {
+            $leases[] = $slease;
         }
     }
 
-    $order = ( $_GET['order'] ) ? $_GET['order'] : 'ip';
+    if (isset($_GET['order']) && in_array($_GET['order'], ['int', 'ip', 'mac', 'hostname', 'descr', 'start', 'end', 'online', 'act'])) {
+        $order = $_GET['order'];
+    } else {
+        $order = 'ip';
+    }
 
     usort($leases,
         function ($a, $b) use ($order) {
-            $cmp = strnatcasecmp($a[$order], $b[$order]);
+            $cmp = ($order === 'ip') ? 0 : strnatcasecmp($a[$order], $b[$order]);
             if ($cmp === 0) {
-                $cmp = strnatcasecmp($a['ip'], $b['ip']);
+                $cmp = ipcmp($a['ip'], $b['ip']);
             }
             return $cmp;
         }
@@ -284,7 +289,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
             @unlink($leasesfile);
             @rename($leasesfile.".new", $leasesfile);
 
-            dhcpd_dhcp_configure(false, 'inet');
+            dhcpd_dhcp4_configure();
         }
     }
     exit;
@@ -372,7 +377,7 @@ legacy_html_escape_form_data($leases);
             <table class="table table-striped">
               <thead>
                 <tr>
-                    <td><?=gettext("Interface"); ?></td>
+                    <td class="act_sort" data-field="int"><?=gettext("Interface"); ?></td>
                     <td class="act_sort" data-field="ip"><?=gettext("IP address"); ?></td>
                     <td class="act_sort" data-field="mac"><?=gettext("MAC address"); ?></td>
                     <td class="act_sort" data-field="hostname"><?=gettext("Hostname"); ?></td>
