@@ -1,7 +1,7 @@
 <?php
 
 /*
- * Copyright (C) 2014-2016 Deciso B.V.
+ * Copyright (C) 2014-2021 Deciso B.V.
  * Copyright (C) 2004-2009 Scott Ullrich <sullrich@gmail.com>
  * Copyright (C) 2011 Seth Mos <seth.mos@dds.nl>
  * Copyright (C) 2003-2004 Manuel Kasper <mk@neon1.net>
@@ -288,34 +288,57 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         asort($pools);
     }
 
-    foreach ($interfaces as $ifname => $ifarr) {
-        if (isset($config['dhcpdv6'][$ifname]['staticmap'])) {
-            foreach($config['dhcpdv6'][$ifname]['staticmap'] as $static) {
-                $slease = array();
-                $slease['ip'] = $static['ipaddrv6'];
-                $slease['type'] = "static";
-                $slease['duid'] = $static['duid'];
-                $slease['start'] = "";
-                $slease['end'] = "";
-                $slease['hostname'] = $static['hostname'];
-                $slease['descr'] = $static['descr'];
-                $slease['act'] = "static";
-                if (in_array($slease['ip'], array_keys($ndpdata))) {
-                    $slease['online'] = 'online';
-                } else {
-                    $slease['online'] = 'offline';
-                }
-
-                $leases[] = $slease;
-            }
+    $duids = [];
+    foreach ($leases as $i => $this_lease) {
+        if (!empty($this_lease['duid'])) {
+            $duids[$this_lease['duid']] = $i;
         }
     }
 
-    if ($_GET['order']) {
-        usort($leases, function ($a, $b) {
-            return strcmp($a[$_GET['order']], $b[$_GET['order']]);
-        });
+    foreach (dhcpd_staticmap() as $static) {
+        if (!isset($static['ipaddrv6'])) {
+            continue;
+        }
+
+        $slease = [];
+        $slease['ip'] = $static['ipaddrv6'];
+        $slease['if'] = $static['interface'];
+        $slease['type'] = 'static';
+        $slease['duid'] = $static['duid'];
+        $slease['start'] = '';
+        $slease['end'] = '';
+        $slease['hostname'] = $static['hostname'];
+        $slease['descr'] = $static['descr'];
+        $slease['act'] = 'static';
+        $slease['online'] = in_array($slease['ip'], array_keys($ndpdata)) ? 'online' : 'offline';
+
+        if (isset($duids[$slease['duid']])) {
+            /* update lease with static data */
+            foreach ($slease as $key => $value) {
+                if (!empty($value)) {
+                    $leases[$duids[$slease['duid']]][$key] = $slease[$key];
+                }
+            }
+        } else {
+            $leases[] = $slease;
+	}
     }
+
+    if (isset($_GET['order']) && in_array($_GET['order'], ['int', 'ip', 'iaid', 'duid', 'hostname', 'descr', 'start', 'end', 'online', 'act'])) {
+        $order = $_GET['order'];
+    } else {
+        $order = 'ip';
+    }
+
+    usort($leases,
+        function ($a, $b) use ($order) {
+            $cmp = ($order === 'ip') ? 0 : strnatcasecmp($a[$order], $b[$order]);
+            if ($cmp === 0) {
+                $cmp = ipcmp($a['ip'], $b['ip']);
+            }
+            return $cmp;
+        }
+    );
 } elseif ($_SERVER['REQUEST_METHOD'] === 'POST') {
     if (!empty($_POST['deleteip']) && is_ipaddr($_POST['deleteip'])) {
         killbypid('/var/dhcpd/var/run/dhcpdv6.pid', 'TERM', true);
@@ -358,7 +381,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
             @unlink($leasesfile);
             @rename($leasesfile.".new", $leasesfile);
 
-            dhcpd_dhcp_configure(false, 'inet6');
+            dhcpd_dhcp6_configure();
         }
     }
     exit;
@@ -388,6 +411,11 @@ legacy_html_escape_form_data($leases);
               location.reload();
           });
       });
+      // keep sorting in place.
+      $(".act_sort").click(function(){
+          var all = <?=!empty($_GET['all']) ? 1 : 0;?> ;
+          document.location = document.location.origin + window.location.pathname +"?all="+all+"&order="+$(this).data('field');
+      });
   });
   </script>
 <?php include("fbegin.inc"); ?>
@@ -434,16 +462,16 @@ endif;?>
             <table class="table table-striped">
               <thead>
                 <tr>
-                    <th><?=gettext("Interface"); ?></th>
-                    <th><?=gettext("IPv6 address"); ?></th>
-                    <th><?=gettext("IAID"); ?></th>
-                    <th><?=gettext("DUID"); ?></th>
-                    <th><?=gettext("Hostname/MAC"); ?></th>
-                    <th><?=gettext("Description"); ?></th>
-                    <th><?=gettext("Start"); ?></th>
-                    <th><?=gettext("End"); ?></th>
-                    <th><?=gettext("Online"); ?></th>
-                    <th><?=gettext("Lease Type"); ?></th>
+                    <th class="act_sort" data-field="int"><?=gettext("Interface"); ?></th>
+                    <th class="act_sort" data-field="ip"><?=gettext("IPv6 address"); ?></th>
+                    <th class="act_sort" data-field="iaid"><?=gettext("IAID"); ?></th>
+                    <th class="act_sort" data-field="duid"><?=gettext("DUID/MAC"); ?></th>
+                    <th class="act_sort" data-field="hostname"><?=gettext("Hostname"); ?></th>
+                    <th class="act_sort" data-field="descr"><?=gettext("Description"); ?></th>
+                    <th class="act_sort" data-field="start"><?=gettext("Start"); ?></th>
+                    <th class="act_sort" data-field="end"><?=gettext("End"); ?></th>
+                    <th class="act_sort" data-field="online"><?=gettext("Online"); ?></th>
+                    <th class="act_sort" data-field="act"><?=gettext("Lease Type"); ?></th>
                     <th class="text-nowrap"></th>
                 </tr>
               </thead>
@@ -454,37 +482,44 @@ endif;?>
                 if (!($data['act'] == 'active' || $data['act'] == 'static' || $_GET['all'] == 1)) {
                     continue;
                 }
-                if ($data['act'] == "static") {
-                    foreach ($config['dhcpdv6'] as $dhcpif => $dhcpifconf) {
-                        if (isset($dhcpifconf['staticmap'])) {
-                            foreach ($dhcpifconf['staticmap'] as $staticent) {
-                                if ($data['ip'] == $staticent['ipaddr']) {
-                                    $data['int'] = htmlspecialchars($interfaces[$dhcpif]['descr']);
-                                    $data['if'] = $dhcpif;
-                                    break;
-                                }
-                            }
-                        }
-                        /* exit as soon as we have an interface */
-                        if ($data['if'] != "") {
-                            break;
-                        }
+                if (!isset($data['if'])) {
+                    $data['if'] = convert_real_interface_to_friendly_interface_name(guess_interface_from_ip($data['ip']));
+                }
+                $data['int'] = htmlspecialchars($interfaces[$data['if']]['descr']);
+
+                $mac_from_ndp = !empty($ndpdata[$data['ip']]) ? $ndpdata[$data['ip']]['mac'] : "";
+                $vendor_from_ndp = empty($mac_from_ndp) ? "" : ($mac_man[strtoupper(implode("", explode(":", substr($mac_from_ndp, 0, 8))))] ?? "");
+
+                $mac_from_duid = "";
+                $duid_formatted = $data['duid'];
+                $duid_type = substr($data['duid'], 0, 5);
+                if ($duid_type === "00:01" || $duid_type === "00:03"){
+                    $duid_subtype = substr($data['duid'], 6, 5);
+                    if ($duid_subtype === "00:01") {
+                        $mac_from_duid = substr($data['duid'], -17, 17);
+                        $duid_formatted = substr($data['duid'], 0, strlen($data['duid']) - 17) . '<u>' . $mac_from_duid . '</u>';
+                    }
+                }
+                $vendor_from_duid = empty($mac_from_duid) ? "" : ($mac_man[strtoupper(implode("", explode(":", substr($mac_from_duid, 0, 8))))] ?? "");
+
+                $duid_content = $duid_formatted;
+                if (!empty($vendor_from_duid)) {
+                    $duid_content .= '<br/><small><i>'.$vendor_from_duid.'</i></small>';
+                }
+                if (!empty($mac_from_ndp) && $mac_from_duid !== $mac_from_ndp) {
+                    $duid_content .= '</br>'.gettext('NDP MAC').': '.$mac_from_ndp;
+                    if (!empty($vendor_from_ndp)) {
+                        $duid_content .= '<br/><small><i>'.$vendor_from_ndp.'</i></small>';
                     }
-                } else {
-                  $data['if'] = convert_real_interface_to_friendly_interface_name(guess_interface_from_ip($data['ip']));
-                  $data['int'] = htmlspecialchars($interfaces[$data['if']]['descr']);
                 }
                 ?>
                 <tr>
                   <td><?=$data['int'];?></td>
                   <td><?=$data['ip'];?></td>
                   <td><?=$data['iaid'];?></td>
-                  <td><?=$data['duid'];?></td>
-                  <td>
-                    <?=!empty($data['hostname']) ? htmlentities($data['hostname']) : "";?>
-                    <?=!empty($ndpdata[$data['ip']]) ? $ndpdata[$data['ip']]['mac'] : "";?>
-                  </td>
-                  <td><?=htmlentities($data['descr']);?></td>
+                  <td><?=$duid_content;?></td>
+                  <td><?= !empty($data['hostname']) ? html_safe($data['hostname']) : '' ?></td>
+                  <td><?= html_safe($data['descr']);?></td>
                   <td><?=$data['type'] != "static" ? adjust_utc($data['start']) : "";?></td>
                   <td><?=$data['type'] != "static" ? adjust_utc($data['end']) : "";?></td>
                   <td>
