@@ -181,7 +181,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
             local_user_del($a_user[$id]);
             $userdeleted = $a_user[$id]['name'];
             unset($a_user[$id]);
-            write_config();
+            write_config(sprintf('The user "%s" was successfully removed.', $userdeleted));
             $savemsg = sprintf(gettext('The user "%s" was successfully removed.'), $userdeleted);
             header(url_safe('Location: /system_usermanager.php?savemsg=%s', array($savemsg)));
             exit;
@@ -191,7 +191,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         $certdeleted = lookup_cert($a_user[$id]['cert'][$pconfig['certid']]);
         $certdeleted = $certdeleted['descr'];
         unset($a_user[$id]['cert'][$pconfig['certid']]);
-        write_config();
+        write_config(sprintf('The certificate association "%s" was successfully removed.', $certdeleted));
         $savemsg = sprintf(gettext('The certificate association "%s" was successfully removed.'), $certdeleted);
         header(url_safe('Location: /system_usermanager.php?savemsg=%s&act=edit&userid=%d', array($savemsg, $id)));
         exit;
@@ -385,7 +385,12 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
 
             local_user_set_groups($userent, $pconfig['groups']);
             local_user_set($userent);
-            write_config();
+            if (isset($id)) {
+                $audit_msg = sprintf("user \"%s\" changed", $userent['name']);
+            } else {
+                $audit_msg = sprintf("user \"%s\" created", $userent['name']);
+            }
+            write_config($audit_msg);
             // XXX: signal backend that the user has changed.
             configdp_run('auth user changed', [$userent['name']]);
 
@@ -409,11 +414,6 @@ legacy_html_escape_form_data($a_user);
 
 include("head.inc");
 
-$main_buttons = array();
-if (!isset($_GET['act'])) {
-    $main_buttons[] = array('label' => gettext('Add'), 'href' => 'system_usermanager.php?act=new');
-}
-
 ?>
 <script src="<?= cache_safe('/ui/js/jquery.qrcode.js') ?>"></script>
 <script src="<?= cache_safe('/ui/js/qrcode.js') ?>"></script>
@@ -486,7 +486,7 @@ $( document ).ready(function() {
       event.preventDefault();
       const url="system_usermanager_import_ldap.php";
       var oWin = window.open(url,"OPNsense","width=620,height=400,top=150,left=150,scrollbars=yes");
-      if (oWin==null || typeof(oWin)=="undefined") {
+      if (oWin == null || typeof(oWin) == "undefined") {
         alert("<?= html_safe(gettext('Popup blocker detected. Action aborted.')) ?>");
       }
     });
@@ -508,9 +508,9 @@ $( document ).ready(function() {
 
                 $('#downloadFile').ready(function() {
                     $('#downloadFile').get(0).click();
+                    // reload form after download
+                    setTimeout(window.location.reload.bind(window.location), 100);
                 });
-                // reload form
-                location.reload();
             }
         },'json');
     });
@@ -600,7 +600,7 @@ $( document ).ready(function() {
                   <tr>
                     <td><i class="fa fa-info-circle text-muted"></i> <?=gettext("Username");?></td>
                     <td>
-                      <input name="usernamefld" type="text" class="formfld user" id="usernamefld" size="20" maxlength="32" value="<?=$pconfig['usernamefld'];?>" <?= $pconfig['scope'] == "system" || !empty($pconfig['user_dn']) ? "readonly=\"readonly\"" : "";?> />
+                      <input name="usernamefld" type="text" id="usernamefld" size="20" maxlength="32" value="<?=$pconfig['usernamefld'];?>" <?= $pconfig['scope'] == "system" || !empty($pconfig['user_dn']) ? "readonly=\"readonly\"" : "";?> />
                       <input name="oldusername" type="hidden" id="oldusername" value="<?=$pconfig['usernamefld'];?>" />
                     </td>
                   </tr>
@@ -609,7 +609,7 @@ $( document ).ready(function() {
                   <tr>
                     <td><i class="fa fa-info-circle text-muted"></i> <?=gettext("User distinguished name");?></td>
                     <td>
-                      <input name="user_dn" type="text" class="formfld user" id="user_dn" size="20" value="<?=$pconfig['user_dn'];?>" readonly="readonly" />
+                      <input name="user_dn" type="text" id="user_dn" size="20" value="<?=$pconfig['user_dn'];?>" readonly="readonly" />
                     </td>
                   </tr>
 <?php
@@ -617,8 +617,8 @@ $( document ).ready(function() {
                   <tr>
                     <td><i class="fa fa-info-circle text-muted"></i> <?=gettext("Password");?></td>
                     <td>
-                      <input name="passwordfld1" type="password" class="formfld pwd" id="passwordfld1" size="20" value="" /><br/>
-                      <input name="passwordfld2" type="password" class="formfld pwd" id="passwordfld2" size="20" value="" />
+                      <input name="passwordfld1" type="password" id="passwordfld1" size="20" value="" /><br/>
+                      <input name="passwordfld2" type="password" id="passwordfld2" size="20" value="" />
                       <small><?= gettext("(confirmation)"); ?></small><br/><br/>
                       <input type="checkbox" name="gen_new_password" <?= !empty($pconfig['gen_new_password']) ? 'checked="checked"' : '' ?>/>
                       <small><?=gettext('Generate a scrambled password to prevent local database logins for this user.') ?></small>
@@ -990,7 +990,32 @@ $( document ).ready(function() {
                       <th><?=gettext("Username"); ?></th>
                       <th><?=gettext("Full name"); ?></th>
                       <th><?=gettext("Groups"); ?></th>
-                      <th></th>
+                      <th class="text-nowrap">
+                        <a href="system_usermanager.php?act=new" class="btn btn-primary btn-xs" data-toggle="tooltip" title="<?= html_safe(gettext('Add')) ?>">
+                          <i class="fa fa-plus fa-fw"></i>
+                        </a>
+<?php
+                        $can_import = false;
+                        if (!empty($config['system']['webgui']['authmode'])) {
+                            $servers = explode(',', $config['system']['webgui']['authmode']);
+                            foreach ($servers as $server) {
+                                $authcfg_type = auth_get_authserver($server)['type'];
+                                if ($authcfg_type == 'ldap' || $authcfg_type == 'ldap-totp') {
+                                    $can_import = true;
+                                }
+                            }
+                        }
+?>
+<?php if ($can_import): ?>
+                        <button type="submit" name="import"
+                                id="import_ldap_users"
+                                data-toggle="tooltip"
+                                class="btn btn-primary btn-xs"
+                                title="<?= html_safe(gettext('Import')) ?>">
+                            <i class="fa fa-cloud-download fa-fw"></i>
+                        </button>
+<?php endif ?>
+                      </th>
                     </tr>
                   </thead>
                   <tbody>
@@ -1046,29 +1071,7 @@ $( document ).ready(function() {
                           </tr>
                         </table>
                       </td>
-                      <td class="text-nowrap">
-<?php
-                        $can_import = false;
-                        if (!empty($config['system']['webgui']['authmode'])) {
-                            $servers = explode(',', $config['system']['webgui']['authmode']);
-                            foreach ($servers as $server) {
-                                $authcfg_type = auth_get_authserver($server)['type'];
-                                if ($authcfg_type == 'ldap' || $authcfg_type == 'ldap-totp') {
-                                    $can_import = true;
-                                }
-                            }
-                        }
-?>
-<?php if ($can_import): ?>
-                          <button type="submit" name="import"
-                                  id="import_ldap_users"
-                                  data-toggle="tooltip"
-                                  class="btn btn-primary btn-xs"
-                                  title="<?= html_safe(gettext('Import')) ?>">
-                              <i class="fa fa-cloud-download fa-fw"></i>
-                          </button>
-<?php endif ?>
-                      </td>
+                      <td class="text-nowrap"></td>
                     </tr>
                   </tbody>
                 </table>
