@@ -28,9 +28,8 @@
  */
 
 require_once("guiconfig.inc");
-require_once("plugins.inc.d/openvpn.inc");
-require_once("services.inc");
 require_once("interfaces.inc");
+require_once("plugins.inc.d/openvpn.inc");
 
 $a_client = &config_read_array('openvpn', 'openvpn-client');
 
@@ -62,7 +61,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         $copy_fields = "auth_user,auth_pass,disable,mode,protocol,interface
             ,local_port,server_addr,server_port,resolve_retry,remote_random,reneg-sec
             ,proxy_addr,proxy_port,proxy_user,proxy_passwd,proxy_authtype,description
-            ,custom_options,ns_cert_type,dev_mode,caref,certref,crypto,digest,engine
+            ,custom_options,ns_cert_type,dev_mode,caref,certref,crypto,digest
             ,tunnel_network,tunnel_networkv6,remote_network,remote_networkv6,use_shaper
             ,compression,passtos,no_tun_ipv6,route_no_pull,route_no_exec,verbosity_level";
 
@@ -103,7 +102,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         $init_fields = "auth_user,auth_pass,disable,mode,protocol,interface
             ,local_port,server_addr,server_port,resolve_retry,remote_random,reneg-sec
             ,proxy_addr,proxy_port,proxy_user,proxy_passwd,proxy_authtype,description
-            ,custom_options,ns_cert_type,dev_mode,caref,certref,crypto,digest,engine
+            ,custom_options,ns_cert_type,dev_mode,caref,certref,crypto,digest
             ,tunnel_network,tunnel_networkv6,remote_network,remote_networkv6,use_shaper
             ,compression,passtos,no_tun_ipv6,route_no_pull,route_no_exec,verbosity_level";
 
@@ -294,6 +293,10 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         if (($pconfig['mode'] != "p2p_shared_key") && empty($pconfig['certref']) && empty($pconfig['auth_user']) && empty($pconfig['auth_pass'])) {
             $input_errors[] = gettext("If no Client Certificate is selected, a username and password must be entered.");
         }
+        $prev_opt = (isset($id) && !empty($a_client[$id])) ? $a_client[$id]['custom_options'] : "";
+        if ($prev_opt != str_replace("\r\n", "\n", $pconfig['custom_options']) && !userIsAdmin($_SESSION['Username'])) {
+            $input_errors[] = gettext('Advanced options may only be edited by system administrators due to the increased possibility of privilege escalation.');
+        }
 
         if (count($input_errors) == 0) {
             // save data
@@ -302,7 +305,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
             $copy_fields = "auth_user,auth_pass,protocol,dev_mode,local_port,reneg-sec
                 ,server_addr,server_port,resolve_retry,proxy_addr,proxy_port,remote_random
                 ,proxy_authtype,proxy_user,proxy_passwd,description,mode,crypto,digest
-                ,engine,tunnel_network,tunnel_networkv6,remote_network,remote_networkv6
+                ,tunnel_network,tunnel_networkv6,remote_network,remote_networkv6
                 ,use_shaper,compression,passtos,no_tun_ipv6,route_no_pull,route_no_exec
                 ,verbosity_level,interface";
 
@@ -362,14 +365,7 @@ legacy_html_escape_form_data($pconfig);
 
 include("head.inc");
 
-$main_buttons = array();
-
-if (empty($act)) {
-    $main_buttons[] = array('href' => 'vpn_openvpn_client.php?act=new', 'label' => gettext('Add'));
-}
-
 ?>
-
 <body>
 <?php include("fbegin.inc"); ?>
 <script>
@@ -563,7 +559,7 @@ $( document ).ready(function() {
               <td>
                 <input name="description" type="text" class="form-control unknown" size="30" value="<?=$pconfig['description'];?>" />
                 <div class="hidden" data-for="help_for_description">
-                  <small><?=gettext("You may enter a description here for your reference (not parsed)"); ?>.</small>
+                  <?=gettext("You may enter a description here for your reference (not parsed)."); ?>
                 </div>
               </td>
             </tr>
@@ -637,18 +633,6 @@ $( document ).ready(function() {
               foreach ($aliaslist as $aliasip => $aliasif) {
                   $interfaces[$aliasif.'|'.$aliasip] = $aliasip." (".get_vip_descr($aliasip).")";
               }
-              $grouplist = return_gateway_groups_array();
-              foreach ($grouplist as $name => $group) {
-                  if ($group['ipprotocol'] != "inet") {
-                      continue;
-                  }
-                  if ($group[0]['vip'] != '') {
-                      $vipif = $group[0]['vip'];
-                  } else {
-                      $vipif = $group[0]['int'];
-                  }
-                  $interfaces[$name] = "GW Group {$name}";
-              }
               $interfaces['lo0'] = "Localhost";
               $interfaces['any'] = "any";
               foreach ($interfaces as $iface => $ifacename) :
@@ -942,23 +926,6 @@ $( document ).ready(function() {
               </div>
             </td>
           </tr>
-          <tr id="engine">
-            <td><i class="fa fa-info-circle text-muted"></i> <?=gettext("Hardware Crypto"); ?></td>
-            <td>
-              <select name="engine" class="form-control">
-<?php
-              $engines = openvpn_get_engines();
-              foreach ($engines as $name => $desc) :
-                  $selected = "";
-                  if ($name == $pconfig['engine']) {
-                      $selected = " selected=\"selected\"";
-                  }?>
-                <option value="<?=$name;?>"<?=$selected?>><?=htmlspecialchars($desc);?></option>
-<?php
-              endforeach; ?>
-              </select>
-            </td>
-          </tr>
          </table>
         </div>
        </div>
@@ -1058,7 +1025,7 @@ $( document ).ready(function() {
                                 endforeach; ?>
               </select>
               <div class="hidden" data-for="help_for_compression">
-                <?=gettext("Compress tunnel packets using the LZO algorithm. Adaptive compression will dynamically disable compression for a period of time if OpenVPN detects that the data in the packets is not being compressed efficiently."); ?>
+                <?=gettext("Compress tunnel packets using the LZ4/LZO algorithm. The LZ4 generally offers the best preformance with least CPU usage. For backwards compatibility use the LZO (which is identical to the older option --comp-lzo yes). In the partial mode (the option --compress with an empty algorithm) compression is turned off, but the packet framing for compression is still enabled, allowing a different setting to be pushed later. The legacy LZO algorithm with adaptive compression mode will dynamically disable compression for a period of time if OpenVPN detects that the data in the packets is not being compressed efficiently."); ?>
               </div>
             </td>
           </tr>
@@ -1085,7 +1052,7 @@ $( document ).ready(function() {
             <td>
               <input name="route_no_pull" type="checkbox" value="yes" <?=!empty($pconfig['route_no_pull']) ? "checked=\"checked\"" : "" ;?> />
               <div class="hidden" data-for="help_for_route_no_pull">
-                <?=sprintf(gettext("Don't add or remove routes automatically. Instead pass routes to %s--route-up%s script using environmental variables"),'<strong>','</strong>') ?>.
+                <?=gettext("This option effectively bars the server from adding routes to the client's routing table, however note that this option still allows the server to set the TCP/IP properties of the client's TUN/TAP interface"); ?>.
               </div>
             </td>
           </tr>
@@ -1094,7 +1061,7 @@ $( document ).ready(function() {
             <td>
               <input name="route_no_exec" type="checkbox" value="yes" <?=!empty($pconfig['route_no_exec']) ? "checked=\"checked\"" : "" ;?> />
               <div class="hidden" data-for="help_for_route_no_exec">
-                <?=gettext("This option effectively bars the server from adding routes to the client's routing table, however note that this option still allows the server to set the TCP/IP properties of the client's TUN/TAP interface"); ?>.
+                <?= gettext('Do not add or remove routes automatically. Instead pass routes to "--route-up" script using environmental variables.') ?>
               </div>
             </td>
           </tr>
@@ -1113,6 +1080,7 @@ $( document ).ready(function() {
             <td style="width:22%"><a id="help_for_custom_options" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext("Advanced"); ?></td>
             <td style="width:78%">
               <textarea rows="6" cols="78" name="custom_options" id="custom_options"><?=$pconfig['custom_options'];?></textarea>
+              <?=gettext("This option will be removed in the future due to being insecure by nature. In the mean time only full administrators are allowed to change this setting.");?>
               <div class="hidden" data-for="help_for_custom_options">
                 <?=gettext("Enter any additional options you would like to add to the configuration file here."); ?>
               </div>
@@ -1180,7 +1148,17 @@ $( document ).ready(function() {
                 <td><?=gettext("Protocol"); ?></td>
                 <td><?=gettext("Server"); ?></td>
                 <td><?=gettext("Description"); ?></td>
-                <td class="text-nowrap"></td>
+                <td class="text-nowrap">
+                  <a href="vpn_openvpn_client.php?act=new" class="btn btn-primary btn-xs" data-toggle="tooltip" title="<?= html_safe(gettext('Add')) ?>">
+                    <i class="fa fa-plus fa-fw"></i>
+                  </a>
+                  <a data-id="<?= count($a_client) ?>" data-toggle="tooltip" title="<?=gettext("Move selected items to end");?>" class="act_move btn btn-default btn-xs">
+                    <span class="fa fa-arrow-down fa-fw"></span>
+                  </a>
+                  <a data-id="x" title="<?=gettext("delete selected rules"); ?>" data-toggle="tooltip"  class="act_delete btn btn-default btn-xs">
+                    <span class="fa fa-trash-o fa-fw"></span>
+                  </a>
+                </td>
               </tr>
             </thead>
             <tbody>
@@ -1212,7 +1190,7 @@ $( document ).ready(function() {
                       <span class="fa fa-pencil fa-fw"></span>
                     </a>
                     <a data-id="<?=$i;?>" title="<?=gettext("delete client"); ?>" class="act_delete btn btn-default btn-xs">
-                      <span class="fa fa-trash fa-fw"></span>
+                      <span class="fa fa-trash-o fa-fw"></span>
                     </a>
                     <a href="vpn_openvpn_client.php?act=new&amp;dup=<?=$i;?>" class="btn btn-default btn-xs" data-toggle="tooltip" title="<?=gettext("clone client");?>">
                       <span class="fa fa-clone fa-fw"></span>
@@ -1222,17 +1200,6 @@ $( document ).ready(function() {
 <?php
               $i++;
               endforeach;?>
-              <tr>
-                <td colspan="4"></td>
-                <td class="text-nowrap">
-                  <a data-id="<?=$i;?>" data-toggle="tooltip" title="<?=gettext("Move selected items to end");?>" class="act_move btn btn-default btn-xs">
-                    <span class="fa fa-arrow-down fa-fw"></span>
-                  </a>
-                  <a data-id="x" title="<?=gettext("delete selected rules"); ?>" data-toggle="tooltip"  class="act_delete btn btn-default btn-xs">
-                    <span class="fa fa-trash fa-fw"></span>
-                  </a>
-                </td>
-              </tr>
             </tbody>
           </table>
           </div>
