@@ -58,7 +58,7 @@ if (is_numeric($pconfig['filterlogentries'])) {
     }
 
     if (!empty($pconfig['filterlogentriesinterfaces'])) {
-        $config['widgets']['filterlogentriesinterfaces'] = $pconfig['filterlogentriesinterfaces'];
+        $config['widgets']['filterlogentriesinterfaces'] = implode(',', $pconfig['filterlogentriesinterfaces']);
     } elseif (isset($config['widgets']['filterlogentriesinterfaces'])) {
         unset($config['widgets']['filterlogentriesinterfaces']);
     }
@@ -75,27 +75,34 @@ $nentriesinterfaces = isset($config['widgets']['filterlogentriesinterfaces']) ?
 
 ?>
 <script>
-    $(window).on("load", function() {
+    $("#dashboard_container").on("WidgetsReady", function() {
         // needed to display the widget settings menu
         $("#log-configure").removeClass("disabled");
         // icons
         const field_type_icons = {'pass': 'fa-play', 'block': 'fa-ban', 'rdr': 'fa-exchange', 'nat': 'fa-exchange'};
 
         var interface_descriptions = {};
+        var nentriesinterfaces =  "<?= $nentriesinterfaces ?>".split(",");
         ajaxGet('/api/diagnostics/interface/getInterfaceNames', {}, function(data, status) {
             interface_descriptions = data;
+            $.each(interface_descriptions, function(i_d, i_name){
+                $('#filterlogentriesinterfaces').append($.inArray(i_d, nentriesinterfaces) > -1 ? $('<option>', {value:i_d, text:i_name, selected: "selected" }) : $('<option>', {value:i_d, text:i_name }));
+            });
+            $('#filterlogentriesinterfaces').selectpicker('refresh');
+            fetch_log();
         });
         function fetch_log(){
             var record_spec = [];
             // read heading, contains field specs
-            $("#filter-log-entries > tbody > tr:first > td").each(function () {
+            $("#filter-log-entries > thead > tr > th").each(function () {
                 record_spec.push({
                     'column-id': $(this).data('column-id'),
                     'type': $(this).data('type'),
                     'class': $(this).attr('class')
                 });
             });
-            ajaxGet('/api/diagnostics/firewall/log/', {'limit': 100}, function(data, status) {
+            var last_digest = $("#filter-log-entries > tbody > tr:first > td:first").text();
+            ajaxGet('/api/diagnostics/firewall/log/', {'digest': last_digest, 'limit': 100}, function(data, status) {
                 var filtact = [];
 
                 if ($("#actpass").is(':checked')) {
@@ -106,15 +113,11 @@ $nentriesinterfaces = isset($config['widgets']['filterlogentriesinterfaces']) ?
                 }
 
                 let record;
+                let showinterfaces = $("#filterlogentriesinterfaces").val();
                 while ((record=data.pop()) != null) {
                     var intf = record['interface'];
-
-                    if (interface_descriptions[record['interface']] != undefined) {
-                        intf = interface_descriptions[record['interface']].toLowerCase();
-                    }
-
-                    if ($("#filterlogentriesinterfaces").val() == "" || $("#filterlogentriesinterfaces").val() == intf) {
-                        if (filtact.length == 0 || filtact.indexOf(record['action']) !== -1 ) {
+                    if (showinterfaces.length == 0 || $.inArray(intf, showinterfaces) > -1) {
+                        if ((filtact.length == 0 || filtact.indexOf(record['action']) !== -1) && record['__digest__'] != last_digest) {
                             var log_tr = $("<tr>");
                             log_tr.hide();
                             $.each(record_spec, function(idx, field){
@@ -125,13 +128,17 @@ $nentriesinterfaces = isset($config['widgets']['filterlogentriesinterfaces']) ?
                                     case 'icon':
                                         var icon = field_type_icons[record[column_name]];
                                         if (icon != undefined) {
-                                            log_td.html('<i class="fa '+icon+'" aria-hidden="true"></i>');
+                                            // prepare popover content
+                                            let popContent = "@" + record.rulenr;
+                                            popContent += record.label.length > 0 ? " Label: " + record.label : '';
+                                            popContent += "<br><sub><?= gettext('click the Act icon to track this rule in Live View') ?></sub>"
+                                            log_td.html('<a target="_blank" href="/ui/diagnostics/firewall/log?rid=' + record.rid + '" type="button" data-toggle="popover" data-trigger="hover" \
+                                                        data-html="true" data-title="<?= gettext('Matched rule') ?>" data-content="' + popContent + '"><i class="fa ' + icon + '" aria-hidden="true"></i></a>');
                                             if (record[column_name] == 'pass') {
-                                                log_td.addClass('text-success');
+                                                log_td.find('a').addClass('text-success');
                                             } else {
-                                                log_td.addClass('text-danger');
+                                                log_td.find('a').addClass('text-danger');
                                             }
-
                                         }
                                         break;
                                     case 'time':
@@ -152,6 +159,9 @@ $nentriesinterfaces = isset($config['widgets']['filterlogentriesinterfaces']) ?
                                         // may support ports, but needs IPv6 fixup
                                         log_td.text(record[column_name]);
                                         break;
+                                    case 'destination_port':
+                                        log_td.text(record[column_name]);
+                                        break;
                                     default:
                                         if (record[column_name] != undefined) {
                                             log_td.text(record[column_name]);
@@ -159,12 +169,20 @@ $nentriesinterfaces = isset($config['widgets']['filterlogentriesinterfaces']) ?
                                 }
                                 log_tr.append(log_td);
                             });
-                            $("#filter-log-entries > tbody > tr:first").after(log_tr);
+                            $("#filter-log-entries > tbody > tr:first").before(log_tr);
                         }
                     }
                 }
-                $("#filter-log-entries > tbody > tr:gt("+(parseInt($("#filterlogentries").val()))+")").remove();
+                //hide popover before elem remove
+                $('[data-toggle="popover"]').popover('hide');
+                $("#filter-log-entries > tbody > tr:gt("+(parseInt($("#filterlogentries").val() - 1))+")").remove();
                 $("#filter-log-entries > tbody > tr").show();
+                //enable popover with full width for long descriptions
+                $('[data-toggle="popover"]').popover({
+                    container: 'body'
+                }).on('show.bs.popover', function() {
+                    $(this).data("bs.popover").tip().css("max-width", "100%")
+                });
             });
 
             // schedule next fetch
@@ -173,7 +191,6 @@ $nentriesinterfaces = isset($config['widgets']['filterlogentriesinterfaces']) ?
             setTimeout(fetch_log, update_interval_ms);
         }
 
-        fetch_log();
     });
 </script>
 
@@ -195,20 +212,14 @@ $nentriesinterfaces = isset($config['widgets']['filterlogentriesinterfaces']) ?
 <?php endfor ?>
           </select><br/>
           <label for="filterlogentriesinterfaces"><?= gettext('Interfaces to display:'); ?></label><br/>
-          <select id="filterlogentriesinterfaces" name="filterlogentriesinterfaces" class="selectpicker_widget">
-            <option value=""><?= gettext('All') ?></option>
-<?php foreach (get_configured_interface_with_descr() as $iface => $ifacename): ?>
-            <option value="<?= html_safe($iface) ?>" <?= $nentriesinterfaces == $iface ? 'selected="selected"' : '' ?>>
-              <?= html_safe($ifacename) ?>
-            </option>
-<?php endforeach ?>
+          <select id="filterlogentriesinterfaces" name="filterlogentriesinterfaces[]" class="selectpicker_widget" multiple="multiple" title=<?= gettext('All'); ?>>
           </select><br/><br/>
           <table style="width:348px">
             <tr>
               <td><label for="actblock"><input id="actblock" name="actblock" type="checkbox" value="Block" <?=in_array('Block', $nentriesacts) ? "checked=\"checked\"" : "";?> />Block</label></td>
               <td><label for="actreject"><input id="actreject" name="actreject" type="checkbox" value="Reject" <?=in_array('Reject', $nentriesacts) ? "checked=\"checked\"" : "";?> />Reject</label></td>
-              <td><button name="submit_firewall_logs_widget" type="submit" class="btn btn-primary" value="yes"><?= gettext('Save') ?></button></td>
               <td><label for="actpass"><input id="actpass" name="actpass" type="checkbox" value="Pass" <?=in_array('Pass', $nentriesacts) ? "checked=\"checked\"" : "";?> />Pass</label></td>
+              <td><button name="submit_firewall_logs_widget" type="submit" class="btn btn-primary" value="yes"><?= gettext('Save') ?></button></td>
             </tr>
           </table>
         </td>
@@ -218,13 +229,18 @@ $nentriesinterfaces = isset($config['widgets']['filterlogentriesinterfaces']) ?
 </div>
 
 <table class="table table-striped table-condensed" id="filter-log-entries">
-  <tbody>
+  <thead>
     <tr>
-      <td data-column-id="action" data-type="icon" class="text-center"><strong><?= gettext('Act') ?></strong></td>
-      <td data-column-id="__timestamp__" data-type="time"><strong><?= gettext('Time') ?></strong></td>
-      <td data-column-id="interface" data-type="interface" class="text-center"><strong><?= gettext('Interface') ?></strong></td>
-      <td data-column-id="src" data-type="source_address"><strong><?= gettext('Source') ?></strong></td>
-      <td data-column-id="dst" data-type="destination_address"><strong><?= gettext('Destination') ?></strong></td>
+      <th data-column-id="__digest__" data-type="string" class="hidden"><?= gettext('Hash') ?></th>
+      <th data-column-id="action" data-type="icon" class="text-center"><?= gettext('Act') ?></th>
+      <th data-column-id="__timestamp__" data-type="time"><?= gettext('Time') ?></th>
+      <th data-column-id="interface" data-type="interface" class="text-center"><?= gettext('Interface') ?></th>
+      <th data-column-id="src" data-type="source_address"><?= gettext('Source') ?></th>
+      <th data-column-id="dst" data-type="destination_address"><?= gettext('Destination') ?></th>
+      <th data-column-id="dstport" data-type="destination_port"><?= gettext('Dest Port') ?></th>
     </tr>
+  </thead>
+  <tbody>
+    <tr></tr>
   </tbody>
 </table>
